/*

  $Id: Auth4cAD.h 689 2018-11-28 17:54:35Z john $

  Copyright (c) 2008, The 4c Company. All rights reserved.
*/

#ifndef __Auth4cAD_Auth4cAD_h__
#define __Auth4cAD_Auth4cAD_h__

#if defined (Auth4cAD_Export)
#	undef Auth4cAD_Export
#endif /* Auth4cAD_Export*/

#if defined (_WINDOWS)
#	include <windows.h>
#	if defined (AUTH4CAD_EXPORTS)
#		define Auth4cAD_Export __declspec( dllexport )
#	else
#		define Auth4cAD_Export __declspec( dllimport )
#	endif /* AUTH4CAD_EXPORTS */
#	pragma warning( disable : 4355 4251 )
#else
#	define Auth4cAD_Export
#endif /* _WINDOWS */

#include <Auth4cData/AuthenticationLibrary.h>

#if defined (LINUX)
#	include <ldap.h>
#elif defined (_WINDOWS)
#	include <winldap.h>
#endif /* LINUX */

namespace Auth4cAD
{
	class Authenticator : public Auth4cData::Authenticator
	{
		private:
			std::string _strHost;
			std::string _strBaseDN;
			std::string _strSearchUser;
			std::string _strSearchPassword;
			std::string _strSearchTemplate;

		public:
			Authenticator( const Auth4cData::InitMap& initAuthenticator,
							const Auth4cData::Domain& domain = Auth4cData::Domain(),
							const std::string& strDescription = std::string(),
							int nAuthenticationLibrary = 0 );
			Authenticator( const Authenticator& authenticator );
			virtual ~Authenticator();

			const Authenticator& operator=( const Authenticator& authenticator );

			virtual Auth4cData::AuthenticatedUser authenticate( const Auth4cData::User& user, const Auth4cData::AuthenticationToken& authToken, const Auth4cData::Profile& profile );
			virtual void user( Auth4cData::User& user, bool bRecurse = true );
			virtual void team( Auth4cData::Team& team );
			virtual void findUsers( Auth4cData::UserMap& userS, const std::string strSearch );

//			virtual bool passwd( const Auth4cData::AuthenticatedUser& user, Auth4cData::AuthenticationToken& authTokenCurrent, Auth4cData::AuthenticationToken& authTokenNew );

		protected:
			LDAP* connectLDAP();
			void disconnectLDAP( LDAP* pld );
			Auth4cData::AuthenticatedUser authenticate( const Auth4cData::User& user, const Auth4cData::AuthenticationToken& authToken, const Auth4cData::Profile& profile, LDAP* pld );
			void user( Auth4cData::User& user, LDAP* pld, bool bTeams = true );
			void team( Auth4cData::Team& team, LDAP* pld );
			bool authenticate( const std::string& strDN, const Auth4cData::AuthenticationToken& authToken, LDAP* pld );
			void user( Auth4cData::User& user, const std::string& strSearch, LDAP* pld, bool bTeams = true );
			std::vector<std::string> groupMembers( const std::string& strGroup, LDAP* pld );
	};

	class AuthenticationLibrary : public Auth4cData::AuthenticationLibrary
	{
		public:
			AuthenticationLibrary();
			AuthenticationLibrary( const Auth4cData::AuthenticationLibrary* ptemplate );
			AuthenticationLibrary( const AuthenticationLibrary& library );
			virtual ~AuthenticationLibrary();

			const AuthenticationLibrary& operator=( const AuthenticationLibrary& library );

			virtual Auth4cData::Authenticator* authenticator( const Auth4cData::Domain& domain,
																const std::string& strDescription,
																const Auth4cData::InitMap& initMap );
	};
};

#endif /* !__Auth4cAD_Auth4cAD_h__ */
