/*

  $Id: Auth4cAuth4c.h 689 2018-11-28 17:54:35Z john $

  Copyright (c) 2008, The 4c Company. All rights reserved.
*/

#ifndef __Auth4cAuth4c_Auth4cAuth4c_h__
#define __Auth4cAuth4c_Auth4cAuth4c_h__

#if defined (Auth4cAuth4c_Export)
#	undef Auth4cAuth4c_Export
#endif /* Auth4cAuth4c_Export*/

#if defined (_WINDOWS)
#	include <windows.h>
#	if defined (AUTH4CAUTH4C_EXPORTS)
#		define Auth4cAuth4c_Export __declspec( dllexport )
#	else
#		define Auth4cAuth4c_Export __declspec( dllimport )
#	endif /* AUTH4CAUTH4C_EXPORTS */
#	pragma warning( disable : 4355 4251 )
#else
#	define Auth4cAuth4c_Export
#endif /* _WINDOWS */

#include <Auth4cData/AuthenticationLibrary.h>

#include <librfcdal/DalMySQLDatabase.h>

namespace Auth4cAuth4c
{
	class Auth4cAuth4c_Export Authenticator : public Auth4cData::Authenticator
	{
		private:
//			RfcDal::DalMySQLDatabase _db;

		public:
			Authenticator( const Auth4cData::InitMap& initAuthenticator,
							const Auth4cData::Domain& domain = Auth4cData::Domain(),
							const std::string& strDescription = std::string(),
							int nAuthenticationLibrary = 0 );
			Authenticator( const Authenticator& authenticator );
			virtual ~Authenticator();

			const Authenticator& operator=( const Authenticator& authenticator );

			virtual Auth4cData::AuthenticatedUser authenticate( const Auth4cData::User& user, const Auth4cData::AuthenticationToken& authToken, const Auth4cData::Profile& profile );
			virtual void user( Auth4cData::User& user, bool bRecurse = true );
			virtual void team( Auth4cData::Team& team );
			virtual void findUsers( Auth4cData::UserMap& userS, const std::string strSearch );

			virtual void users( Auth4cData::UserMap& userS );
			virtual void passwd( const Auth4cData::AuthenticatedUser& user, Auth4cData::AuthenticationToken& authTokenCurrent, Auth4cData::AuthenticationToken& authTokenNew );
			virtual void createUser( const Auth4cData::User& user );
			virtual void updateUser( const Auth4cData::User& user );
			virtual void deleteUser( const Auth4cData::User& user );
			virtual void createTeamMember( Auth4cData::Team& team, Auth4cData::User& user );
			virtual void deleteTeamMember( Auth4cData::Team& team, Auth4cData::User& user );

		protected:
			Auth4cData::AuthenticatedUser authenticate( const Auth4cData::User& user, const Auth4cData::AuthenticationToken& authToken, const Auth4cData::Profile& profile, RfcDal::DalDatabase& db );
			void user( Auth4cData::User& user, RfcDal::DalDatabase& db, bool bRecurse = true );
			void team( Auth4cData::Team& team, RfcDal::DalDatabase& db );
			void findUsers( Auth4cData::UserMap& userS, const std::string strSearch, RfcDal::DalDatabase& db );

			void init( RfcDal::DalDatabase& db );
			void user( Auth4cData::User& user, const std::string& strSearch, RfcDal::DalDatabase& db );
			void incrementBadLogins( const Auth4cData::User& user, RfcDal::DalDatabase& db  );
			void resetBadLogins( const Auth4cData::User& user, RfcDal::DalDatabase& db  );
	};

	class Auth4cAuth4c_Export AuthenticationLibrary : public Auth4cData::AuthenticationLibrary
	{
		public:
			AuthenticationLibrary();
			AuthenticationLibrary( const Auth4cData::AuthenticationLibrary* ptemplate );
			AuthenticationLibrary( const AuthenticationLibrary& library );
			virtual ~AuthenticationLibrary();

			const AuthenticationLibrary& operator=( const AuthenticationLibrary& library );

			virtual Auth4cData::Authenticator* authenticator( const Auth4cData::Domain& domain,
																const std::string& strDescription,
																const Auth4cData::InitMap& initMap );
	};
};

#endif /* !__Auth4cAuth4c_Auth4cAuth4c_h__ */
