#include <Auth4cCData/Auth4cFactory.h>

#include <Auth4cDataXML/Auth4cFactory.h>

#include <Auth4cData/Factory.h>
#include <Auth4cData/PasswordAuthenticationToken.h>

#include <Base4c/EventLogger.h>

#include <librfc/XMLDoc.hpp>

#include <iostream>
#include <utility>
#include <map>
#include <string>

#include <string.h>

std::string strAuth4cCData( "Auth4cCData" );
Base4c::EventLogger g_logger( strAuth4cCData );
std::map<std::pair<std::string, std::string>, Auth4cData::AuthenticatedUser> g_userS;

extern "C" Auth4cCData_Exports int auth4c_init( const char* pszInit )
{
	try
	{
		Auth4cDataXML::Factory::initialize( pszInit );

		return 0;
	}
	catch( const Auth4cData::Exception& e )
	{
		std::cerr << e << "\n";
		return e.errorCode();
	}
}

extern "C" Auth4cCData_Exports USER auth4c_authenticate( const char* pszUserId, const char* pszPassword, const char* pszProfile )
{
	try
	{
		Auth4cData::Factory::domains() = Auth4cDataXML::Factory::domains();

		Auth4cData::User user( Auth4cData::User::fromText( pszUserId ) );

		Auth4cData::PasswordAuthenticationToken token( pszPassword );

		Auth4cData::Profile profile( pszProfile );

		Auth4cData::AuthenticatedUser userA = Auth4cDataXML::Factory::authenticate( user, token, profile );

		std::pair<std::string, std::string> userId = std::make_pair( userA.domain().domain(), userA.userId() );
		g_userS[userId] = userA;

		Auth4cData::AuthenticatedUser& ruserA = g_userS[userId];

		return &ruserA;
	}
	catch( const Auth4cData::Exception& e )
	{
		std::cerr << e << "\n";
		return NULL;
	}
}

extern "C" Auth4cCData_Exports USER auth4c_authenticateD( const char* pszDomain, const char* pszUserId, const char* pszPassword, const char* pszProfile )
{
	try
	{
		Auth4cData::Domain domain( Auth4cData::Customer::allCustomers, pszDomain );
		Auth4cData::User user( domain, pszUserId );

		Auth4cData::PasswordAuthenticationToken token( pszPassword );

		Auth4cData::Profile profile( pszProfile );

		Auth4cData::AuthenticatedUser userA = Auth4cDataXML::Factory::authenticate( user, token, profile );

		std::pair<std::string, std::string> userId = std::make_pair( userA.domain().domain(), userA.userId() );
		g_userS[userId] = userA;

		Auth4cData::AuthenticatedUser& ruserA = g_userS[userId];

		return &ruserA;
	}
	catch( const Auth4cData::Exception& e )
	{
		std::cerr << e << "\n";
		return NULL;
	}
}

extern "C" Auth4cCData_Exports int auth4c_persistSession( USER user )
{
	Auth4cData::AuthenticatedUser* puser = (Auth4cData::AuthenticatedUser*)user;

	try
	{
		Auth4cDataXML::Factory::persistSession( *puser );

		return 0;
	}
	catch( const Auth4cData::Exception& e )
	{
		std::cerr << e << "\n";
		return -1;
	}
}

extern "C" Auth4cCData_Exports USER auth4c_retrieveSession( const char* pszSession )
{
	try
	{
		Auth4cData::SessionString session( pszSession );

		Auth4cData::AuthenticatedUser userA = Auth4cDataXML::Factory::retrieveSession( session );

		std::pair<std::string, std::string> userId = std::make_pair( userA.domain().domain(), userA.userId() );
		g_userS[userId] = userA;

		Auth4cData::AuthenticatedUser& ruserA = g_userS[userId];

		return &ruserA;
	}
	catch( const Auth4cData::Exception& e )
	{
		std::cerr << e << "\n";
		return NULL;
	}
}

extern "C" Auth4cCData_Exports int auth4c_killSession( const char* pszSession )
{
	try
	{
		Auth4cData::SessionString session( pszSession );

		Auth4cDataXML::Factory::killSession( session );

		return 0;
	}
	catch( const Auth4cData::Exception& e )
	{
		std::cerr << e << "\n";
		return e.errorCode();
	}
}

extern "C" Auth4cCData_Exports int auth4c_session( char* pszSession, int nMaxLen, USER user )
{
	Auth4cData::AuthenticatedUser* puser = (Auth4cData::AuthenticatedUser*)user;

	try
	{
		strncpy( pszSession, puser->session().str().c_str(), nMaxLen );
		return strlen( pszSession );
	}
	catch( const Auth4cData::Exception& e )
	{
		std::cerr << e << "\n";
		return -1;
	}
}

extern "C" Auth4cCData_Exports int auth4c_customer( char* pszCustomer, int nMaxLen, USER user )
{
	Auth4cData::AuthenticatedUser* puser = (Auth4cData::AuthenticatedUser*)user;

	try
	{
		strncpy( pszCustomer, puser->domain().customer().customer().c_str(), nMaxLen );
		return strlen( pszCustomer );
	}
	catch( const Auth4cData::Exception& e )
	{
		std::cerr << e << "\n";
		return -1;
	}
}

extern "C" Auth4cCData_Exports int auth4c_domain( char* pszDomain, int nMaxLen, USER user )
{
	Auth4cData::AuthenticatedUser* puser = (Auth4cData::AuthenticatedUser*)user;

	try
	{
		strncpy( pszDomain, puser->domain().domain().c_str(), nMaxLen );
		return strlen( pszDomain );
	}
	catch( const Auth4cData::Exception& e )
	{
		std::cerr << e << "\n";
		return -1;
	}
}

extern "C" Auth4cCData_Exports int auth4c_userId( char* pszId, int nMaxLen, USER user )
{
	Auth4cData::AuthenticatedUser* puser = (Auth4cData::AuthenticatedUser*)user;

	try
	{
		strncpy( pszId, puser->userId().c_str(), nMaxLen );
		return strlen( pszId );
	}
	catch( const Auth4cData::Exception& e )
	{
		std::cerr << e << "\n";
		return -1;
	}
}

extern "C" Auth4cCData_Exports int auth4c_userName( char* pszName, int nMaxLen, USER user )
{
	Auth4cData::AuthenticatedUser* puser = (Auth4cData::AuthenticatedUser*)user;

	try
	{
		strncpy( pszName, puser->name().c_str(), nMaxLen );
		return strlen( pszName );
	}
	catch( const Auth4cData::Exception& e )
	{
		std::cerr << e << "\n";
		return -1;
	}
}

extern "C" Auth4cCData_Exports int auth4c_userContact( char* pszContact, int nMaxLen, USER user )
{
	Auth4cData::AuthenticatedUser* puser = (Auth4cData::AuthenticatedUser*)user;

	try
	{
		strncpy( pszContact, puser->contact().c_str(), nMaxLen );
		return strlen( pszContact );
	}
	catch( const Auth4cData::Exception& e )
	{
		std::cerr << e << "\n";
		return -1;
	}
}

extern "C" Auth4cCData_Exports int auth4c_isMemberOf( int nTeam, USER user )
{
	Auth4cData::AuthenticatedUser* puser = (Auth4cData::AuthenticatedUser*)user;

	try
	{
		Auth4cData::Team team( puser->domain().customer(), nTeam );

		return puser->isMemberOf( team ) ? 1 : 0;
	}
	catch( const Auth4cData::Exception& e )
	{
		std::cerr << e << "\n";
		return 0;
	}
}

extern "C" Auth4cCData_Exports int auth4c_hasRole( int nProduct, int nRole, USER user )
{
	Auth4cData::AuthenticatedUser* puser = (Auth4cData::AuthenticatedUser*)user;

	try
	{
		Auth4cData::Product product( nProduct );
		Auth4cData::Role role( product, nRole );
		return puser->hasRole( role ) ? 1 : 0;
	}
	catch( const Auth4cData::Exception& e )
	{
		std::cerr << e << "\n";
		return 0;
	}
}

extern "C" Auth4cCData_Exports int auth4c_hasPermission( int nProduct, int nPermission, USER user )
{
	Auth4cData::AuthenticatedUser* puser = (Auth4cData::AuthenticatedUser*)user;

	try
	{
		Auth4cData::Product product( nProduct );
		Auth4cData::Permission permission( product, nPermission );
		return puser->hasPermission( permission ) ? 1 : 0;
	}
	catch( const Auth4cData::Exception& e )
	{
		std::cerr << e << "\n";
		return 0;
	}
}

extern "C" Auth4cCData_Exports void auth4c_close( USER user )
{
	Auth4cData::AuthenticatedUser* puser = (Auth4cData::AuthenticatedUser*)user;

	std::cerr << "Deleting user\n";

	std::pair<std::string, std::string> userId = std::make_pair( puser->domain().domain(), puser->userId() );
	std::map<std::pair<std::string, std::string>, Auth4cData::AuthenticatedUser>::iterator iter = g_userS.find( userId );
	if( g_userS.end() != iter )
	{
		g_userS.erase( iter );
	}
}
