/*

  $Id: Settings.cpp 690 2018-11-28 20:58:27Z john $

  Copyright (c) 2008, The 4c Company. All rights reserved.
*/

#include <Auth4cClientConfig/Settings.h>

#include <Auth4cDataXML/Auth4cFactory.h>

#if defined (LINUX)
#	include <unistd.h>
#	include <fcntl.h>
#	include <sys/stat.h>
#elif defined (_WINDOWS)
#	include <winerror.h>
#endif /* LINUX */

#include <sstream>

#include <stdlib.h>

#if defined (LINUX)
const char szROOT4C[] = "company4c";
#elif defined (_WINDOWS)
const char szROOT4C[] = "Software\\company4c";
#endif /* LINUX */
const char szAUTH4C[] = "Auth4c";
const char szDOMAIN[] = "Domain";
const char szDOMAINS[] = "Domains";
const char szPROFILE[] = "Profile";
const char szPROFILES[] = "Profiles";
const char szUSERID[] = "UserId";
const char szNAME[] = "Name";
const char szISDEFAULT[] = "IsDefault";
const char szCONNECT[] = "Connect";
const char szMASTER[] = "Master";

using namespace Auth4cClientConfig;

Settings* Settings::_psettings( NULL );

//////////////////////////////////////////////////////////////////////////
//
// Settings::Domain
//

Settings::Domain::Domain( const std::string& strDomain /*= std::string()*/ )
	: _strDomain( strDomain )
{
}

Settings::Domain::Domain( const Settings::Domain& domain )
	: _strDomain( domain._strDomain )
{
}

const Settings::Domain& Settings::Domain::operator=( const Settings::Domain& domain )
{
	if( this != &domain )
	{
		_strDomain = domain._strDomain;
	}

	return *this;
}

const std::string& Settings::Domain::domain() const
{
	return _strDomain;
}


bool Settings::Domain::isDefault() const
{
	if( NULL == Settings::_psettings )
	{
		return false;
	}

	return _strDomain == Settings::_psettings->_strDomain;
}

void Settings::Domain::setDefault( bool bDefault )
{
	if( NULL == Settings::_psettings )
	{
		return;
	}

	Settings::_psettings->setDefaultDomain( *this );
}


//////////////////////////////////////////////////////////////////////////
//
// Settings::Profile
//

Settings::Profile::Profile( const std::string& strProfile /*= std::string()*/ )
	: Auth4cData::Profile( strProfile )
{
}

Settings::Profile::Profile( const Settings::Profile& profile )
	: Auth4cData::Profile( profile )
{
}

Settings::Profile::Profile( const Auth4cData::Profile& profile )
	: Auth4cData::Profile( profile )
{
}

const Settings::Profile& Settings::Profile::operator=( const Settings::Profile& profile )
{
	if( this != &profile )
	{
		Auth4cData::Profile::operator=( profile );
	}

	return *this;
}

bool Settings::Profile::isDefault() const
{
	if( NULL == Settings::_psettings )
	{
		return false;
	}

	return profile() == Settings::_psettings->_strProfile;
}

void Settings::Profile::setDefault( bool bDefault )
{
	if( NULL == Settings::_psettings )
	{
		return;
	}

	Settings::_psettings->setDefaultProfile( *this );
}


//////////////////////////////////////////////////////////////////////////
//
// Settings
//

Settings::Settings()
	: _bAdmin( false ),
#if defined (LINUX)
	_docHKCU( "HKCU", "1.0" ),
	_docHKLM( "HKLM", "1.0" ),
	_keyHKCU( NULL ),
	_keyHKLM( NULL ),
#endif /* LINUX */
	_keyRoot4cCU( NULL ),
	_keyRoot4cLM( NULL ),
	_keyRootAuth4cCU( NULL ),
	_keyRootAuth4cLM( NULL )
{
#if defined (LINUX)
	_strHKLM = "/etc/auth4cd/rcauth4c";
	char* pszHome = getenv( "HOME" );
	if( NULL != pszHome )
	{
		_strHKCU = pszHome;
		_strHKCU += "/.rcauth4c";
	}
#endif /* LINUX */
}

void Settings::load( bool bAdmin /*= false*/ )
{
#if defined (LINUX)
	int hHKLM;
	int hHKCU;
#elif defined (_WINDOWS)
	DWORD dwLen;
	LONG lRetCode;
#endif /* LINUX */
	std::string strFailedKeys;

	_bAdmin = bAdmin;

#if defined (LINUX)
	if( _bAdmin )
	{
		hHKLM = ::open( _strHKLM.c_str(), O_RDWR | O_CREAT, S_IWUSR | S_IRUSR | S_IRGRP | S_IROTH );
	}
	else
	{
		hHKLM = ::open( _strHKLM.c_str(), O_RDONLY );
	}

	if( -1 == hHKLM )
	{
		if( strFailedKeys.length() )
		{
			strFailedKeys += ", ";
		}

		strFailedKeys += "HKLM (";
		strFailedKeys += _strHKLM;
		strFailedKeys += ")";
	}

	hHKCU = ::open( _strHKCU.c_str(), O_RDWR | O_CREAT, S_IWUSR | S_IRUSR | S_IRGRP );

	if( -1 == hHKCU )
	{
		if( strFailedKeys.length() )
		{
			strFailedKeys += ", ";
		}

		strFailedKeys += "HKCU";
	}

	if( strFailedKeys.length() )
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_BADCONFIG, strFailedKeys );
	}

	char szBuf[1025];
	int nRead;
	Rfc::String strBuf;
	const char* pszStream;

	nRead = read( hHKLM, szBuf, 1024 );
	while( nRead )
	{
		szBuf[nRead] = '\0';
		strBuf += szBuf;
		nRead = read( hHKLM, szBuf, 1024 );
	}

	if( strBuf.length() )
	{
		pszStream = (const char*)strBuf;
		try
		{
//			_docHKLM.fromStream( pszStream, strBuf.length() );
			_docHKLM.fromStream( pszStream );
		}
		catch( ... )
		{
			if( strFailedKeys.length() )
			{
				strFailedKeys += ", ";
			}

			strFailedKeys += "HKLM";
		}
	}

	strBuf = Rfc::String();

	nRead = read( hHKCU, szBuf, 1024 );
	while( nRead )
	{
		szBuf[nRead] = '\0';
		strBuf += szBuf;
		nRead = read( hHKCU, szBuf, 1024 );
	}

	if( strBuf.length() )
	{
		pszStream = (const char*)strBuf;
		try
		{
//			_docHKCU.fromStream( pszStream, strBuf.length() );
			_docHKCU.fromStream( pszStream );
		}
		catch( ... )
		{
			if( strFailedKeys.length() )
			{
				strFailedKeys += ", ";
			}

			strFailedKeys += "HKCU";
		}
	}

	::close( hHKLM );
	::close( hHKCU );

	if( strFailedKeys.length() )
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_BADCONFIG, strFailedKeys );
	}

	_keyHKCU = _docHKCU.rootNode();
	_keyHKLM = _docHKLM.rootNode();

	if( NULL == _keyHKCU )
	{
		if( strFailedKeys.length() )
		{
			strFailedKeys += ", ";
		}

		strFailedKeys += "HKCU";
	}

	if( NULL == _keyHKLM )
	{
		if( strFailedKeys.length() )
		{
			strFailedKeys += ", ";
		}

		strFailedKeys += "HKLM";
	}

	if( strFailedKeys.length() )
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_BADCONFIG, strFailedKeys );
	}

	_keyRoot4cLM = _keyHKLM->findChild( szROOT4C );

	if( ( NULL == _keyRoot4cLM ) &&
		( _bAdmin ) )
	{
		_keyRoot4cLM = _keyHKLM->addNode( szROOT4C, Rfc::String::null );
	}

	_keyRoot4cCU = _keyHKCU->findChild( szROOT4C );

	if( NULL == _keyRoot4cCU )
	{
		_keyRoot4cCU = _keyHKCU->addNode( szROOT4C, Rfc::String::null );
	}

	if( NULL == _keyRoot4cLM )
	{
		if( strFailedKeys.length() )
		{
			strFailedKeys += ", ";
		}

		strFailedKeys += "HKLM\\company4c";
	}

	if( NULL == _keyRoot4cCU )
	{
		if( strFailedKeys.length() )
		{
			strFailedKeys += ", ";
		}

		strFailedKeys += "HKCU\\company4c";
	}

	if( strFailedKeys.length() )
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_BADCONFIG, strFailedKeys );
	}

// Auth4c
	_keyRootAuth4cLM = _keyRoot4cLM->findChild( szAUTH4C );

	if( ( NULL == _keyRootAuth4cLM ) &&
		( _bAdmin ) )
	{
		_keyRootAuth4cLM = _keyRoot4cLM->addNode( szAUTH4C, Rfc::String::null );
	}

	_keyRootAuth4cCU = _keyRoot4cCU->findChild( szAUTH4C );

	if( NULL == _keyRootAuth4cCU )
	{
		_keyRootAuth4cCU = _keyRoot4cCU->addNode( szAUTH4C, Rfc::String::null );
	}

	if( NULL == _keyRootAuth4cLM )
	{
		if( strFailedKeys.length() )
		{
			strFailedKeys += ", ";
		}

		strFailedKeys += "HKLM\\company4c\\Auth4c";
	}

	if( NULL == _keyRootAuth4cCU )
	{
		if( strFailedKeys.length() )
		{
			strFailedKeys += ", ";
		}

		strFailedKeys += "HKCU\\company4c\\Auth4c";
	}

	if( strFailedKeys.length() )
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_BADCONFIG, strFailedKeys );
	}

	Rfc::XMLNode* pnodeValue;

	pnodeValue = _keyRootAuth4cLM->findChild( szCONNECT );
	if( ( NULL != pnodeValue ) &&
		( !pnodeValue->value().isNull() ) )
	{
		_strAuth4cLocal = (const char*)pnodeValue->value();
	}

	pnodeValue = _keyRootAuth4cLM->findChild( szMASTER );
	if( ( NULL != pnodeValue ) &&
		( !pnodeValue->value().isNull() ) )
	{
		_strAuth4cMaster = (const char*)pnodeValue->value();
	}

	pnodeValue = _keyRootAuth4cCU->findChild( szDOMAIN );
	if( ( NULL != pnodeValue ) &&
		( !pnodeValue->value().isNull() ) )
	{
		_strDomain = (const char*)pnodeValue->value();
	}

	pnodeValue = _keyRootAuth4cCU->findChild( szPROFILE );
	if( ( NULL != pnodeValue ) &&
		( !pnodeValue->value().isNull() ) )
	{
		_strProfile = (const char*)pnodeValue->value();
	}

	pnodeValue = _keyRootAuth4cCU->findChild( szUSERID );
	if( ( NULL != pnodeValue ) &&
		( !pnodeValue->value().isNull() ) )
	{
		_strUserId = (const char*)pnodeValue->value();
	}

	Rfc::XMLNode* keyDomains = _keyRootAuth4cCU->findChild( szDOMAINS );
	if( NULL == keyDomains )
	{
		_keyRootAuth4cCU->addNode( szDOMAINS, Rfc::String::null );
	}
	else
	{
		pnodeValue = keyDomains->findChild( szDOMAIN );
		while( NULL != pnodeValue )
		{
			if( !pnodeValue->value().isNull() )
			{
				Domain domain( (const char*)pnodeValue->value() );
				_domainS[domain.domain()] = domain;
			}

			pnodeValue = keyDomains->findChild( szDOMAIN, pnodeValue );
		}
	}

	Rfc::XMLNode* keyProfiles = _keyRootAuth4cCU->findChild( szPROFILES );
	if( NULL == keyProfiles )
	{
		_keyRootAuth4cCU->addNode( szPROFILES, Rfc::String::null );
	}
	else
	{
		pnodeValue = keyProfiles->findChild( szPROFILE );
		while( NULL != pnodeValue )
		{
			if( !pnodeValue->value().isNull() )
			{
				Profile profile( (const char*)pnodeValue->value() );
				_profileS[profile.profile()] = profile;
			}

			pnodeValue = keyProfiles->findChild( szPROFILE, pnodeValue );
		}
	}
#elif defined (_WINDOWS)
	if( _bAdmin )
	{
		#if defined (_DEBUG)
			lRetCode = RegCreateKeyExA( HKEY_CURRENT_USER, szROOT4C, 0, "", REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL, &_keyRoot4cLM, NULL );
		#else
			lRetCode = RegCreateKeyExA( HKEY_LOCAL_MACHINE, szROOT4C, 0, "", REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL, &_keyRoot4cLM, NULL );
		#endif
	}
	else
	{
		#if defined (_DEBUG)
			lRetCode = RegOpenKeyExA( HKEY_CURRENT_USER, szROOT4C, 0, KEY_READ, &_keyRoot4cLM );
		#else
			lRetCode = RegOpenKeyExA( HKEY_LOCAL_MACHINE, szROOT4C, 0, KEY_READ, &_keyRoot4cLM );
		#endif
	}

	lRetCode = RegCreateKeyExA( HKEY_CURRENT_USER, szROOT4C, 0, "", REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL, &_keyRoot4cCU, NULL );

	if( NULL == _keyRoot4cLM )
	{
		if( strFailedKeys.length() )
		{
			strFailedKeys += ", ";
		}

		strFailedKeys += "HKLM\\Software\\company4c";
	}

	if( NULL == _keyRoot4cCU )
	{
		if( strFailedKeys.length() )
		{
			strFailedKeys += ", ";
		}

		strFailedKeys += "HKCU\\Software\\company4c";
	}

	if( strFailedKeys.length() )
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_BADCONFIG, strFailedKeys );
	}

	if( _bAdmin )
	{
		lRetCode = RegCreateKeyExA( _keyRoot4cLM, szAUTH4C, 0, "", REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL, &_keyRootAuth4cLM, NULL );
	}
	else
	{
		lRetCode = RegOpenKeyExA( _keyRoot4cLM, szAUTH4C, 0, KEY_READ, &_keyRootAuth4cLM );
	}

	lRetCode = RegCreateKeyExA( _keyRoot4cCU, szAUTH4C, 0, "", REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL, &_keyRootAuth4cCU, NULL );

	if( NULL == _keyRootAuth4cLM )
	{
		if( strFailedKeys.length() )
		{
			strFailedKeys += ", ";
		}

		strFailedKeys += "HKLM\\Software\\company4c\\Auth4c";
	}

	if( NULL == _keyRootAuth4cCU )
	{
		if( strFailedKeys.length() )
		{
			strFailedKeys += ", ";
		}

		strFailedKeys += "HKCU\\Software\\company4c\\Auth4c";
	}

	if( strFailedKeys.length() )
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_BADCONFIG, strFailedKeys );
	}

	if( NULL != _keyRootAuth4cLM )
	{
		char szAuth4c[64];
		dwLen = sizeof szAuth4c;
		szAuth4c[0] = 0;
		if( ERROR_SUCCESS != RegQueryValueExA( _keyRootAuth4cLM, szCONNECT, NULL, NULL, (LPBYTE)szAuth4c, &dwLen ) )
			szAuth4c[0] = 0;

		_strAuth4cLocal = szAuth4c;

		dwLen = sizeof szAuth4c;
		szAuth4c[0] = 0;
		if( ERROR_SUCCESS != RegQueryValueExA( _keyRootAuth4cLM, szMASTER, NULL, NULL, (LPBYTE)szAuth4c, &dwLen ) )
			szAuth4c[0] = 0;

		_strAuth4cMaster = szAuth4c;

		if( !_strAuth4cMaster.length() )
		{
			_strAuth4cMaster = _strAuth4cLocal;
		}
	}

	if( NULL != _keyRootAuth4cCU )
	{
		char szDomain[64];
		dwLen = sizeof szDomain;
		szDomain[0] = 0;
		if( ERROR_SUCCESS != RegQueryValueExA( _keyRootAuth4cCU, szDOMAIN, NULL, NULL, (LPBYTE)szDomain, &dwLen ) )
			szDomain[0] = 0;

		_strDomain = szDomain;

// Domains
		HKEY keyDomainS;
		HKEY keyDomain;
		RegCreateKeyExA( _keyRootAuth4cCU, szDOMAINS, 0, "", REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL, &keyDomainS, NULL );
		if( NULL != keyDomainS )
		{
			char szDomain[64];
			DWORD dwIndex = 0;
			DWORD dwLen( sizeof szDomain );
			while( ERROR_NO_MORE_ITEMS != ( lRetCode = RegEnumKeyExA( keyDomainS, dwIndex, szDomain, &dwLen, NULL, NULL, NULL, NULL ) ) )
			{
				if( lRetCode == ERROR_SUCCESS )
				{
					RegOpenKeyExA( keyDomainS, szDomain, 0, KEY_READ, &keyDomain );
					if( keyDomain != NULL )
					{
						RegCloseKey( keyDomain );

						Domain domain( szDomain );
						_domainS[domain.domain()] = domain;
					}
				}

				++dwIndex;
				dwLen = sizeof szDomain;
			}

			RegCloseKey( keyDomainS );
		}

// Profiles
		char szProfile[64];
		dwLen = sizeof szProfile;
		szProfile[0] = 0;
		if( ERROR_SUCCESS != RegQueryValueExA( _keyRootAuth4cCU, szPROFILE, NULL, NULL, (LPBYTE)szProfile, &dwLen ) )
			szProfile[0] = 0;

		_strProfile = szProfile;

#if (0) /* we now get profiles from Auth4c */
		HKEY keyProfileS;
		HKEY keyProfile;
		RegCreateKeyExA( _keyRootAuth4cCU, szPROFILES, 0, "", REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL, &keyProfileS, NULL );
		if( NULL != keyProfileS )
		{
			char szProfile[64];
			DWORD dwIndex = 0;
			DWORD dwLen( sizeof szProfile );
			while( ERROR_NO_MORE_ITEMS != ( lRetCode = RegEnumKeyExA( keyProfileS, dwIndex, szProfile, &dwLen, NULL, NULL, NULL, NULL ) ) )
			{
				if( lRetCode == ERROR_SUCCESS )
				{
					RegOpenKeyExA( keyProfileS, szProfile, 0, KEY_READ, &keyProfile );
					if( keyProfile != NULL )
					{
						RegCloseKey( keyProfile );

						Profile profile( szProfile );
						_profileS[profile.profile()] = profile;
					}
				}

				++dwIndex;
				dwLen = sizeof szProfile;
			}

			RegCloseKey( keyProfileS );
		}
#endif /* 0 */
// Profiles
		char szUserId[64];
		dwLen = sizeof szUserId;
		szUserId[0] = 0;
		if( ERROR_SUCCESS != RegQueryValueExA( _keyRootAuth4cCU, szUSERID, NULL, NULL, (LPBYTE)szUserId, &dwLen ) )
			szUserId[0] = 0;

		_strUserId = szUserId;
	}
#endif /* LINUX */

	_psettings = this;
}

void Settings::loadProfiles()
{
	Auth4cData::ProfileMap profileS( Auth4cDataXML::Factory::profiles() );

	for( Auth4cData::ProfileMap::const_iterator iter = profileS.begin();
			iter != profileS.end();
			++iter )
	{
		Auth4cClientConfig::Settings::Profile profile( iter->second );
		_profileS[profile.profile()] = profile;
	}
}

Settings::~Settings()
{
	if( this == _psettings )
	{
		_psettings = NULL;
	}

#if defined (_WINDOWS)
	if( NULL != _keyRootAuth4cCU )
	{
		RegCloseKey( _keyRootAuth4cCU );
	}

	if( NULL != _keyRootAuth4cLM )
	{
		RegCloseKey( _keyRootAuth4cLM );
	}

	if( NULL != _keyRoot4cCU )
	{
		RegCloseKey( _keyRoot4cCU );
	}

	if( NULL != _keyRoot4cLM )
	{
		RegCloseKey( _keyRoot4cLM );
	}
#endif /* _WINDOWS */

	_keyRoot4cCU = NULL;
	_keyRoot4cLM = NULL;
	_keyRootAuth4cCU = NULL;
	_keyRootAuth4cLM = NULL;
}

const Settings::DomainMap& Settings::domains() const
{
	return _domainS;
}

const std::string& Settings::defaultDomain() const
{
	return _strDomain;
}

const Settings::ProfileMap& Settings::profiles() const
{
	return _profileS;
}

const std::string& Settings::defaultProfile() const
{
	return _strProfile;
}

const std::string& Settings::userId() const
{
	return _strUserId;
}

const std::string& Settings::auth4cLocal() const
{
	return _strAuth4cLocal;
}

const std::string& Settings::auth4cMaster() const
{
	return _strAuth4cMaster;
}

void Settings::setDefaultDomain( const Domain& domain )
{
	DomainMap::const_iterator iter = _domainS.find( domain.domain() );
	if( _domainS.end() == iter )
	{
#if defined (LINUX)
		Rfc::XMLNode* keyDomainS = _keyRootAuth4cCU->findChild( szDOMAINS );
		if( NULL == keyDomainS )
		{
			keyDomainS = _keyRootAuth4cCU->addNode( szDOMAINS, Rfc::String::null );
		}

		if( NULL == keyDomainS )
		{
			throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_BADCONFIG, "Failed to create domains key." );
		}

		keyDomainS->addNode( szDOMAIN, domain.domain().c_str() );
#endif /* LINUX */

		Domain domainNew( domain.domain() );
		_domainS[domainNew.domain()] = domainNew;

		_strDomain = domainNew.domain();
	}
	else
	{
		_strDomain = domain.domain();
	}

#if defined (LINUX)
	Rfc::XMLNode* keyDomain = _keyRootAuth4cCU->findChild( szDOMAIN );
	if( NULL == keyDomain )
	{
		keyDomain = _keyRootAuth4cCU->addNode( szDOMAIN, domain.domain().c_str() );
	}
	else
	{
		keyDomain->value( domain.domain().c_str() );
	}

	save();
#elif defined (_WINDOWS)
	HKEY keyDomainS;
	HKEY keyDomain;
	RegCreateKeyExA( _keyRootAuth4cCU, szDOMAINS, 0, "", REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL, &keyDomainS, NULL );
	if( keyDomainS != NULL )
	{
		RegCreateKeyExA( keyDomainS, domain.domain().c_str(), 0, "", REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL, &keyDomain, NULL );
		if( NULL != keyDomain )
		{
			RegSetValueExA( _keyRootAuth4cCU, szDOMAIN, 0, REG_SZ, (const BYTE*)domain.domain().c_str(), domain.domain().length() );
			RegCloseKey( keyDomain );
		}

		RegCloseKey( keyDomainS );
	}
#endif /* LINUX */
}

void Settings::setDefaultProfile( const Profile& profile )
{
	ProfileMap::const_iterator iter = _profileS.find( profile.profile() );
	if( _profileS.end() == iter )
	{
#if defined (LINUX)
		Rfc::XMLNode* keyProfileS = _keyRootAuth4cCU->findChild( szPROFILES );
		if( NULL == keyProfileS )
		{
			keyProfileS = _keyRootAuth4cCU->addNode( szPROFILES, Rfc::String::null );
		}

		if( NULL == keyProfileS )
		{
			throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_BADCONFIG, "Failed to create profiles key." );
		}

		keyProfileS->addNode( szPROFILE, profile.profile().c_str() );
#endif /* LINUX */

		Profile profileNew( profile.profile() );
		_profileS[profileNew.profile()] = profileNew;

		_strProfile = profileNew.profile();
	}
	else
	{
		_strProfile = profile.profile();
	}

#if defined (LINUX)
	Rfc::XMLNode* keyProfile = _keyRootAuth4cCU->findChild( szPROFILE );
	if( NULL == keyProfile )
	{
		keyProfile = _keyRootAuth4cCU->addNode( szPROFILE, profile.profile().c_str() );
	}
	else
	{
		keyProfile->value( profile.profile().c_str() );
	}

	save();
#elif defined (_WINDOWS)
	HKEY keyProfileS;
	HKEY keyProfile;
	RegCreateKeyExA( _keyRootAuth4cCU, szPROFILES, 0, "", REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL, &keyProfileS, NULL );
	if( keyProfileS != NULL )
	{
		RegCreateKeyExA( keyProfileS, profile.profile().c_str(), 0, "", REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL, &keyProfile, NULL );
		if( NULL != keyProfile )
		{
			RegSetValueExA( _keyRootAuth4cCU, szPROFILE, 0, REG_SZ, (const BYTE*)profile.profile().c_str(), profile.profile().length() );
			RegCloseKey( keyProfile );
		}

		RegCloseKey( keyProfileS );
	}
#endif /* LINUX */
}

void Settings::setUserId( const std::string& strUserId )
{
	_strUserId = strUserId;

#if defined (LINUX)
	Rfc::XMLNode* keyUserId = _keyRootAuth4cCU->findChild( szUSERID );
	if( NULL == keyUserId )
	{
		keyUserId = _keyRootAuth4cCU->addNode( szUSERID, _strUserId.c_str() );
	}
	else
	{
		keyUserId->value( _strUserId.c_str() );
	}

	save();
#elif defined (_WINDOWS)
	RegSetValueExA( _keyRootAuth4cCU, szUSERID, 0, REG_SZ, (const BYTE*)_strUserId.c_str(), _strUserId.length() );
#endif /* LINUX */
}

void Settings::setAuth4cLocal( const std::string& strConnect )
{
#if defined (LINUX)
	Rfc::XMLNode* pnodeValue = _keyRootAuth4cLM->findChild( szCONNECT );
	if( NULL == pnodeValue )
	{
		_keyRootAuth4cLM->addNode( szCONNECT, strConnect.c_str() );
	}
	else
	{
		pnodeValue->value( strConnect.c_str() );
	}

	save();
#elif defined (_WINDOWS)
	LONG lRetCode = RegSetValueExA( _keyRootAuth4cLM, szCONNECT, 0, REG_SZ, (const BYTE*)strConnect.c_str(), strConnect.length() );
	if( lRetCode )
	{
		std::ostringstream os;
		os << lRetCode;
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_BADCONFIG, os.str() );
	}
#endif /* LINUX */
}

void Settings::setAuth4cMaster( const std::string& strConnect )
{
#if defined (LINUX)
	Rfc::XMLNode* pnodeValue = _keyRootAuth4cLM->findChild( szMASTER );
	if( NULL == pnodeValue )
	{
		_keyRootAuth4cLM->addNode( szMASTER, strConnect.c_str() );
	}
	else
	{
		pnodeValue->value( strConnect.c_str() );
	}

	save();
#elif defined (_WINDOWS)
	LONG lRetCode = RegSetValueExA( _keyRootAuth4cLM, szMASTER, 0, REG_SZ, (const BYTE*)strConnect.c_str(), strConnect.length() );
	if( lRetCode )
	{
		std::ostringstream os;
		os << lRetCode;
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_BADCONFIG, os.str() );
	}
#endif /* LINUX */
}

#if defined (LINUX)
void Settings::save()
{
	Rfc::String str;

	str = _docHKCU.stream();
	int hHKCU = ::open( _strHKCU.c_str(), O_RDWR | O_CREAT | O_TRUNC, S_IWUSR | S_IRUSR | S_IRGRP );
	if( -1 == hHKCU )
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_BADCONFIG, "Failed to open HKCU For writing." );
	}
	::write( hHKCU, (const char*)str, str.length() );
	::close( hHKCU );

	if( _bAdmin )
	{
		str = _docHKLM.stream();
		int hHKLM = ::open( _strHKLM.c_str(), O_RDWR | O_CREAT | O_TRUNC, S_IWUSR | S_IRUSR | S_IRGRP | S_IROTH );
		if( -1 == hHKLM )
		{
			throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_BADCONFIG, "Failed to open HKLM For writing." );
		}
		::write( hHKLM, (const char*)str, str.length() );
		::close( hHKLM );
	}
}
#endif /* LINUX */
