/*

  $Id: Auth4cDB.cpp 689 2018-11-28 17:54:35Z john $

  Copyright (c) 2008, The 4c Company. All rights reserved.
*/

#include <Auth4cDal/Auth4cDB.h>

#include <librfcdal/DalMySQLDatabase.h>
#include <librfcdal/DalDynamic.h>
#include <librfcdal/DalDatabaseErrors.h>
#include <Base4c/Mutex.h>

#include <iostream>

namespace Auth4cDal
{
RfcDal::SqlSchema g_schema;

Auth4cDal::Auth4cDBList g_databaseS;
Base4c::Mutex g_mxDatabase( "mxDatabase" );
RfcDal::DalDatabase::ConnectionParameterMap g_map;
}

using namespace Auth4cDal;

/////////////////////////////////////////////////////////////////////
//
// Auth4cDB
//

Auth4cDB::Auth4cDB( RfcDal::DalDatabase* pdb /*= NULL*/ )
	: _pdb( pdb ),
	_nUsageCount( 0 ),
	_mx( "Auth4cDB" )
{
}

Auth4cDB::Auth4cDB( const Auth4cDB& db )
	: _pdb( db._pdb ),
	_nUsageCount( db._nUsageCount ),
	_mx( "Auth4cDB" )
{
}

Auth4cDB::~Auth4cDB()
{
}

Auth4cDB& Auth4cDB::operator=( const Auth4cDB& db )
{
	if( this != &db )
	{
		_pdb = db._pdb;
		_nUsageCount = db._nUsageCount;
	}

	return *this;
}

bool Auth4cDB::inUse() const
{
	return _nUsageCount ? true : false;
}

void Auth4cDB::ucInc()
{
	_mx.lock( "Auth4cDB::ucInc()" );
	++_nUsageCount;
	_mx.unlock();
}

void Auth4cDB::ucDec()
{
	_mx.lock( "Auth4cDB::ucDec()" );
	--_nUsageCount;
	_mx.unlock();
}


/////////////////////////////////////////////////////////////////////
//
// Connection
//

Connection::Connection( Auth4cDB* pdb )
	: _pdb( pdb )
{
	if( NULL != _pdb )
	{
		_pdb->ucInc();
	}
}

Connection::Connection( const Connection& connection )
	: _pdb( connection._pdb )
{
	if( NULL != _pdb )
	{
		_pdb->ucInc();
	}
}

Connection::~Connection()
{
	if( NULL != _pdb )
	{
		_pdb->ucDec();
	}
}

Connection& Connection::operator=( Connection& connection )
{
	if( this != &connection )
	{
		if( NULL != _pdb )
		{
			_pdb->ucDec();
		}
		_pdb = connection._pdb;
		if( NULL != _pdb )
		{
			_pdb->ucInc();
		}
	}

	return *this;
}

RfcDal::DalDatabase& Connection::db()
{
	return *_pdb->_pdb;
}

Connection::operator RfcDal::DalDatabase&()
{
	return db();
}

/////////////////////////////////////////////////////////////////////
//
// Global scope
//

const RfcDal::SqlSchema& Auth4cDal::schema()
{
	return g_schema;
}

Connection Auth4cDal::connection()
{
	g_mxDatabase.lock( "Auth4cDal::connection()" );

	for( std::vector<Auth4cDB>::iterator iterDB = g_databaseS.begin();
			iterDB != g_databaseS.end();
			++iterDB )
	{
		if( !iterDB->inUse() )
		{
			Connection conn( &(*iterDB) );
			g_mxDatabase.unlock();
			return conn;
		}
	}

	std::string strEngine( "mysql" );

	RfcDal::DalDatabase::ConnectionParameterMap::const_iterator iterEngine = g_map.find( "engine" );
	if( iterEngine != g_map.end() )
	{
		strEngine = iterEngine->second;
	}

	//RfcDal::DalDatabase* pdatabase = new RfcDal::DalMySQLDatabase;
	RfcDal::DalDatabase* pdatabase = &RfcDal::DalDynamic::dalDatabase( strEngine );

	try
	{
		pdatabase->connectX( g_map );
		Auth4cDB dbNA( pdatabase );
		g_databaseS.push_back( Auth4cDB( dbNA ) );

		std::cout << "Adding new connection. Count=" << g_databaseS.size() << "\n";

		Connection conn( &dbNA );
		g_mxDatabase.unlock();
		return conn;
	}
	catch( const RfcDal::DalDatabaseException& e )
	{
		g_mxDatabase.unlock();
		delete pdatabase;
		pdatabase = NULL;
		throw;
	}

	Connection* pconn( NULL );
	return *pconn;
}

void Auth4cDal::connect( const std::string& strConnect )
{
	disconnect();

	RfcDal::DalDatabase* pdatabase;
	g_map.clear();
	RfcDal::DalDatabase::connectStringToMap( g_map, strConnect );
	std::string strEngine( "mysql" );

	RfcDal::DalDatabase::ConnectionParameterMap::const_iterator iter = g_map.find( "database" );
	if( iter != g_map.end() )
	{
		RfcDal::SqlSchema schema( iter->second );
		g_schema = schema;
	}
	else
	{
		throw RfcDal::DalDatabaseException( __FILE__, __LINE__, RfcDal::DDBERR_USER, 1, "No Schema/Database specified" );
	}

	RfcDal::DalDatabase::ConnectionParameterMap::const_iterator iterEngine = g_map.find( "engine" );
	if( iterEngine != g_map.end() )
	{
		strEngine = iterEngine->second;
	}

	for( int nIdx = 0; nIdx < 10; ++nIdx )
	{
//		pdatabase = new RfcDal::DalMySQLDatabase;
		pdatabase = &RfcDal::DalDynamic::dalDatabase( strEngine );

		try
		{
			pdatabase->connectX( g_map );
			g_databaseS.push_back( Auth4cDB( pdatabase ) );
		}
		catch( const RfcDal::DalDatabaseException& e )
		{
			delete pdatabase;
			pdatabase = NULL;
			throw;
		}
	}
}

void Auth4cDal::disconnect()
{
	g_databaseS.clear();
}
