#include <Auth4cDal/AuthenticationLibraryXQuery.h>

#if defined (DEBUG)
# include <iostream>
#endif /* DEBUG */

Auth4cDal::AuthenticationLibraryXQuery::AuthenticationLibraryXQuery( const RfcDal::SqlSchema& schema )
	: AuthenticationLibraryQuery( schema ),
	_tbSharedObject( schema, "SharedObject" ),
	_colPlatform( this, _tbSharedObject, "Platform" ),
	_colAuthenticationLibrary1( this, _tbSharedObject, "AuthenticationLibrary" ),
	_colSharedObject( this, _tbSharedObject, "SharedObject" )
{
}

Auth4cDal::AuthenticationLibraryXQuery::AuthenticationLibraryXQuery( const RfcDal::SqlTable& table )
	: AuthenticationLibraryQuery( table ),
	_tbSharedObject( table.schema(), "SharedObject" ),
	_colPlatform( this, _tbSharedObject, "Platform" ),
	_colAuthenticationLibrary1( this, _tbSharedObject, "AuthenticationLibrary" ),
	_colSharedObject( this, _tbSharedObject, "SharedObject" )
{
}

Auth4cDal::AuthenticationLibraryXQuery::~AuthenticationLibraryXQuery()
{
}

const RfcDal::SqlColumn& Auth4cDal::AuthenticationLibraryXQuery::columnPlaform() const
{
	return _colPlatform;
}

const RfcDal::SqlColumn& Auth4cDal::AuthenticationLibraryXQuery::columnSharedObject() const
{
	return _colSharedObject;
}

RfcDal::SqlColumn& Auth4cDal::AuthenticationLibraryXQuery::columnPlaform()
{
	return _colPlatform;
}

RfcDal::SqlColumn& Auth4cDal::AuthenticationLibraryXQuery::columnSharedObject()
{
	return _colSharedObject;
}

void Auth4cDal::AuthenticationLibraryXQuery::constrainPlatform( const RfcDal::SqlColumnPtr& value )
{
	_valPlatformConstraint = value;
}

void Auth4cDal::AuthenticationLibraryXQuery::addConstraints( RfcDal::SqlCompoundCondition& conditionS, bool bAddBaseConstraints /*= true*/ ) 
{
	if( bAddBaseConstraints )
	{
		AuthenticationLibraryQuery::addConstraints( conditionS, bAddBaseConstraints );
	}

	if( _valPlatformConstraint.isValid() )
	{
		RfcDal::SqlCondition condPlatform( _colPlatform, _valPlatformConstraint, RfcDal::SqlCondition::condEqual );
		conditionS.addCondition( condPlatform );
	}
}

int Auth4cDal::AuthenticationLibraryXQuery::select( RfcDal::DalDatabase::Result& result, RfcDal::DalDatabase& database, RfcDal::SqlQueryBase::QueryType type /*= RfcDal::SqlQueryBase::full*/ ) 
{
	RfcDal::SqlSelectStatement statement( _tbAuthenticationLibrary );

	addColumns( statement, RfcDal::SqlQueryBase::invalidToo );

	RfcDal::SqlCompoundCondition condition( RfcDal::SqlCompoundCondition::compAnd );

	addConstraints( condition );

	RfcDal::SqlCondition condJoinPlatform( _colAuthenticationLibrary, _colAuthenticationLibrary1, RfcDal::SqlCondition::condEqual );
	RfcDal::SqlJoin joinSharedObject( _tbSharedObject, RfcDal::SqlJoin::joinInner, condJoinPlatform );
	statement.addJoin( joinSharedObject );

	if( condition.count() )
		statement.setWhere( condition );

#if defined (DEBUG)
	std::cout << statement.toSql( database.language() ) << std::endl;
#endif /* DEBUG */

	return database.execute( result, statement );
}

int Auth4cDal::AuthenticationLibraryXQuery::insert( RfcDal::DalDatabase& database ) 
{

	return 0;
}

int Auth4cDal::AuthenticationLibraryXQuery::update( RfcDal::DalDatabase& database ) 
{

	return 0;
}

int Auth4cDal::AuthenticationLibraryXQuery::remove( RfcDal::DalDatabase& database ) 
{

	return 0;
}
