/*

  $Id: AuthenticatedUser.cpp 689 2018-11-28 17:54:35Z john $

  Copyright (c) 2008, The 4c Company. All rights reserved.
*/

#include <Auth4cData/AuthenticatedUser.h>
#include <Auth4cData/Exception.h>
#include <Auth4cData/Session.h>
#include <Auth4cData/SessionString.h>
#include <Auth4cData/Domain.h>
#include <Auth4cData/Authenticator.h>

using namespace Auth4cData;

AuthenticatedUser::AuthenticatedUser()
: _psession( NULL )
{
}

AuthenticatedUser::AuthenticatedUser( const User& user, const Session& session, const Profile& profile /*= Profile()*/ )
	: User( user ),
	_psession( session.copy() ),
	_profile( profile )
{
}

AuthenticatedUser::AuthenticatedUser( const AuthenticatedUser& user )
	: User( user ),
	_psession( NULL ),
	_profile( user._profile )
{
	if( NULL != user._psession )
	{
		_psession = user._psession->copy();
	}
}

AuthenticatedUser::~AuthenticatedUser()
{
	if ( _psession != NULL )
	{
		delete _psession;
	}
}

const AuthenticatedUser& AuthenticatedUser::operator=( const AuthenticatedUser& user )
{
	if( this != &user )
	{
		User::operator=( user );

		if( _psession != NULL )
		{
			delete _psession;
		}

		_psession = NULL;

		if( NULL != user._psession )
		{
			_psession = user._psession->copy();
		}

		_profile = user._profile;
	}

	return *this;
}

const Session& AuthenticatedUser::session() const 
{
	if( _psession == NULL )
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_BADSESSION, "Null session." );
	}

	return *_psession;
}

const Profile& AuthenticatedUser::profile() const
{
	return _profile;
}

Session& AuthenticatedUser::session() 
{
	if( _psession == NULL )
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_BADSESSION, "Null session." );
	}

	return *_psession;
}

void AuthenticatedUser::passwd( Auth4cData::AuthenticationToken& authTokenCurrent, Auth4cData::AuthenticationToken& authTokenNew )
{
	domain().authenticator().passwd( *this, authTokenCurrent, authTokenNew );
}
