/*

  $Id: Authenticator.cpp 689 2018-11-28 17:54:35Z john $

  Copyright (c) 2008, The 4c Company. All rights reserved.
*/

#include <Auth4cData/Authenticator.h>

using namespace Auth4cData;


Authenticator::Authenticator( const InitMap& initAuthenticator,
								const Domain& domain /*= Domain()*/,
								const std::string& strDescription /*= std::string()*/,
								int nAuthenticationLibrary /*= 0*/ )
	: _domain( domain ),
	_strDescription( strDescription ),
	_nAuthenticationLibrary( nAuthenticationLibrary ),
	_initAuthenticator( initAuthenticator ),
	_bUserTools( false )
{
}

Authenticator::Authenticator( const Authenticator& authenticator )
	: _domain( authenticator._domain ),
	_strDescription( authenticator._strDescription ),
	_nAuthenticationLibrary( authenticator._nAuthenticationLibrary ),
	_initAuthenticator( authenticator._initAuthenticator ),
	_bUserTools( authenticator._bUserTools )
{
}

Authenticator::~Authenticator()
{
}

const Authenticator& Authenticator::operator=( const Authenticator& authenticator )
{
	if( this != &authenticator )
	{
		_domain = authenticator._domain;
		_strDescription = authenticator._strDescription;
		_nAuthenticationLibrary = authenticator._nAuthenticationLibrary;
		_initAuthenticator = authenticator._initAuthenticator;
		_bUserTools = authenticator._bUserTools;
	}

	return *this;
}

const std::string& Authenticator::description() const
{
	return _strDescription;
}

int Authenticator::authenticationLibrary() const
{
	return _nAuthenticationLibrary;
}

//const std::string& Authenticator::authenticatorInitString() const
//{
//	return _strAuthenticatorInitString;
//}

const InitMap& Authenticator::authenticatorInit() const
{
	return _initAuthenticator;
}

const Domain& Authenticator::domain() const
{
	return _domain;
}

bool Authenticator::userTools() const
{
	return _bUserTools;
}

void Authenticator::setDescription( const std::string& strDescription )
{
	_strDescription = strDescription;
}

void Authenticator::setAuthenticationLibrary( int nAuthenticationLibrary )
{
	_nAuthenticationLibrary = nAuthenticationLibrary;
}

void Authenticator::setAuthenticatorInit( const InitMap& initAuthenticator )
{
	_initAuthenticator = initAuthenticator;
}

void Authenticator::setUserTools( bool bUserTools )
{
	_bUserTools = bUserTools;
}

void Authenticator::users( UserMap& userS ) 
{
	throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_NOTIMPLEMENTED, "users" );
}

void Authenticator::passwd( const Auth4cData::AuthenticatedUser& user, Auth4cData::AuthenticationToken& authTokenCurrent, Auth4cData::AuthenticationToken& authTokenNew ) 
{
	throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_NOTIMPLEMENTED, "passwd" );
}

void Authenticator::createUser( const Auth4cData::User& user ) 
{
	throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_NOTIMPLEMENTED, "createUser" );
}

void Authenticator::updateUser( const Auth4cData::User& user ) 
{
	throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_NOTIMPLEMENTED, "updateUser" );
}

void Authenticator::deleteUser( const Auth4cData::User& user ) 
{
	throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_NOTIMPLEMENTED, "deleteUser" );
}

void Authenticator::createTeamMember( Auth4cData::Team& team, Auth4cData::User& user ) 
{
	throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_NOTIMPLEMENTED, "createTeamMember" );
}

void Authenticator::deleteTeamMember( Auth4cData::Team& team, Auth4cData::User& user ) 
{
	throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_NOTIMPLEMENTED, "deleteTeamMember" );
}
