/*

  $Id: Domain.cpp 689 2018-11-28 17:54:35Z john $

  Copyright (c) 2008, The 4c Company. All rights reserved.
*/

#include <Auth4cData/Domain.h>

#include <Auth4cData/User.h>
#include <Auth4cData/Authenticator.h>

using namespace Auth4cData;

Domain::Domain()
	: _pcustomer( NULL ),
	_pauthenticator( NULL )
{
}

Domain::Domain( const Customer& customer, const std::string strDomain )
	: _pcustomer( &customer ),
	_strDomain( strDomain ),
	_pauthenticator( NULL )
{
}

Domain::Domain( const Domain& domain )
	: _pcustomer( domain._pcustomer ),
	_strDomain( domain._strDomain ),
	_strDescription( domain._strDescription ),
	_pauthenticator( domain._pauthenticator )
{
}

Domain::~Domain()
{
}

const Domain& Domain::operator=( const Domain& domain )
{
	if( this != &domain )
	{
		_pcustomer = domain._pcustomer;
		_strDomain = domain._strDomain;
		_strDescription = domain._strDescription;
		_pauthenticator = domain._pauthenticator;
	}

	return *this;
}

bool Domain::operator==( const Domain& domainC ) const
{
	if( domain() == domainC.domain() )
	{
		return true;
	}

	return false;
}

const Customer& Domain::customer() const
{
	if( NULL == _pcustomer )
	{
		throw Exception( __FILE__, __LINE__, nNAX_NOCUSTOMERSET, "Customer pointer is null." );
	}

	return *_pcustomer;
}

const std::string& Domain::domain() const
{
	return _strDomain;
}

const std::string& Domain::description() const
{
	return _strDescription;
}

const UserMap& Domain::users() const
{
	return _userS;
}

Authenticator& Domain::authenticator() const
{
	if( NULL == _pauthenticator )
	{
		Exception e( __FILE__, __LINE__, nNAX_NOAUTHENTICATORSET, "Authenticator pointer is NULL." );
		throw e;
	}

	return *_pauthenticator;
}

void Domain::setDescription( const std::string& strDescription )
{
	_strDescription = strDescription;
}

void Domain::setAuthenticator( Authenticator& authenticator )
{
	_pauthenticator = &authenticator;
}

UserMap& Domain::users( bool bRefresh /*= false*/ ) 
{
	if( bRefresh )
	{
		_userS.clear();

		authenticator().users( _userS );
	}

	return _userS;
}

void Domain::createUser( const Auth4cData::User& user ) 
{
	authenticator().createUser( user );
}

void Domain::deleteUser( const Auth4cData::User& user ) 
{
	authenticator().deleteUser( user );
}
