/*

  $Id: Exception.cpp 737 2020-09-23 18:25:13Z john $

  Copyright (c) 2008, The 4c Company. All rights reserved.
*/

#include <Auth4cData/Exception.h>

namespace Auth4cData
{
	struct Explanation
	{
		int nErrorCode;
		const char* pszExplanation;
	};
};

using namespace Auth4cData;

struct Explanation explanationS[] =
{
	{ nNAX_UNKNOWN, "Unknown Error." },
	{ nNAX_AUTHENTICATIONFAILED, "Authentication Failed." },
	{ nNAX_APPLICATIONABORTED, "Application Aborted." },
	{ nNAX_BADREQUEST, "Bad Request." },
	{ nNAX_RESPONSETIMEOUT, "Response Timeout." },
	{ nNAX_ALIENEXCEPTION, "Alien exception caught." },
	{ nNAX_USERNOTFOUND, "User not found." },
	{ nNAX_CUSTOMERNOTFOUND, "Customer not found." },
	{ nNAX_DOMAINNOTFOUND, "Domain not found." },
	{ nNAX_TEAMNOTFOUND, "Team not found." },
	{ nNAX_PRODUCTNOTFOUND, "Product not found." },
	{ nNAX_PLATFORMNOTFOUND, "Platform not found." },
	{ nNAX_AUTHENTICATIONLIBRARYNOTFOUND, "Authentication library not found." },
	{ nNAX_PERMISSIONNOTFOUND, "Permission not found." },
	{ nNAX_ROLEPERMISSIONNOTFOUND, "Role permission not found." },
	{ nNAX_BADSESSION, "Bad session." },
	{ nNAX_NOTEAMLEADSET, "No team lead set." },
	{ nNAX_NOCUSTOMERSET, "No customer set." },
	{ nNAX_NODOMAINSET, "No domain set." },
	{ nNAX_NOPLATFORMSET, "No platform set." },
	{ nNAX_NOAUTHENTICATIONLIBRARYSET, "No authentication library set." },
	{ nNAX_NOSHAREDOBJECTSET, "No shared object set (for platform)." },
	{ nNAX_NOPAIRSET, "No pair set." },
	{ nNAX_UPDATEFAILED, "Update failed." },
	{ nNAX_MISMATCHEDNODE, "Node name does not match value name." },
	{ nNAX_MALFORMEDXML, "XML is malformed." },
	{ nNAX_DATABASE, "Failed to execute statement." },
	{ nNAX_NOSERVERCONNECT, "Failed to connect to server." },
	{ nNAX_MISSINGDATA, "Attempt to create Auth4c object from incorrect data object type." },
	{ nNAX_BADTEMPLATE, "Bad substitution template." },
	{ nNAX_NOSUCHVARIABLE, "No such variable." },
	{ nNAX_NOSUCHPRODUCT, "No such product." },
	{ nNAX_NOSUCHPROFILE, "No such profile." },
	{ nNAX_NOSHAREDOBJECT, "Failed to load shared object." },
	{ nNAX_NOENTRYPOINT, "Failed to locate entry point." },
	{ nNAX_LIBRARYNOTLOADED, "The AuthenticationLibrary is not loaded." },
	{ nNAX_AUTHENTICATIONLIBRARYINITFAILED, "Failed to initialize the AuthenticationLibrary." },
	{ nNAX_AUTHENTICATORINITFAILED, "Failed to initialize the Authenticator." },
	{ nNAX_NOAUTHENTICATORSET, "Domain has no authenticator set." },
	{ nNAX_NOPRODUCTSET, "Role/Permission has no product set." },
	{ nNAX_NOROLESET, "Permission has no role set." },
	{ nNAX_NODEFAULTPROFILESET, "No default Profile set." },
	{ nNAX_NOUNIQUEUSER, "Failed to uniquely identify a user." },
	{ nNAX_VERSIONMISMATCH, "The server version does not support the client version." },
	{ nNAX_KEYOPENFAILED, "Failed to read a key file." },
	{ nNAX_CONNECTOPENFAILED, "Failed to retrieve connection from file." },
//	{ nNAX_NODOCUMENTSET, "Signature has no document set." },
//	{ nNAX_NODISPOSITIONSET, "Signature has no disposition set." },
//	{ nNAX_DISPOSITIONNOTFOUND, "SignatureDisposition not found." },
//	{ nNAX_DOCPREPFAILED, "Document prep failed." },
//	{ nNAX_DOCSIGNFAILED, "Document sign failed." },
//	{ nNAX_BADCHECKSUM, "Bad checksum." },
//	{ nNAX_BADSIGNATURE, "No qualifying signature found." },
	{ nNAX_BADCONFIG, "Bad Config." },
	{ nNAX_BADUSERFORMAT, "Bad user format." },
	{ nNAX_PERMISSIONDENIED, "Permission Denied." },
	{ nNAX_NOTIMPLEMENTED, "Feature has not been implemented." },
	{ nNAX_FILEOPENFAILED, "File open failed." },
	{ nNAX_SEARCHEXECFAILED, "Failed to execute search." },
	{ 0, NULL }
};

Exception::Exception()
	: _nLine( -1 ),
	_nErrorCode( 0 ),
	_bRemote( false )
{
}

Exception::Exception( const std::string& strFile, int nLine, int nErrorCode, const std::string& strLegend, bool bRemote /*= false*/ )
	: _strFile( strFile ),
	_nLine( nLine ),
	_nErrorCode( nErrorCode ),
	_strLegend( strLegend ),
	_bRemote( bRemote )
{
}

Exception::Exception( const Exception& exception )
	: _strFile( exception._strFile ),
	_nLine( exception._nLine ),
	_nErrorCode( exception._nErrorCode ),
	_strLegend( exception._strLegend ),
	_bRemote( exception._bRemote )
{
}

Exception::~Exception()
{
}

const Exception& Exception::operator=( const Exception& exception )
{
	if( this != &exception )
	{
		_strFile = exception._strFile;
		_nLine = exception._nLine;
		_nErrorCode = exception._nErrorCode;
		_strLegend = exception._strLegend;
		_bRemote = exception._bRemote;
	}

	return *this;
}

const std::string& Exception::file() const
{
	return _strFile;
}

int Exception::line() const
{
	return _nLine;
}

int Exception::errorCode() const
{
	return _nErrorCode;
}

const std::string& Exception::legend() const
{
	return _strLegend;
}

bool Exception::remote() const
{
	return _bRemote;
}

std::string Exception::explainErrorCode( int nErrorCode )
{
	int nIndex( 0 );

	while( ( explanationS[nIndex].pszExplanation != NULL ) &&
			( explanationS[nIndex].nErrorCode != nErrorCode ) )
		++nIndex;

	if( explanationS[nIndex].pszExplanation == NULL )
		return std::string();

	return explanationS[nIndex].pszExplanation;
}

Auth4cData_Export std::ostream& operator<<( std::ostream& stream, const Auth4cData::Exception& e )
{
	stream << "Error Code=" << e.errorCode() << "\n"
			<< "Explanation=\"" << Auth4cData::Exception::explainErrorCode( e.errorCode() ) << "\"\n"
			<< "Legend=\"" << e.legend() << "\"\n"
			<< "Location=" << ( e.remote() ? "REMOTE: " : "" ) << e.file() << " @ " << e.line() << "\n";
	return stream;
}
