/*

  $Id: Factory.cpp 689 2018-11-28 17:54:35Z john $

  Copyright (c) 2008, The 4c Company. All rights reserved.
*/

#include <Auth4cData/Factory.h>
//#include <Auth4cDataDB/Auth4cFactory.h>
#include <Auth4cData/A4CAuth4c.h>

#include <Auth4cDal/ProductQuery.h>
#include <Auth4cDal/ProductRoleQuery.h>
#include <Auth4cDal/ProductPermissionQuery.h>
#include <Auth4cDal/ProductRolePermissionQuery.h>
#include <Auth4cDal/DomainQuery.h>
#include <Auth4cDal/CustomerQuery.h>
#include <Auth4cDal/UserQuery.h>
#include <Auth4cDal/TeamQuery.h>
#include <Auth4cDal/UserTeamQuery.h>
#include <Auth4cDal/AuthenticationLibraryQuery.h>
#include <Auth4cData/AuthenticationToken.h>

#include <MakeVersion.in>

#include <Base4c/EventLogger.h>

#include <sstream>

#if defined (LINUX)
#	include <dlfcn.h>
#endif /* LINUX */

using namespace Auth4cData;


Auth4cData::Exporter* Factory::_pexporter;
Auth4cData::ProductMap Factory::_productS;
Auth4cData::DomainMap Factory::_domainS;
Auth4cData::ProfileMap Factory::_profileS;
Auth4cData::CustomerMap Factory::_customerS;
Auth4cData::TeamMap Factory::_teamS;
//Auth4cData::UserMap Factory::_userS;
Auth4cData::XTeamMap Factory::_xteamS;
Auth4cData::XTeamPtrAlienMap Factory::_alienS;
Auth4cData::XTeamPtrAlienRevMap Factory::_ralienS;
Auth4cData::PlatformMap Factory::_platformS;
Auth4cData::SharedObjectMap Factory::_sharedObjectS;
Auth4cData::AuthenticationLibraryMap Factory::_authenticationLibraryTemplateS;
Auth4cData::AuthenticationLibraryPtrMap Factory::_authenticationLibraryS;
Auth4cData::AuthenticatorPtrMap Factory::_authenticatorS;
Auth4cData::AuthenticatedUser Factory::_userAuth;
Auth4cData::SessionMap Factory::_sessionS;


Auth4cData::ProductMap& Factory::products()
{
	return _productS;
}

Auth4cData::DomainMap& Factory::domains()
{
	return _domainS;
}

Auth4cData::ProfileMap& Factory::profiles()
{
	return _profileS;
}

Auth4cData::CustomerMap& Factory::customers()
{
	return _customerS;
}

Auth4cData::TeamMap& Factory::teams()
{
	return _teamS;
}

//Auth4cData::UserMap& Factory::users()
//{
//	return _userS;
//}

Auth4cData::XTeamMap& Factory::xteams()
{
	return _xteamS;
}

Auth4cData::XTeamPtrAlienMap& Factory::aliens()
{
	return _alienS;
}

Auth4cData::XTeamPtrAlienRevMap& Factory::raliens()
{
	return _ralienS;
}

Auth4cData::PlatformMap& Factory::platforms()
{
	return _platformS;
}

Auth4cData::SharedObjectMap& Factory::sharedObjects()
{
	return _sharedObjectS;
}

Auth4cData::AuthenticationLibraryMap& Factory::authenticationLibraryTemplates()
{
	return _authenticationLibraryTemplateS;
}

Auth4cData::AuthenticationLibraryPtrMap& Factory::authenticationLibraries()
{
	return _authenticationLibraryS;
}

Auth4cData::AuthenticatorPtrMap& Factory::authenticators()
{
	return _authenticatorS;
}

Auth4cData::Permission& Factory::createRolePermission( Auth4cData::Role& prole, const Auth4cData::Permission& ppermission )
{
	Auth4cData::Permission perm( prole, ppermission.permission(), ppermission.description() );
	
	Auth4cData::PermissionMap& permS( prole.permissions() );
	Auth4cData::PermissionMap::iterator iter = permS.find( ppermission.permission() );
	if( permS.end() == iter )
	{
		permS[ppermission.permission()] = perm;
	}

	iter = permS.find( ppermission.permission() );
	if( permS.end() == iter )
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_PERMISSIONNOTFOUND, "Could not create role permission." );
	}
	return iter->second;
}

Auth4cData::Permission& Factory::createPermission( Auth4cData::Product& pproduct, int nPerm )
{
	Auth4cData::Permission perm( pproduct, nPerm );
	
	Auth4cData::PermissionMap& permS( pproduct.permissions() );
	Auth4cData::PermissionMap::iterator iter = permS.find( perm.permission() );
	if( permS.end() == iter )
	{
		permS[nPerm] = perm;
	}

	iter = permS.find( nPerm );
	if( permS.end() == iter )
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_PERMISSIONNOTFOUND, "Could not create permission." );
	}
	return iter->second;
}

Auth4cData::Role& Factory::createRole( Auth4cData::Product& pproduct, int nRole )
{
	Auth4cData::Role role( pproduct, nRole );
	
	Auth4cData::RoleMap& roleS( pproduct.roles() );
	Auth4cData::RoleMap::iterator iter = roleS.find( role.role() );
	if( roleS.end() == iter )
	{
		roleS[nRole] = role;
	}

	iter = roleS.find( nRole );
	if( roleS.end() == iter )
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_PERMISSIONNOTFOUND, "Could not create role." );
	}
	return iter->second;
}

Auth4cData::Product& Factory::createProduct( int nProduct )
{
	Auth4cData::Product product( nProduct );
	
	Auth4cData::ProductMap& productS( products() );
	Auth4cData::ProductMap::iterator iter = productS.find( nProduct );
	if( productS.end() == iter )
	{
		productS[nProduct] = product;
	}

	iter = productS.find( nProduct );
	if( productS.end() == iter )
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_PERMISSIONNOTFOUND, "Could not create product." );
	}
	return iter->second;
}

Auth4cData::AuthenticatedUser& Factory::authenticate( const Auth4cData::User& user, const AuthenticationToken& authToken, const Auth4cData::Profile& profile ) 
{
	Auth4cData::DomainMap::const_iterator iterDomain = _domainS.find( user.domain().domain() );
	if( _domainS.end() == iterDomain )
	{
		std::ostringstream os;
		os << "No such Domain - " << user.domain().domain() << ".";
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_AUTHENTICATIONFAILED, os.str() );
	}

	Auth4cData::AuthenticatorPtrMap::const_iterator iterAuthenticator = _authenticatorS.find( user.domain().domain() );
	if( _authenticatorS.end() == iterAuthenticator )
	{
		std::ostringstream os;
		os << "No such Authenticator - " << user.domain().domain() << ".";
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_AUTHENTICATIONFAILED, os.str() );
	}

	_userAuth = iterAuthenticator->second->authenticate( user, authToken, profile );

	return _userAuth;
}

Auth4cData::AuthenticatedUser& Factory::authenticatedUser() 
{
	if( !_userAuth.session().str().length() )
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_AUTHENTICATIONFAILED, "No user authenticated." );
	}

	return _userAuth;
}

bool Factory::passwd( const Auth4cData::AuthenticatedUser& user, Auth4cData::AuthenticationToken& authTokenCurrent, Auth4cData::AuthenticationToken& authTokenNew ) 
{
	Auth4cData::DomainMap::const_iterator iterDomain = _domainS.find( user.domain().domain() );
	if( _domainS.end() == iterDomain )
	{
		std::ostringstream os;
		os << "No such Domain - " << user.domain().domain() << ".";
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_AUTHENTICATIONFAILED, os.str() );
	}

	Auth4cData::AuthenticatorPtrMap::const_iterator iterAuthenticator = _authenticatorS.find( user.domain().domain() );
	if( _authenticatorS.end() == iterAuthenticator )
	{
		std::ostringstream os;
		os << "No such Authenticator - " << user.domain().domain() << ".";
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_AUTHENTICATIONFAILED, os.str() );
	}

	iterAuthenticator->second->passwd( user, authTokenCurrent, authTokenNew );

	return true;
}

void Factory::persistSession( Auth4cData::AuthenticatedUser& userP ) 
{
	_sessionS.addSession( userP );
}

Auth4cData::AuthenticatedUser& Factory::retrieveSession( const Auth4cData::Session& session ) 
{
	_userAuth = _sessionS.session( session );
	return _userAuth;
}

void Factory::killSession( const Auth4cData::Session& session ) 
{
	_sessionS.kill( session );
}

void Factory::user( Auth4cData::User& userF, bool bRecurse /*= true*/ ) 
{
	Auth4cData::DomainMap::iterator iterDomain = _domainS.find( userF.domain().domain() );
	if( _domainS.end() == iterDomain )
	{
		throw Exception( __FILE__, __LINE__, nNAX_DOMAINNOTFOUND, userF.domain().domain() );
	}

	iterDomain->second.authenticator().user( userF, bRecurse );
}

void Factory::team( Auth4cData::Team& teamF, bool bReload ) 
{
	TeamMap::iterator iterTeam = _teamS.find( teamF.team() );
	if( _teamS.end() == iterTeam )
	{
		std::ostringstream os;
		os << teamF.team();
		throw Exception( __FILE__, __LINE__, nNAX_TEAMNOTFOUND, os.str() );
	}

	teamF = iterTeam->second;

	if( ( bReload ) ||
		( !teamF.members().size() ) )
	{
		teamF.loadMembers();
	}
}

const XTeamPtrMap& Factory::teams( const AlienKey& alien )
{
	static XTeamPtrMap empty;

	XTeamPtrAlienMap::const_iterator iter = _alienS.find( alien );
	if( _alienS.end() != iter )
		return iter->second;

	return empty;
}

const XTeamPtrMap& Factory::teams( const Team& team )
{
	static XTeamPtrMap empty;

	XTeamPtrAlienRevMap::const_iterator iter = _ralienS.find( team.team() );
	if( _ralienS.end() != iter )
		return iter->second;

	return empty;
}

void Factory::mergeRolePermissionsIntoPermissions( ProductMap& productS )
{
	PRStatus sGranted( PRStatus::statusGranted );

	for( ProductMap::iterator iterProduct = productS.begin();
			iterProduct != productS.end();
			++iterProduct )
	{
		for( RoleMap::const_iterator iterRole = iterProduct->second.roles().begin();
				iterRole != iterProduct->second.roles().end();
				++iterRole )
		{
			if( iterRole->second.status() == sGranted )
			{
				iterProduct->second.permissions().merge( iterRole->second.permissions() );
			}
		}
	}
}

Auth4cData::Exporter* Factory::exporter()
{
	return _pexporter;
}

void Factory::setExporter( Auth4cData::Exporter* pexporter )
{
	_pexporter = pexporter;
}

Base4c::Version Factory::version()
{
	return Base4c::Version( AUTH4CVERSION );
}

void Factory::setAuthenticatedUser( const AuthenticatedUser& user )
{
	_userAuth = user;
}

bool Factory::hasProductPermission()
{
	Auth4cData::Permission permission( _productS[A4CAuth4C::nA4CD_Auth4c], A4CAuth4C::nA4CP_Maintain_Products );

	return _userAuth.hasPermission( permission );
}
