/*

  $Id: PRStatus.cpp 387 2015-12-08 18:53:52Z john $

  Copyright (c) 2008, The 4c Company. All rights reserved.
*/

#include <Auth4cData/PRStatus.h>

using namespace Auth4cData;

//////////////////////////////////////////////////////////////////////////
//
// PRStatus
//

PRStatus::PRStatus( PRStatusRaw status /*= statusDenied*/ )
	: _status( status )
{
}

PRStatus::PRStatus( const PRStatus& status )
	: _status( status._status )
{
}

PRStatus::~PRStatus()
{
}

const PRStatus& PRStatus::operator=( const PRStatus& status )
{
	if( this != &status )
	{
		_status = status._status;
	}

	return *this;
}

PRStatus PRStatus::operator+( const PRStatus& status ) const
{
	PRStatus rtn( *this );
	rtn += status;
	return rtn;
}

const PRStatus& PRStatus::operator+=( const PRStatus& status )
{
	switch( _status )
	{
		case statusRevoked:
			// Do nothing, it's already revoked
			break;

		case statusDenied:
		case statusGranted:
		{
			switch( status._status )
			{
				case statusRevoked:
					_status = statusRevoked;
					break;

				case statusDenied:
					// No change
					break;

				case statusGranted:
					_status = statusGranted;
					break;
			}
		}
	}

	return *this;
}

bool PRStatus::operator==( const PRStatus& status ) const
{
	return _status == status._status;
}

PRStatus::operator PRStatusRaw() const
{
	return _status;
}

PRStatus PRStatus::fromInt( int nStatus )
{
	PRStatus rtn = PRStatus::statusDenied;

	switch( nStatus )
	{
		case -1:
			rtn = PRStatus::statusRevoked;
			break;

		case 0:
			rtn = PRStatus::statusDenied;
			break;

		case 1:
			rtn = PRStatus::statusGranted;
			break;
	}

	return rtn;
}
