/*

  $Id: Permission.cpp 689 2018-11-28 17:54:35Z john $

  Copyright (c) 2008, The 4c Company. All rights reserved.
*/

#include <Auth4cData/Permission.h>

#include <sstream>

using namespace Auth4cData;

Permission::Permission()
	: _pproduct( NULL ),
	_prole( NULL ),
	_nPermission( -1 ),
	_status( PRStatus::statusDenied )
{
}

Permission::Permission( const Product& product, int nPermission, const std::string& strDescription, PRStatus status /*= statusDenied*/ )
	: _pproduct( &product ),
	_prole( NULL ),
	_nPermission( nPermission ),
	_strDescription( strDescription ),
	_status( status )
{
}

Permission::Permission( const Role& role, int nPermission, const std::string& strDescription, PRStatus status /*= statusDenied*/ )
	: _pproduct( NULL ),
	_prole( &role ),
	_nPermission( nPermission ),
	_strDescription( strDescription ),
	_status( status )
{
}

Permission::Permission( const Permission& permission )
	: _pproduct( permission._pproduct ),
	_prole( permission._prole ),
	_nPermission( permission._nPermission ),
	_strDescription( permission._strDescription ),
	_status( permission._status )
{
}

Permission::~Permission()
{
}

const Permission& Permission::operator=( const Permission& permission )
{
	if( this != &permission )
	{
		_pproduct = permission._pproduct;
		_prole = permission._prole;
		_nPermission = permission._nPermission;
		_strDescription = permission._strDescription;
		_status = permission._status;
	}

	return *this;
}

const Product& Permission::product() const
{
	if( NULL == _pproduct )
	{
		std::ostringstream os;
		os << "Permission " << _nPermission << " - " << _strDescription;
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_NOPRODUCTSET, os.str() );
	}

	return *_pproduct;
}

bool Permission::hasRole()
{
	return NULL != _prole;
}

const Role& Permission::role() const
{
	if( NULL == _prole )
	{
		std::ostringstream os;
		os << "Permission " << _nPermission << " - " << _strDescription;
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_NOROLESET, os.str() );
	}

	return *_prole;
}

int Permission::permission() const
{
	return _nPermission;
}

const std::string& Permission::description() const
{
	return _strDescription;
}

bool Permission::hasPermission() const
{
	return ( PRStatus::statusGranted == _status );
}

const PRStatus& Permission::status() const
{
	return _status;
}

PRStatus& Permission::status()
{
	return _status;
}

void Permission::setDescription( const std::string& strDescription )
{
	_strDescription = strDescription;
}

void Permission::setStatus( const PRStatus& status )
{
	_status = status;
}

void Permission::resetProduct( const Product& product )
{
	_pproduct = &product;
}


//////////////////////////////////////////////////////////////////////////
//
// PermissionMap
//

void PermissionMap::merge( const PermissionMap& map )
{
	for( PermissionMap::const_iterator iterTheirs = map.begin();
			iterTheirs != map.end();
			++iterTheirs )
	{
		merge( iterTheirs->second );
	}
}

void PermissionMap::merge( const Permission& permission )
{
	PermissionMap::iterator iterMine = find( permission.permission() );
	if( end() != iterMine )
	{
		iterMine->second.status() += permission.status();
	}
}
