/*

  $Id: Product.cpp 403 2015-12-10 14:13:42Z john $

  Copyright (c) 2008, The 4c Company. All rights reserved.
*/

#include <Auth4cData/Product.h>

using namespace Auth4cData;

Product::Product( int nProduct /*= -1*/ )
	: _nProduct( nProduct )
{
}

Product::Product( int nProduct,
					const InitMapMap& initProduct,
					const std::string& strDescription /*= std::string()*/ )
	: _nProduct( nProduct ),
	_strDescription( strDescription ),
	_initProduct( initProduct )
{
}

Product::Product( const Product& product )
	: _nProduct( product._nProduct ),
	_strDescription( product._strDescription ),
	_initProduct( product._initProduct ),
	_roleS( product._roleS ),
	_permissionS( product._permissionS )
{
	resetPR();
}

Product::~Product()
{
}

const Product& Product::operator=( const Product& product )
{
	if( this != &product )
	{
		_nProduct = product._nProduct;
		_strDescription = product._strDescription;
		_initProduct = product._initProduct;
		_roleS = product._roleS;
		_permissionS = product._permissionS;

		resetPR();
	}

	return *this;
}

int Product::product() const
{
	return _nProduct;
}

const std::string& Product::description() const
{
	return _strDescription;
}

const InitMapMap& Product::productInit() const
{
	return _initProduct;
}

const RoleMap& Product::roles() const
{
	return _roleS;
}

const PermissionMap& Product::permissions() const
{
	return _permissionS;
}

void Product::setDescription( const std::string& strDescription )
{
	_strDescription = strDescription;
}

void Product::setProductInit( const InitMapMap& initProduct )
{
	_initProduct = initProduct;
}

void Product::updateInitMap( const std::string& strProfile, const std::string& strOldParam, const std::string& strName, const std::string& strValue, bool bAddNew /*=true*/ )
{
	InitMapMap::iterator iterMM = _initProduct.find( strProfile );
	if( _initProduct.end() == iterMM )
	{
		if( bAddNew )
		{
			_initProduct[strProfile] = InitMap();
			iterMM = _initProduct.find( strProfile );
		}
		else
		{
			return;
		}
	}
	
	InitMap& initProfile( iterMM->second );
	
	if( strOldParam != strName )
	{
		Auth4cData::InitMap::const_iterator iter = initProfile.find( strOldParam );
		if( initProfile.end() != iter )
		{
			initProfile.erase( strOldParam );
		}
	}
	if( bAddNew )
	{
		initProfile[strName] = strValue;
	}
}

RoleMap& Product::roles()
{
	return _roleS;
}

PermissionMap& Product::permissions()
{
	return _permissionS;
}

bool Product::hasRole( const Role& role ) const
{
	RoleMap::const_iterator iter = _roleS.find( role.role() );
	if( _roleS.end() == iter )
	{
		return false;
	}

	return iter->second.hasRole();
}

bool Product::hasPermission( const Permission& permission ) const
{
	PermissionMap::const_iterator iter = _permissionS.find( permission.permission() );
	if( _permissionS.end() == iter )
	{
		return false;
	}

	return iter->second.hasPermission();
}

void Product::resetPR()
{
	for( Auth4cData::PermissionMap::iterator iterPermission = _permissionS.begin();
			iterPermission != _permissionS.end();
			++iterPermission )
	{
		iterPermission->second.resetProduct( *this );
	}

	for( Auth4cData::RoleMap::iterator iterRole = _roleS.begin();
			iterRole != _roleS.end();
			++iterRole )
	{
		iterRole->second.resetProduct( *this );
	}
}
