/*

  $Id: Role.cpp 689 2018-11-28 17:54:35Z john $

  Copyright (c) 2008, The 4c Company. All rights reserved.
*/

#include <Auth4cData/Role.h>
#include <Auth4cData/Exception.h>

#include <sstream>

using namespace Auth4cData;

Role::Role()
	: _pproduct( NULL ),
	_nRole( -1 ),
	_status( PRStatus::statusDenied )
{
}

Role::Role( const Product& product, int nRole, const std::string& strDescription, PRStatus status /*= statusDenied*/ )
	: _pproduct( &product ),
	_nRole( nRole ),
	_strDescription( strDescription ),
	_status( status )
{
}

Role::Role( const Role& role )
	: _pproduct( role._pproduct ),
	_nRole( role._nRole ),
	_strDescription( role._strDescription ),
	_status( role._status ),
	_permissionS( role._permissionS )
{
}

Role::~Role()
{
}

const Role& Role::operator=( const Role& role )
{
	if( this != &role )
	{
		_pproduct = role._pproduct;
		_nRole = role._nRole;
		_strDescription = role._strDescription;
		_status = role._status;
		_permissionS = role._permissionS;
	}

	return *this;
}

const Product& Role::product() const
{
	if( NULL == _pproduct )
	{
		std::ostringstream os;
		os << "Role " << _nRole << " - " << _strDescription;
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_NOPRODUCTSET, os.str() );
	}

	return *_pproduct;
}

int Role::role() const
{
	return _nRole;
}

const std::string& Role::description() const
{
	return _strDescription;
}

bool Role::hasRole() const
{
	return ( PRStatus::statusGranted == _status );
}

const PRStatus& Role::status() const
{
	return _status;
}

PRStatus& Role::status()
{
	return _status;
}

const PermissionMap& Role::permissions() const
{
	return _permissionS;
}

void Role::setDescription( const std::string& strDescription )
{
	_strDescription = strDescription;
}

void Role::setStatus( const PRStatus& status )
{
	_status = status;
}

PermissionMap& Role::permissions()
{
	return _permissionS;
}

void Role::resetProduct( const Product& product )
{
	_pproduct = &product;
}


//////////////////////////////////////////////////////////////////////////
//
// RoleMap
//

void RoleMap::merge( const RoleMap& map )
{
	for( RoleMap::const_iterator iterTheirs = map.begin();
			iterTheirs != map.end();
			++iterTheirs )
	{
		merge( iterTheirs->second );
	}
}

void RoleMap::merge( const Role& role )
{
	RoleMap::iterator iterMine = find( role.role() );
	if( end() != iterMine )
	{
		iterMine->second.status() += role.status();
	}
}
