/*

  $Id: SessionMap.cpp 689 2018-11-28 17:54:35Z john $

  Copyright (c) 2008, The 4c Company. All rights reserved.
*/

#include <Auth4cData/SessionMap.h>

using namespace Auth4cData;

void SessionMap::addSession( const AuthenticatedUser& user ) 
{
	iterator iter = find( user.session().str() );
	if( end() != iter )
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_BADSESSION, "Session already exists." );
	}

	operator[]( user.session().str() ) = user;
}

const AuthenticatedUser& SessionMap::session( const Session& session ) 
{
	iterator iter = find( session.str() );

	if( end() == iter )
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_BADSESSION, "No such session." );
	}

	try
	{
		iter->second.session().access();
	}
	catch( Auth4cData::Exception& e )
	{
		if( Auth4cData::nNAX_BADSESSION == e.errorCode() )
		{
			erase( iter );
			throw;
		}
	}

	return iter->second;
}

void SessionMap::kill( const Session& session ) 
{
	iterator iter = find( session.str() );

	if( end() == iter )
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_BADSESSION, "No such session." );
	}

	erase( iter );
}

void SessionMap::cleanUp()
{
}
