/*

  $Id: SharedObject.cpp 689 2018-11-28 17:54:35Z john $

  Copyright (c) 2008, The 4c Company. All rights reserved.
*/

#include <Auth4cData/SharedObject.h>

#include <Auth4cData/Platform.h>
#include <Auth4cData/AuthenticationLibrary.h>

using namespace Auth4cData;

SharedObject::SharedObject()
	: _pplatform( NULL ),
	_pauthenticationLibrary( NULL )
{
}

SharedObject::SharedObject( const Platform& platform, const AuthenticationLibrary& authenticationLibrary, const std::string& strSharedObject )
	: _pplatform( &platform ),
	_pauthenticationLibrary( &authenticationLibrary ),
	_strSharedObject( strSharedObject )
{
	_key = std::make_pair( _pplatform->platform(), _pauthenticationLibrary->authenticationLibrary() );
}

SharedObject::SharedObject( const SharedObject& object )
	: _pplatform( object._pplatform ),
	_pauthenticationLibrary( object._pauthenticationLibrary ),
	_strSharedObject( object._strSharedObject ),
	_key( object._key )
{
}

SharedObject::~SharedObject()
{
}

const SharedObject& SharedObject::operator=( const SharedObject& object )
{
	if( this != &object )
	{
		_pplatform = object._pplatform;
		_pauthenticationLibrary = object._pauthenticationLibrary;
		_strSharedObject = object._strSharedObject;
		_key = object._key;
	}

	return *this;
}

const SharedObject::Key& SharedObject::key() const
{
	return _key;
}

const Platform& SharedObject::platform() const
{
	if( NULL == _pplatform )
	{
		throw Exception( __FILE__, __LINE__, nNAX_NOPLATFORMSET, "No platform set" );
	}

	return *_pplatform;
}

const AuthenticationLibrary& SharedObject::authenticationLibrary() const
{
	if( NULL == _pauthenticationLibrary )
	{
		throw Exception( __FILE__, __LINE__, nNAX_NOAUTHENTICATIONLIBRARYSET, "No authenticationlibrary set" );
	}

	return *_pauthenticationLibrary;
}

const std::string& SharedObject::sharedObject() const
{
	return _strSharedObject;
}
