/*

  $Id: Team.cpp 689 2018-11-28 17:54:35Z john $

  Copyright (c) 2008, The 4c Company. All rights reserved.
*/

#include <Auth4cData/Team.h>

#include <Auth4cData/User.h>
#include <Auth4cData/Authenticator.h>
#include <Auth4cData/Customer.h>

#include <sstream>

using namespace Auth4cData;

Team::Team()
	: _pcustomer( NULL )
{
}

Team::Team( const Customer& customer, int nTeam, const std::string& strName )
	: _nTeam( nTeam ),
	_pcustomer( &customer ),
	_strName( strName )
{
}

Team::Team( const Team& team )
	:_nTeam( team._nTeam ),
	_pcustomer( team._pcustomer ),
	_strName( team._strName ),
	_memberS( team._memberS ),
	_productS( team._productS )
{
}

Team::~Team()
{
}

const Team& Team::operator=( const Team& team )
{
	if( this != &team )
	{
		_pcustomer = team._pcustomer;
		_nTeam = team._nTeam;
		_strName = team._strName;
		_memberS = team._memberS;
		_productS = team._productS;
	}

	return *this;
}

int Team::team() const
{
	return _nTeam;
}

const Customer& Team::customer() const
{
	if( NULL == _pcustomer )
	{
		throw Exception( __FILE__, __LINE__, nNAX_NOCUSTOMERSET, "The customer has not been set." );
	}

	return *_pcustomer;
}

const std::string& Team::name() const
{
	return _strName;
}

const TeamMemberMap& Team::members() const
{
	return _memberS;
}

const ProductMap& Team::products() const
{
	return _productS;
}

void Team::setCustomer( const Customer& customer )
{
	_pcustomer = &customer;
}

void Team::setName( const std::string& strName )
{
	_strName = strName;
}

ProductMap& Team::products()
{
	return _productS;
}

TeamMemberMap& Team::members()
{
	return _memberS;
}

void Team::loadMembers()
{
	_memberS.clear();

	for( DomainPtrMap::const_iterator iter = customer().domains().begin();
			iter != customer().domains().end();
			++iter )
	{
		iter->second->authenticator().team( *this );
	}
}
