/*

  $Id: Auth4cFactory.cpp 699 2018-12-11 17:51:05Z john $

  Copyright (c) 2008, The 4c Company. All rights reserved.
*/

#include <Auth4cDataDB/Auth4cFactory.h>

#include <Auth4cDal/Auth4cDB.h>

#include <Auth4cDal/ProductQuery.h>
#include <Auth4cDal/ProductParamQuery.h>
#include <Auth4cDal/ProductRoleQuery.h>
#include <Auth4cDal/ProductPermissionQuery.h>
#include <Auth4cDal/ProductRolePermissionQuery.h>
#include <Auth4cDal/DomainQuery.h>
#include <Auth4cDal/ProfileQuery.h>
#include <Auth4cDal/ProfileHostQuery.h>
#include <Auth4cDal/CustomerQuery.h>
#include <Auth4cDal/UserQuery.h>
#include <Auth4cDal/TeamQuery.h>
#include <Auth4cDal/TeamMapQuery.h>
#include <Auth4cDal/UserTeamQuery.h>
#include <Auth4cDal/AuthenticationLibraryXQuery.h>
#include <Auth4cDal/TeamProductRoleQuery.h>
#include <Auth4cDal/TeamProductPermissionQuery.h>
#include <Auth4cDal/GenericProductRoleQuery.h>
#include <Auth4cDal/GenericProductPermissionQuery.h>
#include <Auth4cDal/PlatformQuery.h>
#include <Auth4cDal/SharedObjectQuery.h>
#include <Auth4cDal/AuthenticationLibraryParamQuery.h>
#include <Auth4cDal/AuthenticatorParamQuery.h>
#include <Auth4cDal/ProductParamQuery.h>

#include <MakeVersion.in>

#include <Base4c/EventLogger.h>

#include <librfcdal/SqlInteger.h>
#include <librfcdal/SqlText.h>
#include <librfcdal/SqlNull.h>

#include <sstream>

#if defined (LINUX)
#	include <dlfcn.h>
#	include <uuid/uuid.h>
#endif /* LINUX */

using namespace Auth4cDataDB;

Auth4cData::Factory::Mode Factory::_mode;

void Factory::initialize( const std::string& strConnect, Mode mode )
{
	_mode = mode;

	try
	{
		Auth4cDal::connect( strConnect );

		RfcDal::DalDatabase::Result result;
		RfcDal::DalDatabase::Row row;

		Auth4cDal::ProductQuery qryProduct( Auth4cDal::schema() );
		qryProduct.select( result, Auth4cDal::connection() );
		for( RfcDal::DalDatabase::Result::const_iterator iter = result.begin();
				iter != result.end();
				++iter )
		{
			row = *iter;
			productFromRow( row );
		}

		result.clear();

		Auth4cDal::ProfileQuery qryProfile( Auth4cDal::schema() );
		qryProfile.select( result, Auth4cDal::connection() );
		for( RfcDal::DalDatabase::Result::const_iterator iter = result.begin();
				iter != result.end();
				++iter )
		{
			row = *iter;
			profileFromRow( row );
		}

		result.clear();

		Auth4cDal::CustomerQuery qryCustomer( Auth4cDal::schema() );
		qryCustomer.select( result, Auth4cDal::connection() );
		for( RfcDal::DalDatabase::Result::const_iterator iter = result.begin();
				iter != result.end();
				++iter )
		{
			row = *iter;
			customerFromRow( row );
		}

		result.clear();

		if( ( modeAuthenticator == mode ) ||
			( modeAll == mode ) )
		{
			Auth4cDal::PlatformQuery qryPlatform( Auth4cDal::schema() );
			qryPlatform.select( result, Auth4cDal::connection() );
			for( RfcDal::DalDatabase::Result::const_iterator iter = result.begin();
					iter != result.end();
					++iter )
			{
				row = *iter;
				try
				{
					platformFromRow( row );
				}
				catch( const Auth4cData::Exception& e )
				{
					Base4c::EventLogger::theLogger()->logError( e.legend(), Base4c::EventLogger::daemon );
				}
			}

			result.clear();

/*		Auth4cDal::AuthenticationLibraryXQuery qryAuthenticationLibrary( Auth4cDal::schema() );
#if defined (LINUX)
			qryAuthenticationLibrary.constrainPlatform( RfcDal::SqlText( "LNX" ) );
#elif defined (_WINDOWS)
			qryAuthenticationLibrary.constrainPlatform( RfcDal::SqlText( "WIN" ) );
#endif LINUX */

			Auth4cDal::AuthenticationLibraryQuery qryAuthenticationLibrary( Auth4cDal::schema() );
			qryAuthenticationLibrary.select( result, Auth4cDal::connection() );
			for( RfcDal::DalDatabase::Result::const_iterator iter = result.begin();
					iter != result.end();
					++iter )
			{
				row = *iter;
				try
				{
					authenticationLibraryFromRow( row );
				}
				catch( const Auth4cData::Exception& e )
				{
					Base4c::EventLogger::theLogger()->logError( e.legend(), Base4c::EventLogger::daemon );
				}
			}

			result.clear();

			Auth4cDal::SharedObjectQuery qrySharedObject( Auth4cDal::schema() );
			qrySharedObject.select( result, Auth4cDal::connection() );
			for( RfcDal::DalDatabase::Result::const_iterator iter = result.begin();
					iter != result.end();
					++iter )
			{
				row = *iter;
				try
				{
					sharedObjectFromRow( row );
				}
				catch( const Auth4cData::Exception& e )
				{
					Base4c::EventLogger::theLogger()->logError( e.legend(), Base4c::EventLogger::daemon );
				}
			}

			if( modeAuthenticator == mode )
			{
				for( Auth4cData::AuthenticationLibraryMap::iterator iter = Auth4cData::Factory::authenticationLibraryTemplates().begin();
						iter != Auth4cData::Factory::authenticationLibraryTemplates().end();
						++iter )
				{
					try
					{
						iter->second.load();
					}
					catch( const RfcDal::DalDatabaseException& /*e*/ )
					{
					}
					catch( const Auth4cData::Exception& e )
					{
						Base4c::EventLogger::theLogger()->logError( e.legend(), Base4c::EventLogger::daemon );
					}
				}
			}
		}

		result.clear();

		Auth4cDal::DomainQuery qryDomain( Auth4cDal::schema() );
		qryDomain.select( result, Auth4cDal::connection() );
		for( RfcDal::DalDatabase::Result::const_iterator iter = result.begin();
				iter != result.end();
				++iter )
		{
			row = *iter;
			try
			{
				domainFromRow( row );
			}
			catch( const Auth4cData::Exception& e )
			{
				Base4c::EventLogger::theLogger()->logError( e.legend(), Base4c::EventLogger::daemon );
			}
		}

		result.clear();

		Auth4cDal::TeamQuery qryTeam( Auth4cDal::schema() );
		qryTeam.select( result, Auth4cDal::connection() );
		for( RfcDal::DalDatabase::Result::const_iterator iter = result.begin();
				iter != result.end();
				++iter )
		{
			try
			{
				row = *iter;
				teamFromRow( row );
			}
			catch( const Auth4cData::Exception& e )
			{
				Base4c::EventLogger::theLogger()->logError( e.legend(), Base4c::EventLogger::daemon );
			}
		}

		result.clear();

		Auth4cDal::TeamMapQuery qryMap( Auth4cDal::schema() );
		qryMap.select( result, Auth4cDal::connection() );
		for( RfcDal::DalDatabase::Result::const_iterator iter = result.begin();
				iter != result.end();
				++iter )
		{
			row = *iter;
			teamMapFromRow( row );
		}

		result.clear();

		if( modeAdmin == mode )
//			( modeAll == mode ) )
		{
			Auth4cDal::UserQuery qryUser( Auth4cDal::schema() );
			qryUser.select( result, Auth4cDal::connection() );
			for( RfcDal::DalDatabase::Result::const_iterator iter = result.begin();
					iter != result.end();
					++iter )
			{
				row = *iter;
				userFromRow( row );
			}
		}

		result.clear();

		for( Auth4cData::TeamMap::iterator iterTeam = _teamS.begin();
				iterTeam != _teamS.end();
				++iterTeam )
		{
		}
	}
	catch( const RfcDal::DalDatabaseException& e )
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_ALIENEXCEPTION, e.nativeExplanation(), true );
	}
}

void Factory::reinitialize()
{
	try
	{
		RfcDal::DalDatabase::Result result;
		RfcDal::DalDatabase::Row row;

		_productS.clear();

		Auth4cDal::ProductQuery qryProduct( Auth4cDal::schema() );
		qryProduct.select( result, Auth4cDal::connection() );
		for( RfcDal::DalDatabase::Result::const_iterator iter = result.begin();
				iter != result.end();
				++iter )
		{
			row = *iter;
			productFromRow( row );
		}

		result.clear();

		_profileS.clear();

		Auth4cDal::ProfileQuery qryProfile( Auth4cDal::schema() );
		qryProfile.select( result, Auth4cDal::connection() );
		for( RfcDal::DalDatabase::Result::const_iterator iter = result.begin();
				iter != result.end();
				++iter )
		{
			row = *iter;
			profileFromRow( row );
		}

		result.clear();

		_customerS.clear();

		Auth4cDal::CustomerQuery qryCustomer( Auth4cDal::schema() );
		qryCustomer.select( result, Auth4cDal::connection() );
		for( RfcDal::DalDatabase::Result::const_iterator iter = result.begin();
				iter != result.end();
				++iter )
		{
			row = *iter;
			customerFromRow( row );
		}

		result.clear();

		_domainS.clear();

		Auth4cDal::DomainQuery qryDomain( Auth4cDal::schema() );
		qryDomain.select( result, Auth4cDal::connection() );
		for( RfcDal::DalDatabase::Result::const_iterator iter = result.begin();
				iter != result.end();
				++iter )
		{
			row = *iter;
			try
			{
				domainFromRow( row );
			}
			catch( const Auth4cData::Exception& e )
			{
				Base4c::EventLogger::theLogger()->logError( e.legend(), Base4c::EventLogger::daemon );
			}
		}

		result.clear();

		_teamS.clear();

		Auth4cDal::TeamQuery qryTeam( Auth4cDal::schema() );
		qryTeam.select( result, Auth4cDal::connection() );
		for( RfcDal::DalDatabase::Result::const_iterator iter = result.begin();
				iter != result.end();
				++iter )
		{
			try
			{
				row = *iter;
				teamFromRow( row );
			}
			catch( const Auth4cData::Exception& e )
			{
				Base4c::EventLogger::theLogger()->logError( e.legend(), Base4c::EventLogger::daemon );
			}
		}

		result.clear();

		aliens().clear();
		raliens().clear();

		Auth4cDal::TeamMapQuery qryMap( Auth4cDal::schema() );
		qryMap.select( result, Auth4cDal::connection() );
		for( RfcDal::DalDatabase::Result::const_iterator iter = result.begin();
				iter != result.end();
				++iter )
		{
			row = *iter;
			teamMapFromRow( row );
		}

		result.clear();

		if( modeAdmin == _mode )
		{
			Auth4cDal::UserQuery qryUser( Auth4cDal::schema() );
			qryUser.select( result, Auth4cDal::connection() );
			for( RfcDal::DalDatabase::Result::const_iterator iter = result.begin();
					iter != result.end();
					++iter )
			{
				row = *iter;
				userFromRow( row );
			}
		}

		result.clear();
	}
	catch( const RfcDal::DalDatabaseException& e )
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_ALIENEXCEPTION, e.nativeExplanation(), true );
	}
}

Auth4cData::AuthenticatedUser Factory::authenticate( const Auth4cData::User& user, const Auth4cData::AuthenticationToken& authToken, const Auth4cData::Profile& profile )
{
	Auth4cData::AuthenticatedUser userA = Auth4cData::Factory::authenticate( user, authToken, profile );

	authorize( userA );

	return userA;
}

void Factory::persistSession( Auth4cData::AuthenticatedUser& userP )
{
	Auth4cData::Factory::persistSession( userP );
}

Auth4cData::AuthenticatedUser& Factory::retrieveSession( const Auth4cData::Session& session )
{
	return Auth4cData::Factory::retrieveSession( session );
}

//void Factory::authorize( Auth4cData::AuthenticatedUser& user )
void Factory::authorize( Auth4cData::User& user )
{
	Auth4cData::ProductMap& productS( user.products() );
	RfcDal::DalDatabase::Result result;
	Auth4cDal::Connection connection( Auth4cDal::connection() );

	try
	{
		RfcDal::SqlTable tbCPR( Auth4cDal::schema(), "CustomerProductRole", "PR" );
		Auth4cDal::GenericProductRoleQuery qryCPR( tbCPR );
		qryCPR.constrainCustomer( RfcDal::SqlText( user.domain().customer().customer() ) );
		qryCPR.select( result, connection );

		Factory::productRoleMapFromResult( productS, result );
	}
	catch( const RfcDal::DalDatabaseException& e )
	{
		Base4c::EventLogger::theLogger()->logError( e.nativeExplanation(), Base4c::EventLogger::data );
	}

	result.clear();

	try
	{
		RfcDal::SqlTable tbCPP( Auth4cDal::schema(), "CustomerProductPermission", "PP" );
		Auth4cDal::GenericProductPermissionQuery qryCPP( tbCPP );
		qryCPP.constrainCustomer( RfcDal::SqlText( user.domain().customer().customer() ) );
		qryCPP.select( result, connection );

		Factory::productPermissionMapFromResult( productS, result );
	}
	catch( const RfcDal::DalDatabaseException& e )
	{
		Base4c::EventLogger::theLogger()->logError( e.nativeExplanation(), Base4c::EventLogger::data );
	}

	try
	{
		for( Auth4cData::TeamMap::const_iterator iterTeam = user.teams().begin();
				iterTeam != user.teams().end();
				++iterTeam )
		{
			result.clear();

			RfcDal::SqlTable tbTPR( Auth4cDal::schema(), "TeamProductRole", "PR" );
			Auth4cDal::GenericProductRoleQuery qryTPR( tbTPR );
			qryTPR.constrainTeam( RfcDal::SqlInteger( iterTeam->second.team() ) );
			qryTPR.select( result, connection );

			Factory::productRoleMapFromResult( productS, result );

			result.clear();

			RfcDal::SqlTable tbTPP( Auth4cDal::schema(), "TeamProductPermission", "PP" );
			Auth4cDal::GenericProductPermissionQuery qryTPP( tbTPP );
			qryTPP.constrainTeam( RfcDal::SqlInteger( iterTeam->second.team() ) );
			qryTPP.select( result, connection );

			Factory::productPermissionMapFromResult( productS, result );
		}
	}
	catch( const RfcDal::DalDatabaseException& e )
	{
		Base4c::EventLogger::theLogger()->logError( e.nativeExplanation(), Base4c::EventLogger::data );
	}

	result.clear();

	for( Auth4cData::ProductMap::const_iterator iterProduct = productS.begin();
			iterProduct != productS.end();
			++iterProduct )
	{
		std::cout << "Product <" << iterProduct->second.product() << ">\n";
	}

	try
	{
		RfcDal::SqlTable tbUPR( Auth4cDal::schema(), "UserProductRole", "PR" );
		Auth4cDal::GenericProductRoleQuery qryUPR( tbUPR );
		qryUPR.constrainDomain( RfcDal::SqlText( user.domain().domain() ) );
		qryUPR.constrainUserId( RfcDal::SqlText( user.userId() ) );
		qryUPR.select( result, connection );

		Factory::productRoleMapFromResult( productS, result );
	}
	catch( const RfcDal::DalDatabaseException& e )
	{
		Base4c::EventLogger::theLogger()->logError( e.nativeExplanation(), Base4c::EventLogger::data );
	}

	result.clear();

	try
	{
		RfcDal::SqlTable tbUPP( Auth4cDal::schema(), "UserProductPermission", "PP" );
		Auth4cDal::GenericProductPermissionQuery qryUPP( tbUPP );
		qryUPP.constrainDomain( RfcDal::SqlText( user.domain().domain() ) );
		qryUPP.constrainUserId( RfcDal::SqlText( user.userId() ) );
		qryUPP.select( result, connection );

		Factory::productPermissionMapFromResult( productS, result );
	}
	catch( const RfcDal::DalDatabaseException& e )
	{
		Base4c::EventLogger::theLogger()->logError( e.nativeExplanation(), Base4c::EventLogger::data );
	}

	Auth4cData::Factory::mergeRolePermissionsIntoPermissions( productS );
}

void Factory::user( Auth4cData::User& userF, bool bRecurse )
{
	Auth4cData::Factory::user( userF, bRecurse );

	authorize( userF );
}

Auth4cData::ProfileMap& Factory::profiles()
{
	return Auth4cData::Factory::profiles();
}

Auth4cData::ProfileMap Factory::profiles( const std::string& strHost )
{
	Auth4cDal::ProfileHostQuery qryProfile( Auth4cDal::schema() );

	qryProfile.constrainHost( RfcDal::SqlText( strHost ) );

	RfcDal::DalDatabase::Result result;

	qryProfile.select( result, Auth4cDal::connection() );

	Auth4cData::ProfileMap rtn;

	for( RfcDal::DalDatabase::Result::iterator iter = result.begin();
			iter != result.end();
			++iter )
	{
		Auth4cData::Profile profileN( Auth4cDataDB::Factory::profileFromRowNI( *iter ) );
		rtn[profileN.profile()] = profileN;
	}

	for( Auth4cData::ProfileMap::iterator iterProfileS = profiles().begin();
			iterProfileS != profiles().end();
			++iterProfileS )
	{
		rtn[iterProfileS->second.profile()] = iterProfileS->second;
	}

	return rtn;
}

Auth4cData::Product& Factory::productFromRow( RfcDal::DalDatabase::Row& row )
{
	static Auth4cData::Product productError( -1 );

	Auth4cDal::ProductQuery qryProduct( Auth4cDal::schema() );

	RfcDal::DalDatabase::Result result0;
	RfcDal::DalDatabase::Result result1;
	RfcDal::DalDatabase::Row row1;
	RfcDal::DalDatabase::Row::const_iterator column0;
	RfcDal::DalDatabase::Row::const_iterator column1;

	column0 = row.find( qryProduct.columnProduct().id() );
	if( column0 != row.end() )
	{
		Auth4cData::Product productTemp( column0->second.asInt() );

		column0 = row.find( qryProduct.columnDescription().id() );
		if( column0 != row.end() )
		{
			productTemp.setDescription( column0->second.string() );
		}

		try
		{
			RfcDal::DalDatabase::Result resultParam;
			Auth4cDal::ProductParamQuery qryParam( Auth4cDal::schema() );
			qryParam.constrainProduct( RfcDal::SqlInteger( productTemp.product() ) );
			qryParam.select( resultParam, Auth4cDal::connection() );
			Auth4cData::InitMapMap mapInit( initMapMapFromResult( resultParam ) );

			if( mapInit.size() )
			{
				productTemp.setProductInit( mapInit );
			}
		}
		catch( const RfcDal::DalDatabaseException& e )
		{
			// Backward compatibility, table may not exist
		}

		products()[productTemp.product()] = productTemp;

		Auth4cData::Product& product( products()[productTemp.product()] );

// Permissions
		Auth4cDal::ProductPermissionQuery qryPermission( Auth4cDal::schema() );
		qryPermission.constrainProduct( RfcDal::SqlInteger( product.product() ) );
		qryPermission.select( result1, Auth4cDal::connection() );

		Auth4cData::PermissionMap permissionS = permissionMapFromResult( product, result1 );
		product.permissions() = permissionS;

		result1.clear();

// Roles
		Auth4cDal::ProductRoleQuery qryRole( Auth4cDal::schema() );
		qryRole.constrainProduct( RfcDal::SqlInteger( product.product() ) );
		qryRole.select( result1, Auth4cDal::connection() );

		//Auth4cData::RoleMap roleS = roleMapFromResult( product, result1 ); //copying invalidates RolePermission::Role pointers
		//product.roles() = roleS;
		product.roles() = roleMapFromResult( product, result1 );

		return product;
	}

	return productError;
}

Auth4cData::Domain& Factory::domainFromRow( RfcDal::DalDatabase::Row& row )
{
	static Auth4cData::Domain domainError;

	Auth4cDal::DomainQuery qryDomain( Auth4cDal::schema() );
	RfcDal::DalDatabase::Row::const_iterator columnCustomer;
	RfcDal::DalDatabase::Row::const_iterator columnDomain;
	RfcDal::DalDatabase::Row::const_iterator columnDescription;

	columnCustomer = row.find( qryDomain.columnCustomer().id() );
	if( row.end() == columnCustomer )
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_MISSINGDATA, "Customer column missing." );
	}

	columnDescription = row.find( qryDomain.columnDescription().id() );
	if( row.end() == columnDescription )
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_MISSINGDATA, "Description column missing." );
	}

	Auth4cData::CustomerMap::iterator iterCustomer = customers().find( columnCustomer->second.string() );
	if( customers().end() == iterCustomer )
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_CUSTOMERNOTFOUND, columnCustomer->second.string() );
	}

	columnDomain = row.find( qryDomain.columnDomain().id() );
	if( row.end() == columnDomain )
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_MISSINGDATA, "Domain column missing." );
	}

	Auth4cData::Domain domain( iterCustomer->second, columnDomain->second.string() );
	domain.setDescription( columnDescription->second.string() );
	domains()[domain.domain()] = domain;

	Auth4cData::Domain& domainRef = domains()[domain.domain()];

	try
	{
		domainRef.setAuthenticator( authenticatorFromDomainRow( row ) );
	}
	catch( const Auth4cData::Exception& e )
	{
		throw;
	}

	iterCustomer->second.domains()[domainRef.domain()] = &domainRef;

	return domainRef;
}

Auth4cData::Profile& Factory::profileFromRow( RfcDal::DalDatabase::Row& row )
{
	Auth4cData::Profile profile( profileFromRowNI( row ) );

	profiles()[profile.profile()] = profile;

	Auth4cData::Profile& profileRef = profiles()[profile.profile()];

	return profileRef;
}

Auth4cData::Profile Factory::profileFromRowNI( RfcDal::DalDatabase::Row& row )
{
	Auth4cData::Profile rtn;

	Auth4cDal::ProfileQuery qryProfile( Auth4cDal::schema() );
	RfcDal::DalDatabase::Row::const_iterator column;

	column = row.find( qryProfile.columnProfile().id() );
	if( row.end() == column )
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_MISSINGDATA, "Profile column missing." );
	}

	if( row.end() != column )
	{
		bool bDefault( false );

		RfcDal::DalDatabase::Row::const_iterator columnDefault = row.find( qryProfile.columnIsDefault().id() );

		if( ( row.end() != columnDefault ) &&
			( columnDefault->second.string() == "Y" ) )
		{
			bDefault = true;
		}

		Auth4cData::Profile profile( column->second.string(), bDefault );

		rtn = profile;
	}

	return rtn;
}

Auth4cData::Authenticator& Factory::authenticatorFromDomainRow( RfcDal::DalDatabase::Row& row )
{
	static Auth4cData::Authenticator* pauthenticator( NULL );
	
	Auth4cDal::DomainQuery qry( Auth4cDal::schema() );
	RfcDal::DalDatabase::Row::const_iterator columnDomain = row.find( qry.columnDomain().id() );
	RfcDal::DalDatabase::Row::const_iterator columnDescription = row.find( qry.columnDescription().id() );
	RfcDal::DalDatabase::Row::const_iterator columnAuthenticationLibrary = row.find( qry.columnAuthenticationLibrary().id() );
	if( ( row.end() != columnDomain ) &&
		( row.end() != columnDescription ) &&
		( row.end() != columnAuthenticationLibrary ) )
	{
		Auth4cData::AuthenticationLibraryPtrMap::iterator iter = authenticationLibraries().find( columnAuthenticationLibrary->second.asInt() );
		if( authenticationLibraries().end() == iter )
		{
			throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_LIBRARYNOTLOADED, columnAuthenticationLibrary->second.string() );
		}
		else
		{
//			Auth4cData::Domain domain( columnDomain->second.string() );
			Auth4cData::DomainMap::iterator iterDomain = domains().find( columnDomain->second.string() );
			if( domains().end() == iterDomain )
			{
				throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_DOMAINNOTFOUND, columnDomain->second.string() );
			}

			try
			{
				Auth4cData::InitMap mapInit;

				try
				{
					RfcDal::DalDatabase::Result resultParam;
					Auth4cDal::AuthenticatorParamQuery qryParam( Auth4cDal::schema() );
					qryParam.constrainDomain( RfcDal::SqlText( iterDomain->second.domain() ) );
					qryParam.select( resultParam, Auth4cDal::connection() );
					mapInit = initMapFromResult( resultParam );
				}
				catch( const RfcDal::DalDatabaseException& e )
				{
					// Backward compatibility, table may not exist
				}

				pauthenticator = iter->second->authenticator( iterDomain->second,
																columnDescription->second.string(),
																mapInit );

				if( NULL == pauthenticator )
				{
					throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_LIBRARYNOTLOADED, columnAuthenticationLibrary->second.string() );
				}

				RfcDal::DalDatabase::Row::const_iterator columnUserTools = row.find( qry.columnUserTools().id() );
				if( row.end() != columnUserTools )
				{
					pauthenticator->setUserTools( columnUserTools->second.string() == "Y" );
				}

				authenticators()[iterDomain->second.domain()] = pauthenticator;
			}
			catch( const Auth4cData::Exception& e )
			{
				throw;
			}
		}
	}

	return *pauthenticator;
}

Auth4cData::Customer& Factory::customerFromRow( RfcDal::DalDatabase::Row& row )
{
	static Auth4cData::Customer customerError;

	Auth4cDal::CustomerQuery qryCustomer( Auth4cDal::schema() );
	RfcDal::DalDatabase::Row::const_iterator column;

	column = row.find( qryCustomer.columnCustomer().id() );
	if( row.end() != column )
	{
		Auth4cData::Customer customer( column->second.string() );

		column = row.find( qryCustomer.columnName().id() );
		if( row.end() != column )
		{
			customer.setName( column->second.string() );
		}

		customers()[customer.customer()] = customer;
		return customers()[customer.customer()];
	}

	return customerError;
}

Auth4cData::Team& Factory::teamFromRow( RfcDal::DalDatabase::Row& row )
{
	static Auth4cData::Team teamError;

	Auth4cDal::TeamQuery qryTeam( Auth4cDal::schema() );
	RfcDal::DalDatabase::Row::const_iterator columnTeam;
	RfcDal::DalDatabase::Row::const_iterator columnCustomer;
	RfcDal::DalDatabase::Row::const_iterator columnName;

	columnTeam = row.find( qryTeam.columnTeam().id() );
	columnCustomer = row.find( qryTeam.columnCustomer().id() );

	if( ( row.end() != columnTeam ) &&
		( row.end() != columnCustomer ) )
	{
		Auth4cData::CustomerMap::iterator iterCustomer = customers().find( columnCustomer->second.string() );
		if( customers().end() != iterCustomer )
		{
			Auth4cData::Team team( iterCustomer->second, columnTeam->second.asInt() );

			columnName= row.find( qryTeam.columnName().id() );
			if( row.end() != columnName )
			{
				team.setName( columnName->second.string() );
			}

			Auth4cDal::TeamProductRoleQuery qryTPR( Auth4cDal::schema() );
			qryTPR.constrainTeam( RfcDal::SqlInteger( team.team() ) );

			RfcDal::DalDatabase::Result resultTPR;

			qryTPR.select( resultTPR, Auth4cDal::connection() );

			for( RfcDal::DalDatabase::Result::const_iterator iterTPR = resultTPR.begin();
					iterTPR != resultTPR.end();
					++iterTPR )
			{
				RfcDal::DalDatabase::Row::const_iterator columnProduct = iterTPR->find( qryTPR.columnProduct().id() );
				if( columnProduct != iterTPR->end() )
				{
					Auth4cData::ProductMap::const_iterator iterPP = team.products().find( columnProduct->second.asInt() );
					if( iterPP == team.products().end() )
					{
						Auth4cData::ProductMap::iterator iterP = Auth4cData::Factory::products().find( columnProduct->second.asInt() );
						if( iterP != Auth4cData::Factory::products().end() )
						{
							team.products()[iterP->first] = iterP->second;
						}
					}
				}
			}

			teams()[team.team()] = team;

			Auth4cData::Team& teamAdded( teams()[team.team()] );

			iterCustomer->second.teams()[teamAdded.team()] = &teamAdded;

			return teamAdded;
		}
		else
		{
			throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_CUSTOMERNOTFOUND, columnCustomer->second.string() );
		}
	}
	else
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_MISSINGDATA, "Customer/Team" );
	}

	return teamError;
}

void Factory::teamMapFromRow( RfcDal::DalDatabase::Row& row )
{
	Auth4cDal::TeamMapQuery qryMap( Auth4cDal::schema() );

	RfcDal::DalDatabase::Row::const_iterator columnDomain = row.find( qryMap.columnDomain().id() );
	RfcDal::DalDatabase::Row::const_iterator columnAlienGroup = row.find( qryMap.columnAlienGroup().id() );
	RfcDal::DalDatabase::Row::const_iterator columnAlienGroupLeadGroup = row.find( qryMap.columnAlienGroupLeadGroup().id() );
	RfcDal::DalDatabase::Row::const_iterator columnTeam = row.find( qryMap.columnTeam().id() );

	if( ( row.end() != columnDomain ) &&
		( row.end() != columnAlienGroup ) &&
		( row.end() != columnAlienGroupLeadGroup ) &&
		( row.end() != columnTeam ) )
	{
		Auth4cData::XTeamMap& xteamS( xteams() );

// Create the map entry
		Auth4cData::XTeam xteamTemp( columnDomain->second.string(), columnAlienGroup->second.string(),
										columnAlienGroupLeadGroup->second.string(), columnTeam->second.asInt() );

// Put it into the main map
		xteamS[xteamTemp.keyAlienMap()] = xteamTemp;

// Get the version back from the map
		Auth4cData::XTeamMap::iterator iterMap = xteamS.find( xteamTemp.keyAlienMap() );

		if( xteamS.end() != iterMap )
		{
			Auth4cData::XTeamPtrAlienMap::iterator iterAlien = aliens().find( iterMap->second.keyAlien() );
			if( aliens().end() == iterAlien )
			{
//The alien key does not appear in the map. Create a new map
				Auth4cData::XTeamPtrMap xteamS;
				xteamS[iterMap->second.keyAlienMap()] = &iterMap->second;
				aliens()[iterMap->second.keyAlien()] = xteamS;
			}
			else
			{
				Auth4cData::XTeamPtrMap& xteamS( iterAlien->second );
				xteamS[iterMap->second.keyAlienMap()] = &iterMap->second;
			}

			Auth4cData::XTeamPtrAlienRevMap::iterator iterRev = raliens().find( columnTeam->second.asInt() );
			if( raliens().end() == iterRev )
			{
				Auth4cData::XTeamPtrMap xteamS;
				xteamS[iterMap->second.keyAlienMap()] = &iterMap->second;
				raliens()[iterMap->second.team()] = xteamS;
			}
			else
			{
				Auth4cData::XTeamPtrMap& xteamS( iterRev->second );
				xteamS[iterMap->second.keyAlienMap()] = &iterMap->second;
			}
		}
	}
}

Auth4cData::User& Factory::userFromRow( RfcDal::DalDatabase::Row& row )
{
	static Auth4cData::User userError;

	Auth4cDal::UserQuery qryUser( Auth4cDal::schema() );
	RfcDal::DalDatabase::Row::const_iterator columnUserId;
	RfcDal::DalDatabase::Row::const_iterator columnName;
	RfcDal::DalDatabase::Row::const_iterator columnDomain;
	RfcDal::DalDatabase::Row::const_iterator columnAuthenticationToken;
	RfcDal::DalDatabase::Row::const_iterator columnContact;

	columnUserId = row.find( qryUser.columnUserId().id() );
	columnName = row.find( qryUser.columnName().id() );
	columnDomain = row.find( qryUser.columnDomain().id() );
	columnAuthenticationToken = row.find( qryUser.columnAuthenticationToken().id() );
	columnContact = row.find( qryUser.columnContact().id() );

	if( ( row.end() != columnUserId ) &&
		( row.end() != columnName ) &&
		( row.end() != columnDomain ) &&
		( row.end() != columnAuthenticationToken ) &&
		( row.end() != columnContact ) )
	{
//		Auth4cData::DomainMap& domainS( domains() );
//		Auth4cData::CustomerMap& customerS( customers() );

		Auth4cData::DomainMap::iterator iterDomain = domains().find( columnDomain->second.string() );

		if( domains().end() != iterDomain )
		{
			Auth4cData::User userTemp( iterDomain->second,
									columnUserId->second.string(),
									columnName->second.string(),
									columnAuthenticationToken->second.string(),
									columnContact->second.string() );

//			users()[std::make_pair( userTemp.domain().domain(), userTemp.userId() )] = userTemp;

			std::pair<std::string, std::string> keyUser = std::make_pair( userTemp.domain().domain(), userTemp.userId() );

			iterDomain->second.users()[keyUser] = userTemp;
			Auth4cData::User& user( iterDomain->second.users()[keyUser] );

			Auth4cData::TeamMap::iterator iterTeam;
			RfcDal::DalDatabase::Result result;
			RfcDal::DalDatabase::Row row;
			RfcDal::DalDatabase::Row::const_iterator columnTeam;
			Auth4cDal::UserTeamQuery qryTeam( Auth4cDal::schema() );
			qryTeam.constrainDomain( RfcDal::SqlText( user.domain().domain() ) );
			qryTeam.constrainUserId( RfcDal::SqlText( user.userId() ) );
			qryTeam.select( result, Auth4cDal::connection() );
			for( RfcDal::DalDatabase::Result::const_iterator iter = result.begin();
					iter != result.end();
					++iter )
			{
				row = *iter;
				columnTeam = row.find( qryTeam.columnTeam().id() );
				if( row.end() != columnTeam )
				{
					iterTeam = teams().find( columnTeam->second.asInt() );
					if( teams().end() != iterTeam )
					{
// We don't do this here. When a team is edited, we get the list of members from the relevant Authenticator(s)
//						iterTeam->second.members()[user.key()] = Auth4cData::TeamMember( user, false );
						user.teams()[iterTeam->second.team()] = iterTeam->second;
					}
				}
			}

			return user;
		}
	}

	return userError;
}

Auth4cData::Platform& Factory::platformFromRow( RfcDal::DalDatabase::Row& row )
{
	static Auth4cData::Platform platformError;

	Auth4cDal::PlatformQuery qryPlatform( Auth4cDal::schema() );
	RfcDal::DalDatabase::Row::const_iterator columnPlatform;
	RfcDal::DalDatabase::Row::const_iterator columnDescription;

	columnPlatform = row.find( qryPlatform.columnPlatform().id() );
	columnDescription = row.find( qryPlatform.columnDescription().id() );

	if( ( row.end() != columnPlatform ) &&
		( row.end() != columnDescription ) )
	{
		Auth4cData::Platform platformTmp( columnPlatform->second.string(),
			columnDescription->second.string() );

		Auth4cData::Factory::platforms()[platformTmp.platform()] = platformTmp;
		Auth4cData::Platform& platform = Auth4cData::Factory::platforms()[platformTmp.platform()];
		return platform;
	}

	return platformError;
}

Auth4cData::SharedObject& Factory::sharedObjectFromRow( RfcDal::DalDatabase::Row& row )
{
	static Auth4cData::SharedObject sharedObjectError;

	Auth4cDal::SharedObjectQuery qryObject( Auth4cDal::schema() );
	RfcDal::DalDatabase::Row::const_iterator columnPlatform;
	RfcDal::DalDatabase::Row::const_iterator columnAuthenticationLibrary;
	RfcDal::DalDatabase::Row::const_iterator columnSharedObject;

	columnPlatform = row.find( qryObject.columnPlatform().id() );
	columnAuthenticationLibrary = row.find( qryObject.columnAuthenticationLibrary().id() );
	columnSharedObject = row.find( qryObject.columnSharedObject().id() );

	if( ( row.end() != columnPlatform ) &&
		( row.end() != columnAuthenticationLibrary ) &&
		( row.end() != columnSharedObject ) )
	{
		Auth4cData::PlatformMap::const_iterator iterPlatform = Auth4cData::Factory::platforms().find( columnPlatform->second.string() );
		if( Auth4cData::Factory::platforms().end() == iterPlatform )
		{
			throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_PLATFORMNOTFOUND, columnPlatform->second.string() );
		}

		Auth4cData::AuthenticationLibraryMap::iterator iterLibrary = Auth4cData::Factory::authenticationLibraryTemplates().find( columnAuthenticationLibrary->second.asInt() );
		if( Auth4cData::Factory::authenticationLibraryTemplates().end() == iterLibrary )
		{
			std::ostringstream os;
			os << columnAuthenticationLibrary->second.asInt();
			throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_AUTHENTICATIONLIBRARYNOTFOUND, os.str() );
		}

		Auth4cData::SharedObject objTemp( iterPlatform->second, iterLibrary->second, columnSharedObject->second.string() );
		Auth4cData::Factory::sharedObjects()[objTemp.key()] = objTemp;
		Auth4cData::SharedObject& object( Auth4cData::Factory::sharedObjects()[objTemp.key()] );
		iterLibrary->second.sharedObjects()[object.platform().platform()] = &object;

		return object;
	}

	return sharedObjectError;
}

Auth4cData::AuthenticationLibrary& Factory::authenticationLibraryFromRow( RfcDal::DalDatabase::Row& row )
{
	static Auth4cData::AuthenticationLibrary libraryError;

	Auth4cDal::AuthenticationLibraryQuery qry( Auth4cDal::schema() );

	RfcDal::DalDatabase::Row::const_iterator columnAuthenticationLibrary = row.find( qry.columnAuthenticationLibrary().id() );
	RfcDal::DalDatabase::Row::const_iterator columnDescription = row.find( qry.columnDescription().id() );
	RfcDal::DalDatabase::Row::const_iterator columnAuthenticationTokenDisposition = row.find( qry.columnAuthenticationTokenDisposition().id() );
	RfcDal::DalDatabase::Row::const_iterator columnUserTools = row.find( qry.columnUserTools().id() );

	if( ( row.end() != columnAuthenticationLibrary ) &&
		( row.end() != columnDescription ) &&
		( row.end() != columnAuthenticationTokenDisposition ) &&
		( row.end() != columnUserTools ) )
	{
		Auth4cData::AuthenticationLibrary::AuthenticationTokenDisposition authenticationTokenDisposition;

		if( columnAuthenticationTokenDisposition->second.string() == "C" )
		{
			authenticationTokenDisposition = Auth4cData::AuthenticationLibrary::dispositionConfig;
		}
		else if( columnAuthenticationTokenDisposition->second.string() == "H" )
		{
			authenticationTokenDisposition = Auth4cData::AuthenticationLibrary::dispositionHashedPassword;
		}
		else if( columnAuthenticationTokenDisposition->second.string() == "N" )
		{
			authenticationTokenDisposition = Auth4cData::AuthenticationLibrary::dispositionNotUsed;
		}
		else if( columnAuthenticationTokenDisposition->second.string() == "P" )
		{
			authenticationTokenDisposition = Auth4cData::AuthenticationLibrary::dispositionClearPassword;
		}
		else if( columnAuthenticationTokenDisposition->second.string() == "X" )
		{
			authenticationTokenDisposition = Auth4cData::AuthenticationLibrary::dispositionMap;
		}

		Auth4cData::InitMap mapInit;

		try
		{
			RfcDal::DalDatabase::Result resultParam;
			Auth4cDal::AuthenticationLibraryParamQuery qryParam( Auth4cDal::schema() );
			qryParam.constrainAuthenticationLibrary( RfcDal::SqlInteger( columnAuthenticationLibrary->second.asInt() ) );
			qryParam.select( resultParam, Auth4cDal::connection() );
			mapInit = initMapFromResult( resultParam );
		}
		catch( const RfcDal::DalDatabaseException& e )
		{
			// Backward compatibility, table may not exist
		}

		Auth4cData::AuthenticationLibrary libraryTemp( columnAuthenticationLibrary->second.asInt(),
														columnDescription->second.string(),
														mapInit,
														authenticationTokenDisposition,
														columnUserTools->second.string() == "Y" );

		Auth4cData::Factory::authenticationLibraryTemplates()[libraryTemp.authenticationLibrary()] = libraryTemp;
		Auth4cData::AuthenticationLibrary& library( Auth4cData::Factory::authenticationLibraryTemplates()[libraryTemp.authenticationLibrary()] );
		return library;
	}

	return libraryError;
}

#if (0)
Auth4cData::AuthenticationLibrary& Factory::authenticationLibraryFromRow( RfcDal::DalDatabase::Row& row )
{
	static Auth4cData::AuthenticationLibrary* plibrary( NULL );
//	Auth4cData::AuthenticationLibrary::Handle handle;

	Auth4cDal::AuthenticationLibraryXQuery qry( Auth4cDal::schema() );
	RfcDal::DalDatabase::Row::const_iterator columnAuthenticationLibrary = row.find( qry.columnAuthenticationLibrary().id() );
	RfcDal::DalDatabase::Row::const_iterator columnSharedObject = row.find( qry.columnSharedObject().id() );
	RfcDal::DalDatabase::Row::const_iterator columnLibraryInitString = row.find( qry.columnLibraryInitString().id() );
	if( ( row.end() != columnAuthenticationLibrary ) &&
		( row.end() != columnSharedObject ) &&
		( row.end() != columnLibraryInitString ) )
	{
#if defined (LINUX)
		handle = dlopen(  columnSharedObject->second.string().c_str(), RTLD_LAZY );
#elif defined (_WINDOWS)
		handle = LoadLibraryA( columnSharedObject->second.string().c_str() );
#endif /* LINUX */
		if( handle == NULL )
		{
#if defined (LINUX)
			throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_NOSHAREDOBJECT, dlerror() );
#elif defined (_WINDOWS)
			throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_NOSHAREDOBJECT, "Could not load shared object" );
#endif /* LINUX */
		}
		else
		{
#if defined (LINUX)
			Auth4cData::AuthenticationLibrary* (*library)( int, const std::string&, const std::string&, Auth4cData::AuthenticationLibrary::Handle ) = (Auth4cData::AuthenticationLibrary* (*)( int, const std::string&, const std::string&, Auth4cData::AuthenticationLibrary::Handle ))dlsym( handle, "library" );
#elif defined (_WINDOWS)
			Auth4cData::AuthenticationLibrary* (*library)( int, const std::string&, const std::string&, Auth4cData::AuthenticationLibrary::Handle ) = (Auth4cData::AuthenticationLibrary* (*)( int, const std::string&, const std::string&, Auth4cData::AuthenticationLibrary::Handle ))GetProcAddress( handle, "library" );
#endif /* LINUX */
			
#if defined (LINUX)
			char* pszError = dlerror();
			if( pszError != NULL )
			{
				dlclose( handle );
				handle = NULL;
				throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_NOENTRYPOINT, "Symbol not found" );
			}
#elif defined (_WINDOWS)
			if( library == NULL )
			{
				FreeLibrary( handle );
				handle = NULL;
				throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_NOENTRYPOINT, "Symbol not found" );
			}
#endif /* LINUX */

			plibrary = library( columnAuthenticationLibrary->second.asInt(),
								columnSharedObject->second.string(),
								columnLibraryInitString->second.string(),
								handle );

			RfcDal::DalDatabase::Row::const_iterator columnDescription = row.find( qry.columnDescription().id() );
			if( row.end() != columnDescription )
			{
				plibrary->setDescription( columnDescription->second.string() );
			}

			RfcDal::DalDatabase::Row::const_iterator columnAuthenticationTokenDisposition = row.find( qry.columnAuthenticationTokenDisposition().id() );
			if( row.end() != columnAuthenticationTokenDisposition )
			{
				if( columnAuthenticationTokenDisposition->second.string() == "C" )
				{
					plibrary->setAuthenticationTokenDisposition( Auth4cData::AuthenticationLibrary::dispositionConfig );
				}
				else if( columnAuthenticationTokenDisposition->second.string() == "H" )
				{
					plibrary->setAuthenticationTokenDisposition( Auth4cData::AuthenticationLibrary::dispositionHashedPassword );
				}
				else if( columnAuthenticationTokenDisposition->second.string() == "N" )
				{
					plibrary->setAuthenticationTokenDisposition( Auth4cData::AuthenticationLibrary::dispositionNotUsed );
				}
				else if( columnAuthenticationTokenDisposition->second.string() == "P" )
				{
					plibrary->setAuthenticationTokenDisposition( Auth4cData::AuthenticationLibrary::dispositionClearPassword );
				}
				else if( columnAuthenticationTokenDisposition->second.string() == "X" )
				{
					plibrary->setAuthenticationTokenDisposition( Auth4cData::AuthenticationLibrary::dispositionMap );
				}
			}

			RfcDal::DalDatabase::Row::const_iterator columnUserTools = row.find( qry.columnUserTools().id() );
			if( row.end() != columnUserTools )
			{
				plibrary->setUserTools( columnUserTools->second.string() == "Y" );
			}

			authenticationLibraries()[plibrary->authenticationLibrary()] = plibrary;
		}
	}

	return *plibrary;
}
#endif

Auth4cData::RoleMap Factory::roleMapFromResult( const Auth4cData::Product& product, const RfcDal::DalDatabase::Result& result )
{
	Auth4cData::RoleMap rtn( product.roles() );
	RfcDal::DalDatabase::Row row;
	Auth4cDal::ProductRoleQuery qryRole( Auth4cDal::schema() );
	RfcDal::DalDatabase::Row::const_iterator columnRole;
	RfcDal::DalDatabase::Row::const_iterator columnDescription;
	RfcDal::DalDatabase::Row::const_iterator columnStatus;
	RfcDal::DalDatabase::Row::const_iterator columnPermission;

	for( RfcDal::DalDatabase::Result::const_iterator iter = result.begin();
			iter != result.end();
			++iter )
	{
		row = *iter;

		columnRole = row.find( qryRole.columnRole().id() );
		if( columnRole != row.end() )
		{
			Auth4cData::Role role( product, columnRole->second.asInt() );

			columnDescription = row.find( qryRole.columnDescription().id() );
			if( columnDescription != row.end() )
			{
				role.setDescription( columnDescription->second.string() );
			}

			columnStatus = row.find( qryRole.columnStatus().id() );
			if( columnStatus != row.end() )
			{
				role.setStatus( Auth4cData::PRStatus::fromInt( columnStatus->second.asInt() ) );
			}

			rtn[role.role()] = role;
		}
	}

	RfcDal::DalDatabase::Result resultPR;
	RfcDal::DalDatabase::Row rowPR;
	Auth4cDal::ProductRolePermissionQuery qryRolePermission( Auth4cDal::schema() );
	qryRolePermission.constrainProduct( RfcDal::SqlInteger( product.product() ) );
	qryRolePermission.select( resultPR, Auth4cDal::connection() );
	for( RfcDal::DalDatabase::Result::const_iterator iterPR = resultPR.begin();
			iterPR != resultPR.end();
			++iterPR )
	{
		rowPR = *iterPR;

		int nRole( -1 );
		int nPermission( -1 );
		Auth4cData::PRStatus status;

		columnRole = rowPR.find( qryRolePermission.columnRole().id() );
		if( columnRole != rowPR.end() )
		{
			nRole = columnRole->second.asInt();
		}

		columnPermission = rowPR.find( qryRolePermission.columnPermission().id() );
		if( columnPermission != rowPR.end() )
		{
			nPermission = columnPermission->second.asInt();
		}

		columnStatus = rowPR.find( qryRolePermission.columnStatus().id() );
		if( columnStatus != rowPR.end() )
		{
			status = Auth4cData::PRStatus::fromInt( columnStatus->second.asInt() );
		}

		if( ( -1 != nRole ) &&
			( -1 != nPermission ) )
		{
			Auth4cData::RoleMap::iterator iterRole = rtn.find( nRole );
			Auth4cData::PermissionMap::const_iterator iterPermission = product.permissions().find( nPermission );

			if( ( rtn.end() != iterRole ) &&
				( product.permissions().end() != iterPermission ) )
			{
				Auth4cData::Permission permissionRole( iterRole->second, nPermission, iterPermission->second.description(), status );
				iterRole->second.permissions()[permissionRole.permission()] = permissionRole;
			}
		}
	}

	return rtn;
}

Auth4cData::PermissionMap Factory::permissionMapFromResult( const Auth4cData::Product& product, const RfcDal::DalDatabase::Result& result )
{
	Auth4cData::PermissionMap rtn;
	RfcDal::DalDatabase::Row row;
	Auth4cDal::ProductPermissionQuery qryPermission( Auth4cDal::schema() );
	RfcDal::DalDatabase::Row::const_iterator columnPermission;
	RfcDal::DalDatabase::Row::const_iterator columnDescription;
	RfcDal::DalDatabase::Row::const_iterator columnStatus;

	for( RfcDal::DalDatabase::Result::const_iterator iter = result.begin();
			iter != result.end();
			++iter )
	{
		row = *iter;

		columnPermission = row.find( qryPermission.columnPermission().id() );
		if( columnPermission != row.end() )
		{
			Auth4cData::Permission permission( product, columnPermission->second.asInt() );

			columnDescription = row.find( qryPermission.columnDescription().id() );
			if( columnDescription != row.end() )
			{
				permission.setDescription( columnDescription->second.string() );
			}

			columnStatus = row.find( qryPermission.columnStatus().id() );
			if( columnStatus != row.end() )
			{
				permission.setStatus( Auth4cData::PRStatus::fromInt( columnStatus->second.asInt() ) );
			}

			rtn[permission.permission()] = permission;
		}
	}

	return rtn;
}

void Factory::productRoleMapFromResult( Auth4cData::ProductMap& productS, const RfcDal::DalDatabase::Result& result )
{
	RfcDal::SqlTable tbPR( Auth4cDal::schema(), "PR", "PR" );
	Auth4cDal::GenericProductRoleQuery qry( tbPR );
	RfcDal::DalDatabase::Row row;
	RfcDal::DalDatabase::Row::const_iterator columnProduct;
	RfcDal::DalDatabase::Row::const_iterator columnRole;
	RfcDal::DalDatabase::Row::const_iterator columnStatus;

	for( RfcDal::DalDatabase::Result::const_iterator iter = result.begin();
			iter != result.end();
			++iter )
	{
		try
		{
			row = *iter;
			columnProduct = row.find( qry.columnProduct().id() );
			columnRole = row.find( qry.columnRole().id() );
			columnStatus = row.find( qry.columnStatus().id() );
			if( ( columnRole != row.end() ) &&
				( columnStatus != row.end() ) &&
				( columnProduct != row.end() ) )
			{
				Auth4cData::ProductMap::iterator iterProduct = productS.find( columnProduct->second.asInt() );
				if( iterProduct == productS.end() )
				{
					Auth4cData::ProductMap::const_iterator iterProductAdd = Auth4cData::Factory::products().find( columnProduct->second.asInt() );
					if( iterProductAdd == Auth4cData::Factory::products().end() )
					{
						std::ostringstream os;
						os << columnProduct->second.asInt();
						throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_PRODUCTNOTFOUND, os.str(), true );
					}

					productS[iterProductAdd->first] = iterProductAdd->second;
					iterProduct = productS.find( columnProduct->second.asInt() );
				}

				Auth4cData::Role role( iterProduct->second, columnRole->second.asInt(), std::string(), Auth4cData::PRStatus::fromInt( columnStatus->second.asInt() ) );
				iterProduct->second.roles().merge( role );
			}
		}
		catch( const Auth4cData::Exception& e )
		{
			std::ostringstream os;
			os << e;
			Base4c::EventLogger::theLogger()->logWarning( os.str(), Base4c::EventLogger::data );
		}
	}
}

void Factory::productPermissionMapFromResult( Auth4cData::ProductMap& productS, const RfcDal::DalDatabase::Result& result )
{
	RfcDal::SqlTable tbPP( Auth4cDal::schema(), "PP", "PP" );
	Auth4cDal::GenericProductPermissionQuery qry( tbPP );
	RfcDal::DalDatabase::Row row;
	RfcDal::DalDatabase::Row::const_iterator columnProduct;
	RfcDal::DalDatabase::Row::const_iterator columnPermission;
	RfcDal::DalDatabase::Row::const_iterator columnStatus;

	for( RfcDal::DalDatabase::Result::const_iterator iter = result.begin();
			iter != result.end();
			++iter )
	{
		try
		{
			row = *iter;
			columnProduct = row.find( qry.columnProduct().id() );
			columnPermission = row.find( qry.columnPermission().id() );
			columnStatus = row.find( qry.columnStatus().id() );
			if( ( columnPermission != row.end() ) &&
				( columnStatus != row.end() ) &&
				( columnProduct != row.end() ) )
			{
				Auth4cData::ProductMap::iterator iterProduct = productS.find( columnProduct->second.asInt() );
				if( iterProduct == productS.end() )
				{
					Auth4cData::ProductMap::const_iterator iterProductAdd = Auth4cData::Factory::products().find( columnProduct->second.asInt() );
					if( iterProductAdd == Auth4cData::Factory::products().end() )
					{
						std::ostringstream os;
						os << columnProduct->second.asInt();
						throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_PRODUCTNOTFOUND, os.str(), true );
					}

					productS[iterProductAdd->first] = iterProductAdd->second;
					iterProduct = productS.find( columnProduct->second.asInt() );
				}

				Auth4cData::Permission permission( iterProduct->second, columnPermission->second.asInt(), std::string(), Auth4cData::PRStatus::fromInt( columnStatus->second.asInt() ) );
				iterProduct->second.permissions().merge( permission );
			}
		}
		catch( const Auth4cData::Exception& e )
		{
			std::ostringstream os;
			os << e;
			Base4c::EventLogger::theLogger()->logWarning( os.str(), Base4c::EventLogger::data );
		}
	}
}


Auth4cData::InitMap Factory::initMapFromResult( const RfcDal::DalDatabase::Result& result )
{
	Auth4cData::InitMap rtn;

	Auth4cDal::AuthenticationLibraryParamQuery qry( Auth4cDal::schema() ); // Could use any of the *ParamQuery classes

	RfcDal::DalDatabase::Row row;
	RfcDal::DalDatabase::Row::const_iterator columnParam;
	RfcDal::DalDatabase::Row::const_iterator columnValue;

	for( RfcDal::DalDatabase::Result::const_iterator iter = result.begin();
			iter != result.end();
			++iter )
	{
		row = *iter;

		columnParam = row.find( qry.columnParam().id() );
		columnValue = row.find( qry.columnValue().id() );

		if( row.end() == columnParam )
		{
			throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_MISSINGDATA, qry.columnParam().id() );
		}

		if( row.end() == columnValue )
		{
			throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_MISSINGDATA, qry.columnValue().id() );
		}

		rtn[columnParam->second.string()] = columnValue->second.string();
	}

	return rtn;
}

Auth4cData::InitMapMap Factory::initMapMapFromResult( const RfcDal::DalDatabase::Result& result )
{
	Auth4cData::InitMapMap rtn;

	Auth4cDal::ProductParamQuery qry( Auth4cDal::schema() ); // Could use any of the *ParamQuery classes

	RfcDal::DalDatabase::Row row;
	RfcDal::DalDatabase::Row::const_iterator columnProfile;
	RfcDal::DalDatabase::Row::const_iterator columnParam;
	RfcDal::DalDatabase::Row::const_iterator columnValue;


	for( RfcDal::DalDatabase::Result::const_iterator iter = result.begin();
			iter != result.end();
			++iter )
	{
		row = *iter;

		columnProfile = row.find( qry.columnProfile().id() );
		columnParam = row.find( qry.columnParam().id() );
		columnValue = row.find( qry.columnValue().id() );

		if( row.end() == columnProfile )
		{
			throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_MISSINGDATA, qry.columnProfile().id() );
		}

		if( row.end() == columnParam )
		{
			throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_MISSINGDATA, qry.columnParam().id() );
		}

		if( row.end() == columnValue )
		{
			throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_MISSINGDATA, qry.columnValue().id() );
		}

		rtn[columnProfile->second.string()][columnParam->second.string()] = columnValue->second.string();
	}

	return rtn;
}

void Factory::createCustomer( Auth4cData::Customer& customer )
{
	Auth4cDal::CustomerQuery qry( Auth4cDal::schema() );
	qry.columnCustomer().setValue( RfcDal::SqlText( customer.customer() ) );
	qry.columnName().setValue( RfcDal::SqlText( customer.name() ) );

	try
	{
		Auth4cDal::Connection conn( Auth4cDal::connection() );

		qry.insert( conn );

		customers()[customer.customer()] = customer;
	}
	catch( const RfcDal::DalDatabaseException& e )
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_DATABASE, e.nativeExplanation(), true );
	}
}

void Factory::updateCustomer( Auth4cData::Customer& customer )
{
	Auth4cDal::CustomerQuery qry( Auth4cDal::schema() );
	qry.constrainCustomer( RfcDal::SqlText( customer.customer() ) );
	qry.columnName().setValue( RfcDal::SqlText( customer.name() ) );

	try
	{
		Auth4cDal::Connection conn( Auth4cDal::connection() );

		qry.update( conn );

		customers()[customer.customer()] = customer;
	}
	catch( const RfcDal::DalDatabaseException& e )
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_DATABASE, e.nativeExplanation(), true );
	}
}

void Factory::deleteCustomer( Auth4cData::Customer& customer )
{
	Auth4cDal::CustomerQuery qry( Auth4cDal::schema() );
	qry.constrainCustomer( RfcDal::SqlText( customer.customer() ) );

	try
	{
		Auth4cDal::Connection conn( Auth4cDal::connection() );

		qry.remove( conn );

		Auth4cData::CustomerMap::iterator iterCustomer = customers().find( customer.customer() );
		if( customers().end() != iterCustomer )
		{
			customers().erase( iterCustomer );
		}
	}
	catch( const RfcDal::DalDatabaseException& e )
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_DATABASE, e.nativeExplanation(), true );
	}
}

void Factory::createDomain( Auth4cData::Domain& domain )
{
	Auth4cDal::DomainQuery qry( Auth4cDal::schema() );
	qry.columnCustomer().setValue( RfcDal::SqlText( domain.customer().customer() ) );
	qry.columnDomain().setValue( RfcDal::SqlText( domain.domain() ) );
	qry.columnDescription().setValue( RfcDal::SqlText( domain.description() ) );

	try
	{
		Auth4cDal::Connection conn( Auth4cDal::connection() );

		qry.insert( conn );

		domains()[domain.domain()] = domain;
		Auth4cData::CustomerMap::iterator iterCustomer = Factory::customers().find( domain.customer().customer() );
		if( Factory::customers().end() != iterCustomer )
		{
			iterCustomer->second.domains()[domain.domain()] = &domains()[domain.domain()];
		}
	}
	catch( const RfcDal::DalDatabaseException& e )
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_DATABASE, e.nativeExplanation(), true );
	}
}

void Factory::updateDomain( Auth4cData::Domain& domain )
{
	Auth4cDal::DomainQuery qry( Auth4cDal::schema() );
	qry.constrainDomain( RfcDal::SqlText( domain.domain() ) );
	qry.columnDescription().setValue( RfcDal::SqlText( domain.description() ) );

	try
	{
		Auth4cDal::Connection conn( Auth4cDal::connection() );

		qry.update( conn );

		domains()[domain.domain()] = domain;
		Auth4cData::CustomerMap::iterator iterCustomer = Factory::customers().find( domain.customer().customer() );
		if( Factory::customers().end() != iterCustomer )
		{
			iterCustomer->second.domains()[domain.domain()] = &domains()[domain.domain()];
		}
	}
	catch( const RfcDal::DalDatabaseException& e )
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_DATABASE, e.nativeExplanation(), true );
	}
}

void Factory::deleteDomain( Auth4cData::Domain& domain )
{
	Auth4cDal::DomainQuery qry( Auth4cDal::schema() );
	qry.constrainDomain( RfcDal::SqlText( domain.domain() ) );

	try
	{
		Auth4cDal::Connection conn( Auth4cDal::connection() );

		qry.remove( conn );

		Auth4cData::CustomerMap::iterator iterCustomer = Factory::customers().find( domain.customer().customer() );
		if( Factory::customers().end() != iterCustomer )
		{
			Auth4cData::DomainPtrMap::iterator iterDomain = iterCustomer->second.domains().find( domain.domain() );
			if( iterCustomer->second.domains().end() != iterDomain )
			{
				iterCustomer->second.domains().erase( iterDomain );
			}
		}

		Auth4cData::DomainMap::iterator iterDomain = Factory::domains().find( domain.domain() );
		if( Factory::domains().end() != iterDomain )
		{
			Factory::domains().erase( iterDomain );
		}
	}
	catch( const RfcDal::DalDatabaseException& e )
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_DATABASE, e.nativeExplanation(), true );
	}
}

void Factory::insertProduct( const Auth4cData::Product& product )
{
	if( !Auth4cData::Factory::hasProductPermission() )
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_PERMISSIONDENIED, "unauthorized action" );
	}

	Auth4cDal::ProductQuery qry( Auth4cDal::schema() );
	qry.columnProduct().setValue( RfcDal::SqlInteger( product.product() ) );
	qry.columnDescription().setValue( RfcDal::SqlText( product.description() ) );

	qry.constrainProduct( RfcDal::SqlInteger( product.product() ) );

	Auth4cDal::Connection conn( Auth4cDal::connection() );

	try
	{
		conn.db().begin();

		//must do parameters before product insert/update
		deleteProductParam( product, "", "", true );
		const Auth4cData::InitMapMap& initS( product.productInit() );
		for( Auth4cData::InitMapMap::const_iterator iterInitMM = initS.begin();
				iterInitMM != initS.end();
				++iterInitMM )
		{
			for( Auth4cData::InitMap::const_iterator iterInitM = iterInitMM->second.begin();
					iterInitM != iterInitMM->second.end();
					++iterInitM )
			{
				insertProductParam( product, iterInitMM->first, iterInitM->first, iterInitM->second );
			}
		}

		// We have to do permissions before roles as each role will create a link entry to one or more permissions
		const Auth4cData::PermissionMap& permissionS( product.permissions() );
		for( Auth4cData::PermissionMap::const_iterator iterPermission = permissionS.begin();
				iterPermission != permissionS.end();
				++iterPermission )
		{
			insertPermission( iterPermission->second, product );
		}

		const Auth4cData::RoleMap& roleS( product.roles() );
		for( Auth4cData::RoleMap::const_iterator iterRole = roleS.begin();
				iterRole != roleS.end();
				++iterRole )
		{
			insertRole( iterRole->second, product );
		}

		RfcDal::DalDatabase::Result result;
		int ret = qry.select( result, conn );
		if( !ret ) //new
		{
			qry.insert( conn );
		}
		else
		{
			qry.update( conn );
		}

		conn.db().commit();
	}
	catch( const Auth4cData::Exception& e )
	{
		conn.db().rollback();
		throw;
	}
	catch( const RfcDal::SqlException& e )
	{
		conn.db().rollback();
		Auth4cData::Exception et( __FILE__, __LINE__, Auth4cData::nNAX_DATABASE, RfcDal::SqlException::explainErrorCode( e.errorCode() ) );
		throw et;
	}
	catch( const RfcDal::DalDatabaseException& e )
	{
		conn.db().rollback();
		Auth4cData::Exception et( __FILE__, __LINE__, Auth4cData::nNAX_DATABASE, e.nativeExplanation() );
		throw et;
	}
	catch( ... )
	{
		conn.db().rollback();
		Auth4cData::Exception et( __FILE__, __LINE__, Auth4cData::nNAX_UNKNOWN, "unknown" );
		throw et;
	}
}

void Factory::insertProductParam( const Auth4cData::Product& product, const std::string& strProfile, const std::string& strName, const std::string& strValue )
{
	if( !Auth4cData::Factory::hasProductPermission() )
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_PERMISSIONDENIED, "unauthorized action" );
	}

	Auth4cDal::ProductParamQuery qry( Auth4cDal::schema() );
	qry.columnProduct().setValue( RfcDal::SqlInteger( product.product() ) );
	qry.columnProfile().setValue( RfcDal::SqlText( strProfile ) );
	qry.columnParam().setValue( RfcDal::SqlText( strName ) );
	qry.columnValue().setValue( RfcDal::SqlText( strValue ) );

	Auth4cDal::Connection conn( Auth4cDal::connection() );

	try
	{
		conn.db().begin();

		qry.insert( conn );

		conn.db().commit();
	}
	catch( const Auth4cData::Exception& e )
	{
		conn.db().rollback();
		throw;
	}
	catch( const RfcDal::SqlException& e )
	{
		conn.db().rollback();
		Auth4cData::Exception et( __FILE__, __LINE__, Auth4cData::nNAX_DATABASE, RfcDal::SqlException::explainErrorCode( e.errorCode() ) );
		throw et;
	}
	catch( const RfcDal::DalDatabaseException& e )
	{
		conn.db().rollback();
		Auth4cData::Exception et( __FILE__, __LINE__, Auth4cData::nNAX_DATABASE, e.nativeExplanation() );
		throw et;
	}
	catch( ... )
	{
		conn.db().rollback();
		Auth4cData::Exception et( __FILE__, __LINE__, Auth4cData::nNAX_UNKNOWN, "unknown" );
		throw et;
	}
}

void Factory::insertPermission( const Auth4cData::Permission& permission, const Auth4cData::Product& product )
{
	if( !Auth4cData::Factory::hasProductPermission() )
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_PERMISSIONDENIED, "unauthorized action" );
	}

	Auth4cDal::ProductPermissionQuery qry( Auth4cDal::schema() );
	qry.columnProduct().setValue( RfcDal::SqlInteger( product.product() ) );
	qry.columnPermission().setValue( RfcDal::SqlInteger( permission.permission() ) );
	qry.columnDescription().setValue( RfcDal::SqlText( permission.description() ) );
	qry.columnStatus().setValue( RfcDal::SqlInteger( permission.status() ) );

	qry.constrainProduct( RfcDal::SqlInteger( product.product() ) );
	qry.constrainPermission( RfcDal::SqlInteger( permission.permission() ) );

	Auth4cDal::Connection conn = Auth4cDal::connection();

	try
	{
		conn.db().begin();

		RfcDal::DalDatabase::Result result;
		int ret = qry.select( result, conn );
		if( !ret ) //new
		{
			qry.insert( conn );
		}
		else
		{
			qry.update( conn );
		}

		conn.db().commit();
	}
	catch( const Auth4cData::Exception& e )
	{
		conn.db().rollback();
		throw;
	}
	catch( const RfcDal::SqlException& e )
	{
		conn.db().rollback();
		Auth4cData::Exception et( __FILE__, __LINE__, Auth4cData::nNAX_DATABASE, RfcDal::SqlException::explainErrorCode( e.errorCode() ) );
		throw et;
	}
	catch( const RfcDal::DalDatabaseException& e )
	{
		conn.db().rollback();
		Auth4cData::Exception et( __FILE__, __LINE__, Auth4cData::nNAX_DATABASE, e.nativeExplanation() );
		throw et;
	}
	catch( ... )
	{
		conn.db().rollback();
		Auth4cData::Exception et( __FILE__, __LINE__, Auth4cData::nNAX_UNKNOWN, "unknown" );
		throw et;
	}
}

void Factory::insertRole( const Auth4cData::Role& role, const Auth4cData::Product& product )
{
	if( !Auth4cData::Factory::hasProductPermission() )
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_PERMISSIONDENIED, "unauthorized action" );
	}

	Auth4cDal::ProductRoleQuery qry( Auth4cDal::schema() );
	qry.columnProduct().setValue( RfcDal::SqlInteger( product.product() ) );
	qry.columnRole().setValue( RfcDal::SqlInteger( role.role() ) );
	qry.columnDescription().setValue( RfcDal::SqlText( role.description() ) );
	qry.columnStatus().setValue( RfcDal::SqlInteger( role.status() ) );

	qry.constrainProduct( RfcDal::SqlInteger( product.product() ) );
	qry.constrainRole( RfcDal::SqlInteger( role.role() ) );

	Auth4cDal::Connection conn = Auth4cDal::connection();

	try
	{
		conn.db().begin();
		
		//permissions must come before role insert/update
		deleteRolePermission( Auth4cData::Permission( role, 0 ), true );
		
		RfcDal::DalDatabase::Result result;
		int ret = qry.select( result, conn );
		if( !ret ) //new
		{
			qry.insert( conn );
		}
		else
		{
			qry.update( conn );
		}

		conn.db().commit(); //need role to exist before adding role permissions

		const Auth4cData::PermissionMap& permissionS( role.permissions() );
		for( Auth4cData::PermissionMap::const_iterator iterPermission = permissionS.begin();
				iterPermission != permissionS.end();
				++iterPermission )
		{
			insertRolePermission( iterPermission->second, role, product );
		}
	}
	catch( const Auth4cData::Exception& e )
	{
		conn.db().rollback();
		throw;
	}
	catch( const RfcDal::SqlException& e )
	{
		conn.db().rollback();
		Auth4cData::Exception et( __FILE__, __LINE__, Auth4cData::nNAX_DATABASE, RfcDal::SqlException::explainErrorCode( e.errorCode() ) );
		throw et;
	}
	catch( const RfcDal::DalDatabaseException& e )
	{
		conn.db().rollback();
		Auth4cData::Exception et( __FILE__, __LINE__, Auth4cData::nNAX_DATABASE, e.nativeExplanation() );
		throw et;
	}
	catch( ... )
	{
		conn.db().rollback();
		Auth4cData::Exception et( __FILE__, __LINE__, Auth4cData::nNAX_UNKNOWN, "unknown" );
		throw et;
	}
}

void Factory::insertRolePermission( const Auth4cData::Permission& permission, const Auth4cData::Role& role, const Auth4cData::Product& product )
{
	if( !Auth4cData::Factory::hasProductPermission() )
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_PERMISSIONDENIED, "unauthorized action" );
	}

	Auth4cDal::ProductRolePermissionQuery qry( Auth4cDal::schema() );
	qry.columnProduct().setValue( RfcDal::SqlInteger( product.product() ) );
	qry.columnRole().setValue( RfcDal::SqlInteger( role.role() ) );
	qry.columnPermission().setValue( RfcDal::SqlInteger( permission.permission() ) );
	qry.columnStatus().setValue( RfcDal::SqlInteger( permission.status() ) );

	Auth4cDal::Connection conn = Auth4cDal::connection();

	try
	{
		conn.db().begin();

		qry.insert( conn );

		conn.db().commit();
	}
	catch( const Auth4cData::Exception& e )
	{
		conn.db().rollback();
		throw;
	}
	catch( const RfcDal::SqlException& e )
	{
		conn.db().rollback();
		Auth4cData::Exception et( __FILE__, __LINE__, Auth4cData::nNAX_DATABASE, RfcDal::SqlException::explainErrorCode( e.errorCode() ) );
		throw et;
	}
	catch( const RfcDal::DalDatabaseException& e )
	{
		conn.db().rollback();
		Auth4cData::Exception et( __FILE__, __LINE__, Auth4cData::nNAX_DATABASE, e.nativeExplanation() );
		throw et;
	}
	catch( ... )
	{
		conn.db().rollback();
		Auth4cData::Exception et( __FILE__, __LINE__, Auth4cData::nNAX_UNKNOWN, "unknown" );
		throw et;
	}
}

void Factory::deleteProduct( const Auth4cData::Product& product, bool bCascade /*= false*/ )
{
	if( !Auth4cData::Factory::hasProductPermission() )
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_PERMISSIONDENIED, "unauthorized action" );
	}

	Auth4cDal::ProductQuery qry( Auth4cDal::schema() );
	qry.constrainProduct( RfcDal::SqlInteger( product.product() ) );

	Auth4cDal::Connection conn = Auth4cDal::connection();

	try
	{
		conn.db().begin();

		if( bCascade )
		{
			// Must delete roles before permissions
			const Auth4cData::RoleMap& roleS( product.roles() );
			for( Auth4cData::RoleMap::const_iterator iterRole = roleS.begin();
					iterRole != roleS.end();
					++iterRole )
			{
				deleteRole( iterRole->second, bCascade );
			}

			const Auth4cData::PermissionMap& permissionS( product.permissions() );
			for( Auth4cData::PermissionMap::const_iterator iterPermission = permissionS.begin();
					iterPermission != permissionS.end();
					++iterPermission )
			{
				deletePermission( iterPermission->second, bCascade );
			}

			Auth4cDal::ProductParamQuery qryInit( Auth4cDal::schema() );
			qryInit.constrainProduct( RfcDal::SqlInteger( product.product() ) );
			qryInit.remove( conn );
		}

		if( 1 != qry.remove( conn ) )
		{
			throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_DATABASE, "Did not select exactly one row" );
		}

		conn.db().commit();
	}
	catch( const Auth4cData::Exception& e )
	{
		conn.db().rollback();
		throw;
	}
	catch( const RfcDal::SqlException& e )
	{
		conn.db().rollback();
		Auth4cData::Exception et( __FILE__, __LINE__, Auth4cData::nNAX_DATABASE, RfcDal::SqlException::explainErrorCode( e.errorCode() ) );
		throw et;
	}
	catch( const RfcDal::DalDatabaseException& e )
	{
		conn.db().rollback();
		Auth4cData::Exception et( __FILE__, __LINE__, Auth4cData::nNAX_DATABASE, e.nativeExplanation() );
		throw et;
	}
	catch( ... )
	{
		conn.db().rollback();
		Auth4cData::Exception et( __FILE__, __LINE__, Auth4cData::nNAX_UNKNOWN, "unknown" );
		throw et;
	}
}

void Factory::deleteProductParam( const Auth4cData::Product& product, const std::string& strProfile, const std::string& strName, bool bCascade /*= false*/ )
{
	if( !Auth4cData::Factory::hasProductPermission() )
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_PERMISSIONDENIED, "unauthorized action" );
	}

	Auth4cDal::ProductParamQuery qry( Auth4cDal::schema() );
	qry.constrainProduct( RfcDal::SqlInteger( product.product() ) );

	Auth4cDal::Connection conn = Auth4cDal::connection();

	try
	{
		conn.db().begin();

		if( bCascade )
		{
			qry.remove( conn );
		}
		else
		{
			qry.constrainProfile( RfcDal::SqlText( strProfile ) );
			qry.constrainParam( RfcDal::SqlText( strName ) );
			if( 1 != qry.remove( conn ) )
			{
				throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_DATABASE, "Did not select exactly one row" );
			}
		}

		conn.db().commit();
	}
	catch( const Auth4cData::Exception& e )
	{
		conn.db().rollback();
		throw;
	}
	catch( const RfcDal::SqlException& e )
	{
		conn.db().rollback();
		Auth4cData::Exception et( __FILE__, __LINE__, Auth4cData::nNAX_DATABASE, RfcDal::SqlException::explainErrorCode( e.errorCode() ) );
		throw et;
	}
	catch( const RfcDal::DalDatabaseException& e )
	{
		conn.db().rollback();
		Auth4cData::Exception et( __FILE__, __LINE__, Auth4cData::nNAX_DATABASE, e.nativeExplanation() );
		throw et;
	}
	catch( ... )
	{
		conn.db().rollback();
		Auth4cData::Exception et( __FILE__, __LINE__, Auth4cData::nNAX_UNKNOWN, "unknown" );
		throw et;
	}
}

void Factory::deletePermission( const Auth4cData::Permission& permission, bool bCascade /*= false*/ )
{
	if( !Auth4cData::Factory::hasProductPermission() )
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_PERMISSIONDENIED, "unauthorized action" );
	}

	Auth4cDal::ProductPermissionQuery qry( Auth4cDal::schema() );
	qry.constrainProduct( RfcDal::SqlInteger( permission.product().product() ) );
	qry.constrainPermission( RfcDal::SqlInteger( permission.permission() ) );

	Auth4cDal::Connection conn = Auth4cDal::connection();

	try
	{
		conn.db().begin();

		if( bCascade )
		{
			const Auth4cData::RoleMap& roleS( permission.product().roles() );
			for( Auth4cData::RoleMap::const_iterator iterRole = roleS.begin();
					iterRole != roleS.end();
					++iterRole )
			{
				Auth4cData::PermissionMap::const_iterator permFind = iterRole->second.permissions().find( permission.permission() );
				if( iterRole->second.permissions().end() != permFind )
				{
					deleteRolePermission( permFind->second );
				}
			}
		}

		if( 1 != qry.remove( conn ) )
		{
			throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_DATABASE, "Did not select exactly one row" );
		}

		conn.db().commit();
	}
	catch( const Auth4cData::Exception& e )
	{
		conn.db().rollback();
		throw;
	}
	catch( const RfcDal::SqlException& e )
	{
		conn.db().rollback();
		Auth4cData::Exception et( __FILE__, __LINE__, Auth4cData::nNAX_DATABASE, RfcDal::SqlException::explainErrorCode( e.errorCode() ) );
		throw et;
	}
	catch( const RfcDal::DalDatabaseException& e )
	{
		conn.db().rollback();
		Auth4cData::Exception et( __FILE__, __LINE__, Auth4cData::nNAX_DATABASE, e.nativeExplanation() );
		throw et;
	}
	catch( ... )
	{
		conn.db().rollback();
		Auth4cData::Exception et( __FILE__, __LINE__, Auth4cData::nNAX_UNKNOWN, "unknown" );
		throw et;
	}
}

void Factory::deleteRole( const Auth4cData::Role& role, bool bCascade /*= false*/ )
{
	if( !Auth4cData::Factory::hasProductPermission() )
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_PERMISSIONDENIED, "unauthorized action" );
	}

	Auth4cDal::ProductRoleQuery qry( Auth4cDal::schema() );
	qry.constrainProduct( RfcDal::SqlInteger( role.product().product() ) );
	qry.constrainRole( RfcDal::SqlInteger( role.role() ) );

	Auth4cDal::Connection conn = Auth4cDal::connection();

	try
	{
		conn.db().begin();

		if( bCascade )
		{
			const Auth4cData::PermissionMap& permissionS( role.permissions() );
			for( Auth4cData::PermissionMap::const_iterator iterPermission = permissionS.begin();
					iterPermission != permissionS.end();
					++iterPermission )
			{
				deleteRolePermission( iterPermission->second, bCascade );
			}

			Auth4cDal::TeamProductRoleQuery qryTeamRole( Auth4cDal::schema() );
			qryTeamRole.constrainProduct( RfcDal::SqlInteger( role.product().product() ) );
			qryTeamRole.constrainRole( RfcDal::SqlInteger( role.role() ) );
			qryTeamRole.remove( conn );
		}

		if( 1 != qry.remove( conn ) )
		{
			throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_DATABASE, "Did not select exactly one row" );
		}

		conn.db().commit();
	}
	catch( const Auth4cData::Exception& e )
	{
		conn.db().rollback();
		throw;
	}
	catch( const RfcDal::SqlException& e )
	{
		conn.db().rollback();
		Auth4cData::Exception et( __FILE__, __LINE__, Auth4cData::nNAX_DATABASE, RfcDal::SqlException::explainErrorCode( e.errorCode() ) );
		throw et;
	}
	catch( const RfcDal::DalDatabaseException& e )
	{
		conn.db().rollback();
		Auth4cData::Exception et( __FILE__, __LINE__, Auth4cData::nNAX_DATABASE, e.nativeExplanation() );
		throw et;
	}
	catch( ... )
	{
		conn.db().rollback();
		Auth4cData::Exception et( __FILE__, __LINE__, Auth4cData::nNAX_UNKNOWN, "unknown" );
		throw et;
	}
}

void Factory::deleteRolePermission( const Auth4cData::Permission& permission, bool bCascade /*=false*/ )
{
	if( !Auth4cData::Factory::hasProductPermission() )
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_PERMISSIONDENIED, "unauthorized action" );
	}

	Auth4cDal::ProductRolePermissionQuery qry( Auth4cDal::schema() );
	qry.constrainProduct( RfcDal::SqlInteger( permission.role().product().product() ) );
	qry.constrainRole( RfcDal::SqlInteger( permission.role().role() ) );

	Auth4cDal::Connection conn = Auth4cDal::connection();

	try
	{
		conn.db().begin();

		if( bCascade )
		{
			qry.remove( conn );
		}
		else 
		{
			qry.constrainPermission( RfcDal::SqlInteger( permission.permission() ) );

			if( 1 != qry.remove( conn ) )
			{
				throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_DATABASE, "Did not select exactly one row" );
			}
		}

		conn.db().commit();
	}
	catch( const Auth4cData::Exception& e )
	{
		conn.db().rollback();
		throw;
	}
	catch( const RfcDal::SqlException& e )
	{
		conn.db().rollback();
		Auth4cData::Exception et( __FILE__, __LINE__, Auth4cData::nNAX_DATABASE, RfcDal::SqlException::explainErrorCode( e.errorCode() ) );
		throw et;
	}
	catch( const RfcDal::DalDatabaseException& e )
	{
		conn.db().rollback();
		Auth4cData::Exception et( __FILE__, __LINE__, Auth4cData::nNAX_DATABASE, e.nativeExplanation() );
		throw et;
	}
	catch( ... )
	{
		conn.db().rollback();
		Auth4cData::Exception et( __FILE__, __LINE__, Auth4cData::nNAX_UNKNOWN, "unknown" );
		throw et;
	}
}

int Factory::nextPermission( Auth4cData::Product& product )
{
	int nNext( 1 );
	Auth4cDal::ProductPermissionQuery qry( Auth4cDal::schema() );
	RfcDal::DalDatabase::Result result;
	qry.constrainProduct( RfcDal::SqlInteger( product.product() ) );

	if( qry.select( result, Auth4cDal::connection(), RfcDal::SqlQueryBase::max ) )
	{
		RfcDal::DalDatabase::Row::const_iterator iter = result.begin()->begin();
		if( result.begin()->end() != iter )
		{
			nNext = iter->second.asInt() + 1;
		}
	}
	return nNext;
}

int Factory::nextRole( Auth4cData::Product& product )
{
	int nNext( 1 );
	Auth4cDal::ProductRoleQuery qry( Auth4cDal::schema() );
	RfcDal::DalDatabase::Result result;
	qry.constrainProduct( RfcDal::SqlInteger( product.product() ) );

	if( qry.select( result, Auth4cDal::connection(), RfcDal::SqlQueryBase::max ) )
	{
		RfcDal::DalDatabase::Row::const_iterator iter = result.begin()->begin();
		if( result.begin()->end() != iter )
		{
			nNext = iter->second.asInt() + 1;
		}
	}
	return nNext;
}

int Factory::nextProduct()
{
	int nNext( 1 );
	Auth4cDal::ProductQuery qry( Auth4cDal::schema() );
	RfcDal::DalDatabase::Result result;

	if( qry.select( result, Auth4cDal::connection(), RfcDal::SqlQueryBase::max ) )
	{
		RfcDal::DalDatabase::Row::const_iterator iter = result.begin()->begin();
		if( result.begin()->end() != iter )
		{
			nNext = iter->second.asInt() + 1;
		}
	}
	return nNext;
}

Base4c::Version Factory::version()
{
	return Base4c::Version( AUTH4CVERSION );
}
