/*

  $Id: Auth4cFactory.cpp 688 2018-11-28 17:10:21Z john $

  Copyright (c) 2008, The 4c Company. All rights reserved.
*/

#include <Auth4cDataXML/Auth4cFactory.h>
#include <Auth4cDataXML/Auth4cService.h>
#include <Auth4cDataXML/Util.h>

#include <Auth4cData/Factory.h>
#include <Auth4cData/SessionString.h>

#include <MakeVersion.in>

#include <librfc/XMLDoc.hpp>

#include <sstream>
#include <crypt.h>
#include <stdlib.h>

rsa_key g_key;

#define DBG std::cerr << __FILE__ << "@" << __LINE__ << "\n";

using namespace Auth4cDataXML;

Auth4cData::CustomerMap Factory::_customerS;
Auth4cData::DomainMap Factory::_domainS;
Auth4cData::ProfileMap Factory::_profileS;
Auth4cData::TeamMap Factory::_teamS;
Auth4cData::ProductMap Factory::_productS;

//////////////////////////////////////////////////////////////////////////
//
// Factory
//

std::map<std::string, Base4c::Version> Factory::about()
{
	std::map<std::string, Base4c::Version> versionS;
	std::ostringstream os;

	Rfc::XMLDoc docR;
	Rfc::XMLDoc docQ( "auth4cservice", "1.0" );
	Rfc::XMLNode* pnodeQ = docQ.rootNode();
	pnodeQ->attribute( "request", "about" );
	Auth4cDataXML::Auth4cService::transact( docR, docQ );

	Rfc::XMLNode* pnodeR = docR.rootNode();
	if( pnodeR->attribute( "response" ) == "about" )
	{
		Rfc::XMLNode* pnodeAbout = pnodeR->findChild( "about" );
		Rfc::XMLNode* pnodeLib( NULL );
		pnodeLib = pnodeAbout->nextChild( pnodeLib );
		while( NULL != pnodeLib )
		{
			os << (const char*)pnodeLib->nameSpace() << "::" << (const char*)pnodeLib->name();
			versionS[os.str()] = Base4c::Version::fromText( (const char*)pnodeLib->value() );
			os.str( "" );
			pnodeLib = pnodeAbout->nextChild( pnodeLib );
		}
	}

	return versionS;
}

void Factory::reinitialize()
{
	Rfc::XMLDoc docR;
	Rfc::XMLDoc docQ( "auth4cservice", "1.0" );
	Rfc::XMLNode* pnodeQ = docQ.rootNode();
	pnodeQ->attribute( "request", "reloadfactory" );
	Auth4cDataXML::Auth4cService::transact( docR, docQ );

	Rfc::XMLNode* pnodeR = docR.rootNode();
	if( pnodeR->attribute( "response" ) == "reloadfactory" )
	{
	}
	else
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_MALFORMEDXML, "No reloadfactory response" );
	}
}

void Factory::initialize( const std::string& strInit )
{
	try
	{
		Auth4cDataXML::Auth4cService::connect( strInit );
	}
	catch( const Base4c::Exception& e )
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_NOSERVERCONNECT, e.legend() );
	}
	catch( const RfcDal::DalDatabaseException& e )
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_DATABASE, "Database Error" );
	}

	Rfc::XMLDoc docR;
	Rfc::XMLDoc docQ( "auth4cservice", "1.0" );
	Rfc::XMLNode* pnodeQ = docQ.rootNode();
	pnodeQ->attribute( "request", "initialize" );
	pnodeQ->attribute( "version", version().text().c_str() );

	try
	{
		Auth4cDataXML::Auth4cService::transact( docR, docQ );
	}
	catch( const Base4c::Exception& e )
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_ALIENEXCEPTION, e.legend() );
	}

	Rfc::XMLNode* pnodeR = docR.rootNode();
	if( pnodeR->attribute( "response" ) == "rsa" )
	{
		g_key.type = PK_PRIVATE;
		int nLen;
		const Rfc::XMLNode* pnodeRSA = pnodeR->findChild( "rsa" );
		if( NULL == pnodeRSA )
		{
			throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_MALFORMEDXML, "No rsa node." );
		}

		const Rfc::XMLNode* pnodeE = pnodeRSA->findChild( "e" );
		if( NULL == pnodeE )
		{
			throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_MALFORMEDXML, "No e node in rsa node." );
		}

// We aren't expecting to get the private key across the wire
		const Rfc::XMLNode* pnodeD( NULL );
// Uncomment the following line if, for testing purposes only, the daemon has
// been modified to send the private key.
//		pnodeD = pnodeRSA->findChild( "d" );

		const Rfc::XMLNode* pnodeN = pnodeRSA->findChild( "N" );
		if( NULL == pnodeN )
		{
			throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_MALFORMEDXML, "No N node in rsa node." );
		}

		const Rfc::XMLNode* pnodeSign;
		const Rfc::XMLNode* pnodeAlloc;
		const Rfc::XMLNode* pnodeUsed;
		const Rfc::XMLNode* pnodeDP;

// e
		pnodeSign = pnodeE->findChild( "sign" );
		if( ( NULL == pnodeSign ) ||
			( pnodeSign->value().isNull() ) )
		{
			throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_MALFORMEDXML, "No sign node in e node in rsa node." );
		}

		pnodeAlloc = pnodeE->findChild( "alloc" );
		if( ( NULL == pnodeAlloc ) ||
			( pnodeAlloc->value().isNull() ) )
		{
			throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_MALFORMEDXML, "No alloc node in e node in rsa node." );
		}

		pnodeUsed = pnodeE->findChild( "used" );
		if( ( NULL == pnodeUsed ) ||
			( pnodeUsed->value().isNull() ) )
		{
			throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_MALFORMEDXML, "No used node in e node in rsa node." );
		}

		pnodeDP = pnodeE->findChild( "dp" );
		if( ( NULL == pnodeDP ) ||
			( pnodeDP->value().isNull() ) )
		{
			throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_MALFORMEDXML, "No dp node in e node in rsa node." );
		}

		g_key.e.sign = atoi( pnodeSign->value() );
		g_key.e.alloc = atoi( pnodeAlloc->value() );
		g_key.e.used = atoi( pnodeUsed->value() );
		g_key.e.dp = new unsigned short[g_key.e.alloc];
		nLen = g_key.e.alloc * 2;
		base64_decode( (unsigned char*)(const char*)pnodeDP->value(), pnodeDP->value().length(), (unsigned char*)g_key.e.dp, &nLen );

// d
// We aren't expecting to get the private key across the wire
		if( NULL != pnodeD )
		{
			pnodeSign = pnodeD->findChild( "sign" );
			if( ( NULL == pnodeSign ) ||
				( pnodeSign->value().isNull() ) )
			{
				throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_MALFORMEDXML, "No sign node in d node in rsa node." );
			}

			pnodeAlloc = pnodeD->findChild( "alloc" );
			if( ( NULL == pnodeAlloc ) ||
				( pnodeAlloc->value().isNull() ) )
			{
				throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_MALFORMEDXML, "No alloc node in d node in rsa node." );
			}

			pnodeUsed = pnodeD->findChild( "used" );
			if( ( NULL == pnodeUsed ) ||
				( pnodeUsed->value().isNull() ) )
			{
				throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_MALFORMEDXML, "No used node in d node in rsa node." );
			}

			pnodeDP = pnodeD->findChild( "dp" );
			if( ( NULL == pnodeDP ) ||
				( pnodeDP->value().isNull() ) )
			{
				throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_MALFORMEDXML, "No dp node in d node in rsa node." );
			}

			g_key.d.sign = atoi( pnodeSign->value() );
			g_key.d.alloc = atoi( pnodeAlloc->value() );
			g_key.d.used = atoi( pnodeUsed->value() );
			g_key.d.dp = new unsigned short[g_key.d.alloc];
			nLen = g_key.d.alloc * 2;
			base64_decode( (unsigned char*)(const char*)pnodeDP->value(), pnodeDP->value().length(), (unsigned char*)g_key.d.dp, &nLen );
		}

		g_key.d.sign = 0;
		g_key.d.alloc = 0;
		g_key.d.used = 0;
		g_key.d.dp = 0;

// N
		pnodeSign = pnodeN->findChild( "sign" );
		if( ( NULL == pnodeSign ) ||
			( pnodeSign->value().isNull() ) )
		{
			throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_MALFORMEDXML, "No sign node in N node in rsa node." );
		}

		pnodeAlloc = pnodeN->findChild( "alloc" );
		if( ( NULL == pnodeAlloc ) ||
			( pnodeAlloc->value().isNull() ) )
		{
			throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_MALFORMEDXML, "No alloc node in N node in rsa node." );
		}

		pnodeUsed = pnodeN->findChild( "used" );
		if( ( NULL == pnodeUsed ) ||
			( pnodeUsed->value().isNull() ) )
		{
			throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_MALFORMEDXML, "No used node in N node in rsa node." );
		}

		pnodeDP = pnodeN->findChild( "dp" );
		if( ( NULL == pnodeDP ) ||
			( pnodeDP->value().isNull() ) )
		{
			throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_MALFORMEDXML, "No dp node in N node in rsa node." );
		}

		g_key.N.sign = atoi( pnodeSign->value() );
		g_key.N.alloc = atoi( pnodeAlloc->value() );
		g_key.N.used = atoi( pnodeUsed->value() );
		g_key.N.dp = new unsigned short[g_key.N.alloc];
		nLen = g_key.N.alloc * 2;
		base64_decode( (unsigned char*)(const char*)pnodeDP->value(), pnodeDP->value().length(), (unsigned char*)g_key.N.dp, &nLen );

//		Auth4cDataXML::dumpkey( g_key );
	}
	else
	{
		const Rfc::XMLNode* pnodeException = pnodeR->findChild( "exception" );
		if( NULL != pnodeException )
		{
			Auth4cData::Exception e( Factory::Exception::fromXML( pnodeException ) );
			throw e;
		}
	}
}

Auth4cData::AuthenticatedUser& Factory::authenticate( const Auth4cData::User& user, const Auth4cData::AuthenticationToken& authToken, const Auth4cData::Profile& profile )
{
	static Auth4cData::AuthenticatedUser rtn;

	Rfc::XMLDoc docR;
	Rfc::XMLDoc docQ( "auth4cservice", "1.0" );
	Rfc::XMLNode* pnodeQ = docQ.rootNode();
	pnodeQ->attribute( "request", "authenticate" );

	unsigned char szEncrypt[4096];
	int nLenEncrypt = sizeof szEncrypt;
	unsigned char szEncode[4096];
	int nLenEncode = sizeof szEncode;
	unsigned char szToken[4096];

	strcpy( (char*)szToken, authToken.str().c_str() );

	if( rsa_exptmod( szToken, strlen( (const char*)szToken ), szEncrypt, &nLenEncrypt, PK_PUBLIC, &g_key) == CRYPT_ERROR)
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_AUTHENTICATIONFAILED, "Failed to encrypt authentication token." );
	}

	base64_encode( szEncrypt, nLenEncrypt, szEncode, &nLenEncode );
	szEncode[nLenEncode] = '\0';

	Factory::User::toXML( pnodeQ, user );
	pnodeQ->addNode( "authenticationtoken", (const char*)szEncode );
	Factory::Profile::toXML( pnodeQ, profile );

	try
	{
		Auth4cDataXML::Auth4cService::transact( docR, docQ );
	}
	catch( const Base4c::Exception& e )
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_ALIENEXCEPTION, e.legend() );
	}

	Rfc::XMLNode* pnodeR = docR.rootNode();
	if( pnodeR->attribute( "response" ) == "authenticateduser" )
	{
		const Rfc::XMLNode* pnodeAuthenticatedUser = pnodeR->findChild( "authenticateduser" );
		if( NULL == pnodeAuthenticatedUser )
		{
			throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_MALFORMEDXML, "No authenticated user node in authenticateduser response." );
		}

		rtn = Factory::AuthenticatedUser::fromXML( pnodeAuthenticatedUser );
	}
	else
	{
		const Rfc::XMLNode* pnodeException = pnodeR->findChild( "exception" );
		if( NULL != pnodeException )
		{
			Auth4cData::Exception e( Factory::Exception::fromXML( pnodeException ) );
			throw e;
		}
	}

	Auth4cData::Factory::setAuthenticatedUser( rtn );

	return rtn;
}

Auth4cData::AuthenticatedUser& Factory::validate( const Auth4cData::User& user, const Auth4cData::AuthenticationToken& authToken )
{
	static Auth4cData::AuthenticatedUser rtn;

	Rfc::XMLDoc docR;
	Rfc::XMLDoc docQ( "auth4cservice", "1.0" );
	Rfc::XMLNode* pnodeQ = docQ.rootNode();
	pnodeQ->attribute( "request", "validate" );

	unsigned char szEncrypt[4096];
	int nLenEncrypt = sizeof szEncrypt;
	unsigned char szEncode[4096];
	int nLenEncode = sizeof szEncode;
	unsigned char szToken[4096];

	strcpy( (char*)szToken, authToken.str().c_str() );

	if( rsa_exptmod( szToken, strlen( (const char*)szToken ), szEncrypt, &nLenEncrypt, PK_PUBLIC, &g_key) == CRYPT_ERROR)
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_AUTHENTICATIONFAILED, "Failed to encrypt authentication token." );
	}

	base64_encode( szEncrypt, nLenEncrypt, szEncode, &nLenEncode );
	szEncode[nLenEncode] = '\0';

	Factory::User::toXML( pnodeQ, user );
	pnodeQ->addNode( "authenticationtoken", (const char*)szEncode );

	Auth4cDataXML::Auth4cService::transact( docR, docQ );

	Rfc::XMLNode* pnodeR = docR.rootNode();
	if( pnodeR->attribute( "response" ) == "authenticateduser" )
	{
		const Rfc::XMLNode* pnodeAuthenticatedUser = pnodeR->findChild( "authenticateduser" );
		if( NULL == pnodeAuthenticatedUser )
		{
			throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_MALFORMEDXML, "No authenticated user node in authenticateduser response." );
		}

		rtn = Factory::AuthenticatedUser::fromXML( pnodeAuthenticatedUser );
	}
	else
	{
		const Rfc::XMLNode* pnodeException = pnodeR->findChild( "exception" );
		if( NULL != pnodeException )
		{
			Auth4cData::Exception e( Factory::Exception::fromXML( pnodeException ) );
			throw e;
		}
	}

//	Auth4cData::Factory::setAuthenticatedUser( rtn );

	return rtn;
}

Auth4cData::AuthenticatedUser& Factory::authenticatedUser()
{
	static Auth4cData::AuthenticatedUser rtn;

	Rfc::XMLDoc docR;
	Rfc::XMLDoc docQ( "auth4cservice", "1.0" );
	Rfc::XMLNode* pnodeQ = docQ.rootNode();
	pnodeQ->attribute( "request", "authenticateduser" );

	Auth4cDataXML::Auth4cService::transact( docR, docQ );

	Rfc::XMLNode* pnodeR = docR.rootNode();
	if( pnodeR->attribute( "response" ) == "authenticateduser" )
	{
		const Rfc::XMLNode* pnodeAuthenticatedUser = pnodeR->findChild( "authenticateduser" );
		if( NULL == pnodeAuthenticatedUser )
		{
			throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_MALFORMEDXML, "No authenticated user node in authenticateduser response." );
		}

		rtn = Factory::AuthenticatedUser::fromXML( pnodeAuthenticatedUser );
	}
	else
	{
		const Rfc::XMLNode* pnodeException = pnodeR->findChild( "exception" );
		if( NULL != pnodeException )
		{
			Auth4cData::Exception e( Factory::Exception::fromXML( pnodeException ) );
			throw e;
		}
	}

	return rtn;
}

bool Factory::passwd( const Auth4cData::AuthenticatedUser& user, Auth4cData::AuthenticationToken& authTokenCurrent, Auth4cData::AuthenticationToken& authTokenNew )
{
	bool bRtn( false );

	Rfc::XMLDoc docR;
	Rfc::XMLDoc docQ( "auth4cservice", "1.0" );
	Rfc::XMLNode* pnodeQ = docQ.rootNode();
	pnodeQ->attribute( "request", "passwd" );

	unsigned char szEncrypt[4096];
	int nLenEncrypt = sizeof szEncrypt;
	unsigned char szEncode[4096];
	int nLenEncode = sizeof szEncode;
	unsigned char szToken[4096];

	Factory::AuthenticatedUser::toXML( pnodeQ, user );

	strcpy( (char*)szToken, authTokenCurrent.str().c_str() );

	nLenEncrypt = sizeof szEncrypt;
	nLenEncode = sizeof szEncode;

	if( rsa_exptmod( szToken, strlen( (const char*)szToken ), szEncrypt, &nLenEncrypt, PK_PUBLIC, &g_key) == CRYPT_ERROR)
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_AUTHENTICATIONFAILED, "Failed to encrypt authentication token." );
	}

	base64_encode( szEncrypt, nLenEncrypt, szEncode, &nLenEncode );
	szEncode[nLenEncode] = '\0';

	pnodeQ->addNode( "authenticationtokencurrent", (const char*)szEncode );

	strcpy( (char*)szToken, authTokenNew.str().c_str() );

	nLenEncrypt = sizeof szEncrypt;
	nLenEncode = sizeof szEncode;

	if( rsa_exptmod( szToken, strlen( (const char*)szToken ), szEncrypt, &nLenEncrypt, PK_PUBLIC, &g_key) == CRYPT_ERROR)
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_AUTHENTICATIONFAILED, "Failed to encrypt authentication token." );
	}

	base64_encode( szEncrypt, nLenEncrypt, szEncode, &nLenEncode );
	szEncode[nLenEncode] = '\0';

	pnodeQ->addNode( "authenticationtokennew", (const char*)szEncode );

	Auth4cDataXML::Auth4cService::transact( docR, docQ );

	Rfc::XMLNode* pnodeR = docR.rootNode();
	if( pnodeR->attribute( "response" ) == "passwd" )
	{
		bRtn = true;
	}
	else
	{
		const Rfc::XMLNode* pnodeException = pnodeR->findChild( "exception" );
		if( NULL != pnodeException )
		{
			Auth4cData::Exception e( Factory::Exception::fromXML( pnodeException ) );
			throw e;
		}
	}

	return bRtn;
}

void Factory::persistSession( Auth4cData::AuthenticatedUser& userP )
{
	Rfc::XMLDoc docR;
	Rfc::XMLDoc docQ( "auth4cservice", "1.0" );
	Rfc::XMLNode* pnodeQ = docQ.rootNode();
	pnodeQ->attribute( "request", "persistsession" );

	Auth4cDataXML::Factory::AuthenticatedUser::toXML( pnodeQ, userP, true );

	Auth4cDataXML::Auth4cService::transact( docR, docQ );

	Rfc::XMLNode* pnodeR = docR.rootNode();
	const Rfc::XMLNode* pnodeException = pnodeR->findChild( "exception" );
	if( NULL != pnodeException )
	{
		Auth4cData::Exception e( Factory::Exception::fromXML( pnodeException ) );
		throw e;
	}
}

Auth4cData::AuthenticatedUser& Factory::retrieveSession( const Auth4cData::Session& session )
{
	static Auth4cData::AuthenticatedUser rtn;

	Rfc::XMLDoc docR;
	Rfc::XMLDoc docQ( "auth4cservice", "1.0" );
	Rfc::XMLNode* pnodeQ = docQ.rootNode();
	pnodeQ->attribute( "request", "retrievesession" );

	Auth4cDataXML::Factory::Session::toXML( pnodeQ, session );

	Auth4cDataXML::Auth4cService::transact( docR, docQ );

	Rfc::XMLNode* pnodeR = docR.rootNode();
	if( pnodeR->attribute( "response" ) == "authenticateduser" )
	{
		const Rfc::XMLNode* pnodeAuthenticatedUser = pnodeR->findChild( "authenticateduser" );
		if( NULL == pnodeAuthenticatedUser )
		{
			throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_MALFORMEDXML, "No authenticated user node in authenticateduser response." );
		}

		rtn = Factory::AuthenticatedUser::fromXML( pnodeAuthenticatedUser );
	}
	else
	{
		const Rfc::XMLNode* pnodeException = pnodeR->findChild( "exception" );
		if( NULL != pnodeException )
		{
			Auth4cData::Exception e( Factory::Exception::fromXML( pnodeException ) );
			throw e;
		}
	}

	Auth4cData::Factory::setAuthenticatedUser( rtn );

	return rtn;
}

void Factory::killSession( const Auth4cData::Session& session )
{
	static Auth4cData::AuthenticatedUser rtn;

	Rfc::XMLDoc docR;
	Rfc::XMLDoc docQ( "auth4cservice", "1.0" );
	Rfc::XMLNode* pnodeQ = docQ.rootNode();
	pnodeQ->attribute( "request", "killsession" );

	Auth4cDataXML::Factory::Session::toXML( pnodeQ, session );

	Auth4cDataXML::Auth4cService::transact( docR, docQ );

	Rfc::XMLNode* pnodeR = docR.rootNode();
	const Rfc::XMLNode* pnodeException = pnodeR->findChild( "exception" );
	if( NULL != pnodeException )
	{
		Auth4cData::Exception e( Factory::Exception::fromXML( pnodeException ) );
		throw e;
	}

}

void Factory::user( Auth4cData::User& userF, bool bRecurse /*= true*/ )
{
	Rfc::XMLDoc docR;
	Rfc::XMLDoc docQ( "auth4cservice", "1.0" );
	Rfc::XMLNode* pnodeQ = docQ.rootNode();
	pnodeQ->attribute( "request", "user" );
	pnodeQ->attribute( "recurse", bRecurse ? "Y" : "N" );

	User::toXML( pnodeQ, userF );

	Auth4cDataXML::Auth4cService::transact( docR, docQ );

	Rfc::XMLNode* pnodeR = docR.rootNode();
	if( pnodeR->attribute( "response" ) == "user" )
	{
		const Rfc::XMLNode* pnodeUser = pnodeR->findChild( "user" );
		if( NULL == pnodeUser )
		{
			throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_MALFORMEDXML, "No User node in user response." );
		}

		userF = Factory::User::fromXML( pnodeUser );
	}
	else
	{
		const Rfc::XMLNode* pnodeException = pnodeR->findChild( "exception" );
		if( NULL != pnodeException )
		{
			Auth4cData::Exception e( Factory::Exception::fromXML( pnodeException ) );
			throw e;
		}
	}
}

void Factory::team( Auth4cData::Team& team, bool bReload )
{
	Rfc::XMLDoc docR;
	Rfc::XMLDoc docQ( "auth4cservice", "1.0" );
	Rfc::XMLNode* pnodeQ = docQ.rootNode();
	pnodeQ->attribute( "request", "team" );
	pnodeQ->attribute( "reload", bReload ? "Y" : "N" );

	Team::toXML( pnodeQ, team );

	Auth4cDataXML::Auth4cService::transact( docR, docQ );

	Rfc::XMLNode* pnodeR = docR.rootNode();
	if( pnodeR->attribute( "response" ) == "team" )
	{
		const Rfc::XMLNode* pnodeTeam = pnodeR->findChild( "team" );
		if( NULL == pnodeTeam )
		{
			throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_MALFORMEDXML, "No Team node in team response." );
		}

		team = Factory::Team::fromXML( pnodeTeam );
	}
	else
	{
		const Rfc::XMLNode* pnodeException = pnodeR->findChild( "exception" );
		if( NULL != pnodeException )
		{
			Auth4cData::Exception e( Factory::Exception::fromXML( pnodeException ) );
			throw e;
		}
	}
}

Auth4cData::TeamMap& Factory::teams()
{
	Rfc::XMLDoc docR;
	Rfc::XMLDoc docQ( "auth4cservice", "1.0" );
	Rfc::XMLNode* pnodeQ = docQ.rootNode();
	pnodeQ->attribute( "request", "teams" );

	Auth4cDataXML::Auth4cService::transact( docR, docQ );

	Rfc::XMLNode* pnodeR = docR.rootNode();
	if( pnodeR->attribute( "response" ) == "teams" )
	{
		const Rfc::XMLNode* pnodeTeams = pnodeR->findChild( "teams" );
		if( NULL == pnodeTeams )
		{
			throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_MALFORMEDXML, "No Teams node in teams response." );
		}

		Auth4cData::TeamMap teamS = Factory::TeamMap::fromXML( pnodeTeams );
		for( Auth4cData::TeamMap::iterator iterTeamNew = teamS.begin();
				iterTeamNew != teamS.end();
				++iterTeamNew )
		{
			Auth4cData::TeamMap::iterator iterTeamG = _teamS.find( iterTeamNew->first );
			if( _teamS.end() == iterTeamG )
			{
				_teamS.insert( *iterTeamNew );
			}
		}

		return _teamS;
	}
	else
	{
		const Rfc::XMLNode* pnodeException = pnodeR->findChild( "exception" );
		if( NULL != pnodeException )
		{
			Auth4cData::Exception e( Factory::Exception::fromXML( pnodeException ) );
			throw e;
		}

		return _teamS;
	}

	return _teamS;
}

Auth4cData::ProductMap& Factory::products()
{
	Rfc::XMLDoc docR;
	Rfc::XMLDoc docQ( "auth4cservice", "1.0" );
	Rfc::XMLNode* pnodeQ = docQ.rootNode();
	pnodeQ->attribute( "request", "products" );

	Auth4cDataXML::Auth4cService::transact( docR, docQ );

	Rfc::XMLNode* pnodeR = docR.rootNode();
	if( pnodeR->attribute( "response" ) == "products" )
	{
		const Rfc::XMLNode* pnodeProducts = pnodeR->findChild( "products" );
		if( NULL == pnodeProducts )
		{
			throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_MALFORMEDXML, "No Products node in products response." );
		}

		Auth4cData::ProductMap productS = Factory::ProductMap::fromXML( pnodeProducts );
		for( Auth4cData::ProductMap::iterator iterProductNew = productS.begin();
				iterProductNew != productS.end();
				++iterProductNew )
		{
			Auth4cData::ProductMap::iterator iterProductG = _productS.find( iterProductNew->first );
			if( _productS.end() == iterProductG )
			{
				_productS.insert( *iterProductNew );
			}
		}

		return _productS;
	}
	else
	{
		const Rfc::XMLNode* pnodeException = pnodeR->findChild( "exception" );
		if( NULL != pnodeException )
		{
			Auth4cData::Exception e( Factory::Exception::fromXML( pnodeException ) );
			throw e;
		}

		return _productS;
	}

	return _productS;
}

Auth4cData::DomainMap& Factory::domains()
{
	Rfc::XMLDoc docR;
	Rfc::XMLDoc docQ( "auth4cservice", "1.0" );
	Rfc::XMLNode* pnodeQ = docQ.rootNode();
	pnodeQ->attribute( "request", "domains" );

	Auth4cDataXML::Auth4cService::transact( docR, docQ );

	Rfc::XMLNode* pnodeR = docR.rootNode();
	if( pnodeR->attribute( "response" ) == "domains" )
	{
		const Rfc::XMLNode* pnodeDomains = pnodeR->findChild( "domains" );
		if( NULL == pnodeDomains )
		{
			throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_MALFORMEDXML, "No Domains node in domains response." );
		}

		Auth4cData::DomainMap domainS = Factory::DomainMap::fromXML( pnodeDomains );
		for( Auth4cData::DomainMap::iterator iterDomainNew = domainS.begin();
				iterDomainNew != domainS.end();
				++iterDomainNew )
		{
			Auth4cData::DomainMap::iterator iterDomainG = _domainS.find( iterDomainNew->first );
			if( _domainS.end() == iterDomainG )
			{
				_domainS.insert( *iterDomainNew );
			}
		}

		return _domainS;
	}
	else
	{
		const Rfc::XMLNode* pnodeException = pnodeR->findChild( "exception" );
		if( NULL != pnodeException )
		{
			Auth4cData::Exception e( Factory::Exception::fromXML( pnodeException ) );
			throw e;
		}

		return _domainS;
	}

	return _domainS;
}

Auth4cData::CustomerMap& Factory::customers()
{
	Rfc::XMLDoc docR;
	Rfc::XMLDoc docQ( "auth4cservice", "1.0" );
	Rfc::XMLNode* pnodeQ = docQ.rootNode();
	pnodeQ->attribute( "request", "customers" );

	Auth4cDataXML::Auth4cService::transact( docR, docQ );

	Rfc::XMLNode* pnodeR = docR.rootNode();
	if( pnodeR->attribute( "response" ) == "customers" )
	{
		const Rfc::XMLNode* pnodeCustomers = pnodeR->findChild( "customers" );
		if( NULL == pnodeCustomers )
		{
			throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_MALFORMEDXML, "No Customers node in customers response." );
		}

		Auth4cData::CustomerMap customerS = Factory::CustomerMap::fromXML( pnodeCustomers );
		for( Auth4cData::CustomerMap::iterator iterCustomerNew = customerS.begin();
				iterCustomerNew != customerS.end();
				++iterCustomerNew )
		{
			Auth4cData::CustomerMap::iterator iterCustomerG = _customerS.find( iterCustomerNew->first );
			if( _customerS.end() == iterCustomerG )
			{
				_customerS.insert( *iterCustomerNew );
			}
		}

		return _customerS;
	}
	else
	{
		const Rfc::XMLNode* pnodeException = pnodeR->findChild( "exception" );
		if( NULL != pnodeException )
		{
			Auth4cData::Exception e( Factory::Exception::fromXML( pnodeException ) );
			throw e;
		}

		return _customerS;
	}

	return _customerS;
}

Auth4cData::ProfileMap& Factory::profiles()
{
	Rfc::XMLDoc docR;
	Rfc::XMLDoc docQ( "auth4cservice", "1.0" );
	Rfc::XMLNode* pnodeQ = docQ.rootNode();
	pnodeQ->attribute( "request", "profiles" );

	Auth4cDataXML::Auth4cService::transact( docR, docQ );

	Rfc::XMLNode* pnodeR = docR.rootNode();
	if( pnodeR->attribute( "response" ) == "profiles" )
	{
		const Rfc::XMLNode* pnodeProfiles = pnodeR->findChild( "profiles" );
		if( NULL == pnodeProfiles )
		{
			throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_MALFORMEDXML, "No Profiles node in profiles response." );
		}

		Auth4cData::ProfileMap profileS = Factory::ProfileMap::fromXML( pnodeProfiles );
		for( Auth4cData::ProfileMap::iterator iterProfileNew = profileS.begin();
				iterProfileNew != profileS.end();
				++iterProfileNew )
		{
			Auth4cData::ProfileMap::iterator iterProfileG = _profileS.find( iterProfileNew->first );
			if( _profileS.end() == iterProfileG )
			{
				_profileS.insert( *iterProfileNew );
			}
		}

		return _profileS;
	}
	else
	{
		const Rfc::XMLNode* pnodeException = pnodeR->findChild( "exception" );
		if( NULL != pnodeException )
		{
			Auth4cData::Exception e( Factory::Exception::fromXML( pnodeException ) );
			throw e;
		}

		return _profileS;
	}

	return _profileS;
}

Auth4cData::AuthenticatedUserMap Factory::who()
{
	Auth4cData::AuthenticatedUserMap rtn;

	Rfc::XMLDoc docR;
	Rfc::XMLDoc docQ( "auth4cservice", "1.0" );
	Rfc::XMLNode* pnodeQ = docQ.rootNode();
	pnodeQ->attribute( "request", "who" );

	Auth4cDataXML::Auth4cService::transact( docR, docQ );

	Rfc::XMLNode* pnodeR = docR.rootNode();
	if( pnodeR->attribute( "response" ) == "who" )
	{
		const Rfc::XMLNode* pnodeAuthenticatedUsers = pnodeR->findChild( "authenticatedusers" );
		if( NULL == pnodeAuthenticatedUsers )
		{
			throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_MALFORMEDXML, "No AuthenticatedUsers node in who response." );
		}

		rtn = Factory::AuthenticatedUserMap::fromXML( pnodeAuthenticatedUsers );
	}
	else
	{
		const Rfc::XMLNode* pnodeException = pnodeR->findChild( "exception" );
		if( NULL != pnodeException )
		{
			Auth4cData::Exception e( Factory::Exception::fromXML( pnodeException ) );
			throw e;
		}
	}

	return rtn;
}

Base4c::Version Factory::version()
{
	return Base4c::Version( AUTH4CVERSION );
}


//////////////////////////////////////////////////////////////////////////
//
// Exception
//

Auth4cData::Exception Factory::Exception::fromXML( const Rfc::XMLNode* pnode )
{
	if( pnode->name() != "exception" )
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_MALFORMEDXML, "Not an exception node." );
	}

	const Rfc::XMLNode* pnodeFile = pnode->findChild( "file" );
	if( ( NULL == pnodeFile ) ||
		( pnodeFile->value().isNull() ) )
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_MALFORMEDXML, "No file node in Exception." );
	}

	const Rfc::XMLNode* pnodeLine = pnode->findChild( "line" );
	if( ( NULL == pnodeLine ) ||
		( pnodeLine->value().isNull() ) ||
		( !atoi( pnodeLine->value() ) ) )
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_MALFORMEDXML, "No line node in Exception." );
	}

	const Rfc::XMLNode* pnodeErrorCode = pnode->findChild( "errorcode" );
	if( ( NULL == pnodeErrorCode ) ||
		( pnodeErrorCode->value().isNull() ) ||
		( !atoi( pnodeErrorCode->value() ) ) )
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_MALFORMEDXML, "No errorcode node in Exception." );
	}

	const Rfc::XMLNode* pnodeLegend = pnode->findChild( "legend" );
	if( ( NULL == pnodeLegend ) ||
		( pnodeLegend->value().isNull() ) )
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_MALFORMEDXML, "No legend node in Exception." );
	}

	Auth4cData::Exception rtn( (const char*)pnodeFile->value(),
								atoi( pnodeLine->value() ),
								atoi( pnodeErrorCode->value() ),
								(const char*)pnodeLegend->value(),
								true );

	return rtn;
}

void Factory::Exception::toXML( Rfc::XMLNode* pnode, const Auth4cData::Exception& in, bool bRecursive /*= false*/ )
{
	Rfc::XMLNode* pnodeMy = pnode->addNode( "exception", Rfc::String() );
	Rfc::String str;
	pnodeMy->addNode( "file", in.file().c_str() );
	str.format( "%i", in.line() );
	pnodeMy->addNode( "line", str );
	str.format( "%i", in.errorCode() );
	pnodeMy->addNode( "errorcode", str );
	pnodeMy->addNode( "legend", in.legend().c_str() );
	pnodeMy->addNode( "remote", ( in.remote() ? "y" : "n" ) );
}


//////////////////////////////////////////////////////////////////////////
//
// AuthenticatedUser
//

Auth4cData::AuthenticatedUser Factory::AuthenticatedUser::fromXML( const Rfc::XMLNode* pnode )
{
	if( pnode->name() != "authenticateduser" )
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_MALFORMEDXML, "Not an authenticateduser node." );
	}

	const Rfc::XMLNode* pnodeUser = pnode->findChild( "user" );
	if( NULL == pnodeUser )
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_MALFORMEDXML, "No user node in AuthenticatedUser." );
	}

	const Rfc::XMLNode* pnodeSession = pnode->findChild( "sessionstring" );
	if( NULL == pnodeSession )
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_MALFORMEDXML, "No session node in AuthenticatedUser." );
	}

	Auth4cData::Profile profile;

	const Rfc::XMLNode* pnodeProfile = pnode->findChild( "profile" );
	if( NULL != pnodeProfile )
	{
		profile = Profile::fromXML( pnodeProfile );
	}

	Auth4cData::User user( User::fromXML( pnodeUser ) );
	Auth4cData::SessionString session( SessionString::fromXML( pnodeSession ) );

	Auth4cData::AuthenticatedUser rtn( user, session, profile );

	return rtn;
}

void Factory::AuthenticatedUser::toXML( Rfc::XMLNode* pnode, const Auth4cData::AuthenticatedUser& in, bool bRecursive /*= false*/ )
{
	Rfc::XMLNode* pnodeMy = pnode->addNode( "authenticateduser", Rfc::String() );
	User::toXML( pnodeMy, in );
	SessionString::toXML( pnodeMy, (const Auth4cData::SessionString&)in.session() );
	Profile::toXML( pnodeMy, in.profile() );
}


//////////////////////////////////////////////////////////////////////////
//
// AuthenticatedUserMap
//

Auth4cData::AuthenticatedUserMap Factory::AuthenticatedUserMap::fromXML( const Rfc::XMLNode* pnode )
{
	Auth4cData::AuthenticatedUserMap rtn;

	if( pnode->name() != "authenticatedusers" )
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_MALFORMEDXML, "Not an authenticatedusers node." );
	}

	const Rfc::XMLNode* pnodeUser( NULL );
	pnodeUser = pnode->findChild( "authenticateduser", pnodeUser );
	while( NULL != pnodeUser )
	{
		Auth4cData::AuthenticatedUser user( AuthenticatedUser::fromXML( pnodeUser ) );
		rtn[user.key()] = user;

		pnodeUser = pnode->findChild( "authenticateduser", pnodeUser );
	}

	return rtn;
}

void Factory::AuthenticatedUserMap::toXML( Rfc::XMLNode* pnode, const Auth4cData::AuthenticatedUserMap& in, bool bRecursive /*= false*/ )
{
	Rfc::XMLNode* pnodeMy = pnode->addNode( "authenticatedusers", Rfc::String() );
	for( Auth4cData::AuthenticatedUserMap::const_iterator iter = in.begin();
			iter != in.end();
			++iter )
	{
		AuthenticatedUser::toXML( pnodeMy, iter->second );
	}
}


//////////////////////////////////////////////////////////////////////////
//
// User
//

Auth4cData::User Factory::User::fromXML( const Rfc::XMLNode* pnode )
{
	if( pnode->name() != "user" )
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_MALFORMEDXML, "Not a user node." );
	}

	const Rfc::XMLNode* pnodeDomain = pnode->findChild( "domain" );
	if( NULL == pnodeDomain )
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_MALFORMEDXML, "No domain node in User." );
	}

	const Rfc::XMLNode* pnodeUserId = pnode->findChild( "userid" );
	if( ( NULL == pnodeUserId ) ||
		( pnodeUserId->value().isNull() ) )
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_MALFORMEDXML, "No userid node in User." );
	}

	const Rfc::XMLNode* pnodeName = pnode->findChild( "name" );
	if( NULL == pnodeName )
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_MALFORMEDXML, "No name node in User." );
	}

	const Rfc::XMLNode* pnodeAuthenticationToken = pnode->findChild( "authenticationtoken" );
	// if the authentication token is hidden, the node will be absent

	const Rfc::XMLNode* pnodeContact = pnode->findChild( "contact" );
	if( NULL == pnodeContact )
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_MALFORMEDXML, "No contact node in User." );
	}

	const Rfc::XMLNode* pnodeTeamS = pnode->findChild( "teams" );
	// doesn't have to have teams

	const Rfc::XMLNode* pnodeProductS = pnode->findChild( "products" );
	// doesn't have to have products

	Auth4cData::Domain domain( Domain::fromXML( pnodeDomain ) );
	_domainS[domain.domain()] = domain;
	Auth4cData::Domain& domainRef( _domainS[domain.domain()] );

	Auth4cData::User rtn( domainRef, (const char*)pnodeUserId->value() );

	if( !pnodeName->value().isNull() )
	{
		rtn.setName( (const char*)pnodeName->value() );
	}

	if( ( NULL != pnodeAuthenticationToken ) &&
		( !pnodeAuthenticationToken->value().isNull() ) )
	{
		rtn.setAuthenticationToken( (const char*)pnodeAuthenticationToken->value() );
	}

	if( !pnodeContact->value().isNull() )
	{
		rtn.setContact( (const char*)pnodeContact->value() );
	}

	if( NULL != pnodeTeamS )
	{
		Auth4cData::TeamMap teamS( TeamMap::fromXML( pnodeTeamS ) );
		rtn.teams() = teamS;
	}

	// Merge all the roles and permissions
//	for( Auth4cData::TeamMap::const_iterator iterTeam = rtn.teams().begin();
//			iterTeam != rtn.teams().end();
//			++iterTeam )
//	{
//		for( Auth4cData::ProductMap::const_iterator iterProduct = iterTeam->second.products().begin();
//				iterProduct != iterTeam->second.products().end();
//				++iterProduct )
//		{
//			rtn.products()[iterProduct->first] = iterProduct->second;
//		}
//	}

	if( NULL != pnodeProductS )
	{
		Auth4cData::ProductMap productS( ProductMap::fromXML( pnodeProductS ) );
		rtn.products() = productS;
	}

	return rtn;
}

void Factory::User::toXML( Rfc::XMLNode* pnode, const Auth4cData::User& in, bool bRecursive /*= false*/ )
{
	Rfc::XMLNode* pnodeMy = pnode->addNode( "user", Rfc::String() );

	try
	{
		Domain::toXML( pnodeMy, in.domain() );
	}
	catch( const Auth4cData::Exception& e )
	{
		throw;
		//@TODO issue a warning?
	}

	pnodeMy->addNode( "userid", in.userId().c_str() );
	pnodeMy->addNode( "name", in.name().c_str() );

	try
	{
		pnodeMy->addNode( "authenticationtoken", in.authenticationToken().c_str() );
	}
	catch( const Auth4cData::Exception& e )
	{
		// If the authentication token is hidden...
/// @TODO: Get the correct error code
/*		if( e.errorCode() == nNAX )
		{
		}
		else
		{
			throw
		}*/
	}
	catch( ... )
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_UNKNOWN, "Unknown Error." );
	}

	pnodeMy->addNode( "contact", in.contact().c_str() );

	TeamMap::toXML( pnodeMy, in.teams() );
	ProductMap::toXML( pnodeMy, in.products() );
}


//////////////////////////////////////////////////////////////////////////
//
// UserMap
//

Auth4cData::UserMap Factory::UserMap::fromXML( const Rfc::XMLNode* pnode )
{
	Auth4cData::UserMap rtn;

	if( pnode->name() != "users" )
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_MALFORMEDXML, "Not a users node." );
	}

	const Rfc::XMLNode* pnodeUser( NULL );
	pnodeUser = pnode->findChild( "user", pnodeUser );
	while( NULL != pnodeUser )
	{
		Auth4cData::User user( User::fromXML( pnodeUser ) );
		rtn[user.key()] = user;

		pnodeUser = pnode->findChild( "user", pnodeUser );
	}

	return rtn;
}

void Factory::UserMap::toXML( Rfc::XMLNode* pnode, const Auth4cData::UserMap& in, bool bRecursive /*= false*/ )
{
	Rfc::XMLNode* pnodeMy = pnode->addNode( "users", Rfc::String() );
	for( Auth4cData::UserMap::const_iterator iter = in.begin();
			iter != in.end();
			++iter )
	{
		User::toXML( pnodeMy, iter->second );
	}
}


//////////////////////////////////////////////////////////////////////////
//
// Session
//

Auth4cData::Session Factory::Session::fromXML( const Rfc::XMLNode* pnode )
{
	if( pnode->name() != "session" )
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_MALFORMEDXML, "Not a sessionstring node." );
	}

	const Rfc::XMLNode* pnodeSessionId = pnode->findChild( "sessionid" );
	if( ( NULL == pnodeSessionId ) ||
		( pnodeSessionId->value().isNull() ) )
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_MALFORMEDXML, "No sessionid node in SessionString." );
	}

	return Auth4cData::Session( (const char*)pnodeSessionId->value() );
}

void Factory::Session::toXML( Rfc::XMLNode* pnode, const Auth4cData::Session& in, bool bRecursive /*= false*/ )
{
	Rfc::XMLNode* pnodeMy = pnode->addNode( "session", Rfc::String() );

	pnodeMy->addNode( "sessionid", in.str().c_str() );
}


//////////////////////////////////////////////////////////////////////////
//
// SessionString
//

Auth4cData::SessionString Factory::SessionString::fromXML( const Rfc::XMLNode* pnode )
{
	if( pnode->name() != "sessionstring" )
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_MALFORMEDXML, "Not a sessionstring node." );
	}

	const Rfc::XMLNode* pnodeSession = pnode->findChild( "session" );
	if( NULL == pnodeSession )
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_MALFORMEDXML, "No session node in SessionString." );
	}

	Auth4cData::Session session( Factory::Session::fromXML( pnodeSession ) );

	return Auth4cData::SessionString( session.str() );
}

void Factory::SessionString::toXML( Rfc::XMLNode* pnode, const Auth4cData::SessionString& in, bool bRecursive /*= false*/ )
{
	Rfc::XMLNode* pnodeMy = pnode->addNode( "sessionstring", Rfc::String() );
	Factory::Session::toXML( pnodeMy, in );
}


//////////////////////////////////////////////////////////////////////////
//
// Profile
//

Auth4cData::Profile Factory::Profile::fromXML( const Rfc::XMLNode* pnode )
{
	if( pnode->name() != "profile" )
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_MALFORMEDXML, "Not a profile node." );
	}

	const Rfc::XMLNode* pnodeProfile = pnode->findChild( "profile" );
	if( ( NULL == pnodeProfile ) ||
		( pnodeProfile->value().isNull() ) )
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_MALFORMEDXML, "No profile node in Profile." );
	}

	Auth4cData::Profile rtn( (const char*)pnodeProfile->value() );

	const Rfc::XMLNode* pnodeProducts = pnode->findChild( "productinitmap" );
	if( NULL != pnodeProducts )
	{
		Auth4cData::ProductInitMap productS( ProductInitMap::fromXML( pnodeProducts ) );
		rtn.setProducts( productS );
	}

	return rtn;
}

void Factory::Profile::toXML( Rfc::XMLNode* pnode, const Auth4cData::Profile& in, bool bRecursive /*= false*/ )
{
	Rfc::XMLNode* pnodeMy = pnode->addNode( "profile", Rfc::String() );

	pnodeMy->addNode( "profile", in.profile().c_str() );
	ProductInitMap::toXML( pnodeMy, in.products() );
}


//////////////////////////////////////////////////////////////////////////
//
// ProfileMap
//

Auth4cData::ProfileMap Factory::ProfileMap::fromXML( const Rfc::XMLNode* pnode )
{
	Auth4cData::ProfileMap rtn;

	if( pnode->name() != "profiles" )
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_MALFORMEDXML, "Not a profiles node." );
	}

	const Rfc::XMLNode* pnodeProfile( NULL );
	pnodeProfile = pnode->findChild( "profile", pnodeProfile );
	while( NULL != pnodeProfile )
	{
		Auth4cData::Profile profile( Profile::fromXML( pnodeProfile ) );
		rtn[profile.profile()] = profile;

		pnodeProfile = pnode->findChild( "profile", pnodeProfile );
	}

	return rtn;
}

void Factory::ProfileMap::toXML( Rfc::XMLNode* pnode, const Auth4cData::ProfileMap& in, bool bRecursive /*= false*/ )
{
	Rfc::XMLNode* pnodeMy = pnode->addNode( "profiles", Rfc::String() );
	for( Auth4cData::ProfileMap::const_iterator iter = in.begin();
			iter != in.end();
			++iter )
	{
		Profile::toXML( pnodeMy, iter->second );
	}
}


//////////////////////////////////////////////////////////////////////////
//
// Domain
//

Auth4cData::Domain Factory::Domain::fromXML( const Rfc::XMLNode* pnode )
{
	if( pnode->name() != "domain" )
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_MALFORMEDXML, "Not a domain node." );
	}

	const Rfc::XMLNode* pnodeCustomer = pnode->findChild( "customer" );
	if( NULL == pnodeCustomer )
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_MALFORMEDXML, "No customer node in Domain." );
	}

	const Rfc::XMLNode* pnodeDomain = pnode->findChild( "domain" );
	if( ( NULL == pnodeDomain ) ||
		( pnodeDomain->value().isNull() ) )
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_MALFORMEDXML, "No domain node in Domain." );
	}

	std::string strDescription;

	const Rfc::XMLNode* pnodeDescription = pnode->findChild( "description" );
	if( ( NULL != pnodeDescription ) &&
		( !pnodeDescription->value().isNull() ) )
	{
		strDescription = (const char*)pnodeDescription->value();
	}

	Auth4cData::Customer customer( Customer::fromXML( pnodeCustomer ) );
	_customerS[customer.customer()] = customer;
	Auth4cData::Customer& customerRef( _customerS[customer.customer()] );

	Auth4cData::Domain rtn( customerRef, (const char*)pnodeDomain->value() );
	rtn.setDescription( strDescription );

	return rtn;
}

void Factory::Domain::toXML( Rfc::XMLNode* pnode, const Auth4cData::Domain& in, bool bRecursive /*= false*/ )
{
	Rfc::XMLNode* pnodeMy = pnode->addNode( "domain", Rfc::String() );

	try
	{
		Customer::toXML( pnodeMy, in.customer() );
	}
	catch( const Auth4cData::Exception& e )
	{
		throw;
		//@TODO
	}
	catch( ... )
	{
	}

	pnodeMy->addNode( "domain", in.domain().c_str() );
	pnodeMy->addNode( "description", in.description().c_str() );
}


//////////////////////////////////////////////////////////////////////////
//
// DomainMap
//

Auth4cData::DomainMap Factory::DomainMap::fromXML( const Rfc::XMLNode* pnode )
{
	Auth4cData::DomainMap rtn;

	if( pnode->name() != "domains" )
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_MALFORMEDXML, "Not a domains node." );
	}

	const Rfc::XMLNode* pnodeDomain( NULL );
	pnodeDomain = pnode->findChild( "domain", pnodeDomain );
	while( NULL != pnodeDomain )
	{
		Auth4cData::Domain domain( Domain::fromXML( pnodeDomain ) );
		rtn[domain.domain()] = domain;

		pnodeDomain = pnode->findChild( "domain", pnodeDomain );
	}

	return rtn;
}

void Factory::DomainMap::toXML( Rfc::XMLNode* pnode, const Auth4cData::DomainMap& in, bool bRecursive /*= false*/ )
{
	Rfc::XMLNode* pnodeMy = pnode->addNode( "domains", Rfc::String() );
	for( Auth4cData::DomainMap::const_iterator iter = in.begin();
			iter != in.end();
			++iter )
	{
		Domain::toXML( pnodeMy, iter->second );
	}
}


//////////////////////////////////////////////////////////////////////////
//
// DomainPtrMap
//

Auth4cData::DomainPtrMap Factory::DomainPtrMap::fromXML( const Rfc::XMLNode* pnode )
{
	Auth4cData::DomainPtrMap rtn;

	if( pnode->name() != "domainptrs" )
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_MALFORMEDXML, "Not a domainptrs node." );
	}

	const Rfc::XMLNode* pnodeDomain( NULL );
	pnodeDomain = pnode->findChild( "domain", pnodeDomain );
	while( NULL != pnodeDomain )
	{
		Auth4cData::Domain domain( Domain::fromXML( pnodeDomain ) );
		Auth4cData::DomainMap::iterator iterDomain = Auth4cData::Factory::domains().find( domain.domain() );
		rtn[iterDomain->second.domain()] = &iterDomain->second;

		pnodeDomain = pnode->findChild( "domain", pnodeDomain );
	}

	return rtn;
}

void Factory::DomainPtrMap::toXML( Rfc::XMLNode* pnode, const Auth4cData::DomainPtrMap& in, bool bRecursive /*= false*/ )
{
	Rfc::XMLNode* pnodeMy = pnode->addNode( "domainptrs", Rfc::String() );
	for( Auth4cData::DomainPtrMap::const_iterator iter = in.begin();
			iter != in.end();
			++iter )
	{
		Domain::toXML( pnodeMy, *iter->second );
	}
}


//////////////////////////////////////////////////////////////////////////
//
// Customer
//

Auth4cData::Customer Factory::Customer::fromXML( const Rfc::XMLNode* pnode )
{
	if( pnode->name() != "customer" )
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_MALFORMEDXML, "Not a customer node." );
	}

	const Rfc::XMLNode* pnodeCustomer = pnode->findChild( "customer" );
	if( ( NULL == pnodeCustomer ) ||
		( pnodeCustomer->value().isNull() ) )
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_MALFORMEDXML, "No customer node in Customer." );
	}

	std::string strName;

	const Rfc::XMLNode* pnodeName = pnode->findChild( "name" );
	if( ( NULL != pnodeName ) &&
		( !pnodeName->value().isNull() ) )
	{
		strName = (const char*)pnodeName->value();
	}

	Auth4cData::Customer rtn( (const char*)pnodeCustomer->value(),
								strName );

	/*const Rfc::XMLNode* pnodeDomainS = pnode->findChild( "domains" );
	if( NULL != pnodeDomainS )
	{
		// We need to update the existing map with the values in the new map, rather than overwrite the existing map
		// as a number of classes maintain pointers to objects that could otherwise move.
		Auth4cData::DomainMap domainS( DomainMap::fromXML( pnodeDomainS ) );
		for( Auth4cData::DomainMap::const_iterator iterDomain = domainS.begin();
				iterDomain != domainS.end();
				++iterDomain )
		{
			_domainS[iterDomain->first] = iterDomain->second;
		}
	}*/

	const Rfc::XMLNode* pnodeDomainS = pnode->findChild( "domainptrs" );
	if( NULL != pnodeDomainS )
	{
		Auth4cData::DomainPtrMap domainS( DomainPtrMap::fromXML( pnodeDomainS ) );

		rtn.domains() = domainS;
	}

	const Rfc::XMLNode* pnodeTeamS = pnode->findChild( "teamptrs" );
	if( NULL != pnodeTeamS )
	{
		Auth4cData::TeamPtrMap teamS( TeamPtrMap::fromXML( pnodeTeamS ) );

		rtn.teams() = teamS;
	}

	return rtn;
}

void Factory::Customer::toXML( Rfc::XMLNode* pnode, const Auth4cData::Customer& in, bool bRecursive /*= false*/ )
{
	Rfc::XMLNode* pnodeMy = pnode->addNode( "customer", Rfc::String() );

	pnodeMy->addNode( "customer", in.customer().c_str() );
	pnodeMy->addNode( "name", in.name().c_str() );

	if( bRecursive )
	{
		DomainPtrMap::toXML( pnodeMy, in.domains() );
		TeamPtrMap::toXML( pnodeMy, in.teams() );
	}
}


//////////////////////////////////////////////////////////////////////////
//
// CustomerMap
//

Auth4cData::CustomerMap Factory::CustomerMap::fromXML( const Rfc::XMLNode* pnode )
{
	Auth4cData::CustomerMap rtn;

	if( pnode->name() != "customers" )
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_MALFORMEDXML, "Not a customers node." );
	}

	const Rfc::XMLNode* pnodeCustomer( NULL );
	pnodeCustomer = pnode->findChild( "customer", pnodeCustomer );
	while( NULL != pnodeCustomer )
	{
		Auth4cData::Customer customer( Customer::fromXML( pnodeCustomer ) );
		rtn[customer.customer()] = customer;

		pnodeCustomer = pnode->findChild( "customer", pnodeCustomer );
	}

	return rtn;
}

void Factory::CustomerMap::toXML( Rfc::XMLNode* pnode, const Auth4cData::CustomerMap& in, bool bRecursive /*= false*/ )
{
	Rfc::XMLNode* pnodeMy = pnode->addNode( "customers", Rfc::String() );
	for( Auth4cData::CustomerMap::const_iterator iter = in.begin();
			iter != in.end();
			++iter )
	{
		Customer::toXML( pnodeMy, iter->second );
	}
}


//////////////////////////////////////////////////////////////////////////
//
// Team
//

Auth4cData::Team Factory::Team::fromXML( const Rfc::XMLNode* pnode )
{
	if( pnode->name() != "team" )
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_MALFORMEDXML, "Not a team node." );
	}

	const Rfc::XMLNode* pnodeTeam = pnode->findChild( "team" );
	if( ( NULL == pnodeTeam ) ||
		( pnodeTeam->value().isNull() ) )
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_MALFORMEDXML, "No team node in Team." );
	}

	const Rfc::XMLNode* pnodeCustomer = pnode->findChild( "customer" );
	if( NULL == pnodeCustomer )
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_MALFORMEDXML, "No customer node in Team." );
	}

	const Rfc::XMLNode* pnodeName = pnode->findChild( "name" );
	if( ( NULL == pnodeName ) ||
		( pnodeName->value().isNull() ) )
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_MALFORMEDXML, "No name node in Team." );
	}

	const Rfc::XMLNode* pnodeMemberS = pnode->findChild( "teammembers" ); //optional
	const Rfc::XMLNode* pnodeProductS = pnode->findChild( "products" ); //optional

	Auth4cData::Customer customer( Customer::fromXML( pnodeCustomer ) );
	_customerS[customer.customer()] = customer;
	Auth4cData::Customer& customerRef( _customerS[customer.customer()] );

	Auth4cData::Team rtn( customerRef, atoi( pnodeTeam->value() ) );

	if( !pnodeName->value().isNull() )
	{
		rtn.setName( (const char*)pnodeName->value() );
	}

	if( NULL != pnodeMemberS )
	{
		Auth4cData::TeamMemberMap memberS( TeamMemberMap::fromXML( pnodeMemberS ) );
		rtn.members() = memberS;
	}

	if( NULL != pnodeProductS )
	{
		Auth4cData::ProductMap productS( ProductMap::fromXML( pnodeProductS ) );
		rtn.products() = productS;
	}

	_teamS[rtn.team()] = rtn;

	return rtn;
}

void Factory::Team::toXML( Rfc::XMLNode* pnode, const Auth4cData::Team& in, bool bRecursive /*= false*/ )
{
	Rfc::XMLNode* pnodeMy = pnode->addNode( "team", Rfc::String() );

	Rfc::String str;
	str.format( "%i", in.team() );
	pnodeMy->addNode( "team", (const char*)str );
	Customer::toXML( pnodeMy, in.customer() );
	pnodeMy->addNode( "name", in.name().c_str() );

	if( bRecursive )
	{
		TeamMemberMap::toXML( pnodeMy, in.members() );
		ProductMap::toXML( pnodeMy, in.products() );
	}
}


//////////////////////////////////////////////////////////////////////////
//
// TeamMap
//

Auth4cData::TeamMap Factory::TeamMap::fromXML( const Rfc::XMLNode* pnode )
{
	Auth4cData::TeamMap rtn;

	if( pnode->name() != "teams" )
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_MALFORMEDXML, "Not a teams node." );
	}

	const Rfc::XMLNode* pnodeTeam( NULL );
	pnodeTeam = pnode->findChild( "team", pnodeTeam );
	while( NULL != pnodeTeam )
	{
		Auth4cData::Team team( Team::fromXML( pnodeTeam ) );
		rtn[team.team()] = team;

		pnodeTeam = pnode->findChild( "team", pnodeTeam );
	}

	return rtn;
}

void Factory::TeamMap::toXML( Rfc::XMLNode* pnode, const Auth4cData::TeamMap& in, bool bRecursive /*= false*/ )
{
	Rfc::XMLNode* pnodeMy = pnode->addNode( "teams", Rfc::String() );
	for( Auth4cData::TeamMap::const_iterator iter = in.begin();
			iter != in.end();
			++iter )
	{
		Team::toXML( pnodeMy, iter->second );
	}
}


//////////////////////////////////////////////////////////////////////////
//
// TeamPtrMap
//

Auth4cData::TeamPtrMap Factory::TeamPtrMap::fromXML( const Rfc::XMLNode* pnode )
{
	Auth4cData::TeamPtrMap rtn;

	if( pnode->name() != "teamptrs" )
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_MALFORMEDXML, "Not a teamptrs node." );
	}

	const Rfc::XMLNode* pnodeTeam( NULL );
	pnodeTeam = pnode->findChild( "team", pnodeTeam );
	while( NULL != pnodeTeam )
	{
		Auth4cData::Team team( Team::fromXML( pnodeTeam ) );
		Auth4cData::TeamMap::iterator iterTeam = Auth4cData::Factory::teams().find( team.team() );
		rtn[iterTeam->second.team()] = &iterTeam->second;

		pnodeTeam = pnode->findChild( "team", pnodeTeam );
	}

	return rtn;
}

void Factory::TeamPtrMap::toXML( Rfc::XMLNode* pnode, const Auth4cData::TeamPtrMap& in, bool bRecursive /*= false*/ )
{
	Rfc::XMLNode* pnodeMy = pnode->addNode( "teamptrs", Rfc::String() );
	for( Auth4cData::TeamPtrMap::const_iterator iter = in.begin();
			iter != in.end();
			++iter )
	{
		Team::toXML( pnodeMy, *iter->second );
	}
}


//////////////////////////////////////////////////////////////////////////
//
// TeamMember
//

Auth4cData::TeamMember Factory::TeamMember::fromXML( const Rfc::XMLNode* pnode )
{
	if( pnode->name() != "teammember"  )
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_MALFORMEDXML, "Not a team member node." );
	}

	const Rfc::XMLNode* pnodeUser = pnode->findChild( "user" );
	if( NULL == pnodeUser )
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_MALFORMEDXML, "No user node in TeamMember." );
	}

	const Rfc::XMLNode* pnodeIsLead = pnode->findChild( "islead" );
	if( ( NULL == pnodeIsLead ) ||
		( pnodeIsLead->value().isNull() ) )
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_MALFORMEDXML, "No islead node in TeamMember." );
	}

	Auth4cData::User user( User::fromXML( pnodeUser ) );

	Auth4cData::TeamMember rtn( user, pnodeIsLead->value() == "Y" );

	return rtn;
}

void Factory::TeamMember::toXML( Rfc::XMLNode* pnode, const Auth4cData::TeamMember& in, bool bRecursive /*= false*/ )
{
	Rfc::XMLNode* pnodeMy = pnode->addNode( "teammember", Rfc::String() );
	User::toXML( pnodeMy, in );
	pnodeMy->addNode( "islead", ( in.isLead() ? "Y" : "N" ) );
}


//////////////////////////////////////////////////////////////////////////
//
// TeamMemberMap
//

Auth4cData::TeamMemberMap Factory::TeamMemberMap::fromXML( const Rfc::XMLNode* pnode )
{
	Auth4cData::TeamMemberMap rtn;

	if( pnode->name() != "teammembers"  )
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_MALFORMEDXML, "Not a teammembers node." );
	}

	const Rfc::XMLNode* pnodeMember( NULL );
	pnodeMember = pnode->findChild( "teammember", pnodeMember );
	while( NULL != pnodeMember )
	{
		Auth4cData::TeamMember teammember( TeamMember::fromXML( pnodeMember ) );
		rtn[teammember.key()] = teammember;

		pnodeMember = pnode->findChild( "teammember", pnodeMember );
	}

	return rtn;
}

void Factory::TeamMemberMap::toXML( Rfc::XMLNode* pnode, const Auth4cData::TeamMemberMap& in, bool bRecursive /*= false*/ )
{
	Rfc::XMLNode* pnodeMy = pnode->addNode( "teammembers", Rfc::String() );
	for( Auth4cData::TeamMemberMap::const_iterator iter = in.begin();
			iter != in.end();
			++iter )
	{
		TeamMember::toXML( pnodeMy, iter->second );
	}
}


//////////////////////////////////////////////////////////////////////////
//
// Permission
//

Auth4cData::Permission Factory::Permission::fromXML( const Rfc::XMLNode* pnode )
{
	if( pnode->name() != "permission" )
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_MALFORMEDXML, "Not a permission node." );
	}

	const Rfc::XMLNode* pnodePermission = pnode->findChild( "permission" );
	if( ( NULL == pnodePermission ) ||
		( pnodePermission->value().isNull() ) )
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_MALFORMEDXML, "No permission node in Permission." );
	}

	const Rfc::XMLNode* pnodeDescription = pnode->findChild( "description" );
//	if( NULL == pnodeDescription )
//	{
//		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_MALFORMEDXML, "No description node in Permission." );
//	}

	const Rfc::XMLNode* pnodeStatus = pnode->findChild( "status" );
	if( ( NULL == pnodeStatus ) ||
		( pnodeStatus->value().isNull() ) )
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_MALFORMEDXML, "No status node in Permission." );
	}

	Auth4cData::Product* pproduct( NULL ); // Could just as easily be Role...
	Auth4cData::Permission rtn( *pproduct, atoi( pnodePermission->value() ) );

	if( ( NULL != pnodeDescription ) &&
		( !pnodeDescription->value().isNull() ) )
	{
		rtn.setDescription( (const char*)pnodeDescription->value() );
	}

	rtn.setStatus( Auth4cData::PRStatus::fromInt( atoi( pnodeStatus->value() ) ) );
	
	return rtn;
}

void Factory::Permission::toXML( Rfc::XMLNode* pnode, const Auth4cData::Permission& in, bool bRecursive /*= false*/ )
{
	Rfc::XMLNode* pnodeMy = pnode->addNode( "permission", Rfc::String() );

	Rfc::String str;
	str.format( "%i", in.permission() );
	pnodeMy->addNode( "permission", str );
	pnodeMy->addNode( "description", in.description().c_str() );
	str.format( "%i", (Auth4cData::PRStatus::PRStatusRaw)in.status() );
	pnodeMy->addNode( "status", str );
}


//////////////////////////////////////////////////////////////////////////
//
// PermissionMap
//

Auth4cData::PermissionMap Factory::PermissionMap::fromXML( const Rfc::XMLNode* pnode )
{
	Auth4cData::PermissionMap rtn;

	if( pnode->name() != "permissions" )
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_MALFORMEDXML, "Not a permissions node." );
	}

	const Rfc::XMLNode* pnodePermission( NULL );
	pnodePermission = pnode->findChild( "permission", pnodePermission );
	while( NULL != pnodePermission )
	{
		Auth4cData::Permission permission( Permission::fromXML( pnodePermission ) );
		rtn[permission.permission()] = permission;

		pnodePermission = pnode->findChild( "permission", pnodePermission );
	}

	return rtn;
}

void Factory::PermissionMap::toXML( Rfc::XMLNode* pnode, const Auth4cData::PermissionMap& in, bool bRecursive /*= false*/ )
{
	Rfc::XMLNode* pnodeMy = pnode->addNode( "permissions", Rfc::String() );
	for( Auth4cData::PermissionMap::const_iterator iter = in.begin();
			iter != in.end();
			++iter )
	{
		Permission::toXML( pnodeMy, iter->second );
	}
}


//////////////////////////////////////////////////////////////////////////
//
// Product
//

Auth4cData::Product Factory::Product::fromXML( const Rfc::XMLNode* pnode )
{
	if( pnode->name() != "product" )
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_MALFORMEDXML, "Not a product node." );
	}

	const Rfc::XMLNode* pnodeProduct = pnode->findChild( "product" );
	if( ( NULL == pnodeProduct ) || 
		( pnodeProduct->value().isNull() ) )
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_MALFORMEDXML, "No product node in Product." );
	}

	const Rfc::XMLNode* pnodeDescription = pnode->findChild( "description" );
	if( NULL == pnodeDescription )
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_MALFORMEDXML, "No description node in Product." );
	}

	const Rfc::XMLNode* pnodeInitMapMap = pnode->findChild( "initmapmap" );
	const Rfc::XMLNode* pnodePermissionS = pnode->findChild( "permissions" );
	const Rfc::XMLNode* pnodeRoleS = pnode->findChild( "roles" );

	Auth4cData::Product rtn( atoi( pnodeProduct->value() ) );

	if( !pnodeDescription->value().isNull() )
	{
		rtn.setDescription( (const char*)pnodeDescription->value() );
	}

	if( NULL != pnodeInitMapMap )
	{
		rtn.setProductInit( InitMapMap::fromXML( pnodeInitMapMap ) );
	}

	if( NULL != pnodePermissionS )
	{
		Auth4cData::PermissionMap permissionS( PermissionMap::fromXML( pnodePermissionS ) );

		rtn.permissions() = permissionS;
	}

	if( NULL != pnodeRoleS )
	{
		Auth4cData::RoleMap roleS( RoleMap::fromXML( pnodeRoleS ) );

		rtn.roles() = roleS;
	}

	return rtn;
}

void Factory::Product::toXML( Rfc::XMLNode* pnode, const Auth4cData::Product& in, bool bRecursive /*= false*/ )
{
	Rfc::XMLNode* pnodeMy = pnode->addNode( "product", Rfc::String() );

	Rfc::String str;
	str.format( "%i", in.product() );
	pnodeMy->addNode( "product", (const char*)str );
	pnodeMy->addNode( "description", in.description().c_str() );
	
	InitMapMap::toXML( pnodeMy, in.productInit() );
	RoleMap::toXML( pnodeMy, in.roles() );
	PermissionMap::toXML( pnodeMy, in.permissions() );
}


//////////////////////////////////////////////////////////////////////////
//
// ProductMap
//

Auth4cData::ProductMap Factory::ProductMap::fromXML( const Rfc::XMLNode* pnode )
{
	Auth4cData::ProductMap rtn;

	if( pnode->name() != "products" )
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_MALFORMEDXML, "Not a products node." );
	}

	const Rfc::XMLNode* pnodeProduct( NULL );
	pnodeProduct = pnode->findChild( "product", pnodeProduct );
	while( NULL != pnodeProduct )
	{
		Auth4cData::Product product( Product::fromXML( pnodeProduct ) );
		rtn[product.product()] = product;

		pnodeProduct = pnode->findChild( "product", pnodeProduct );
	}

	return rtn;
}

void Factory::ProductMap::toXML( Rfc::XMLNode* pnode, const Auth4cData::ProductMap& in, bool bRecursive /*= false*/ )
{
	Rfc::XMLNode* pnodeMy = pnode->addNode( "products", Rfc::String() );
	for( Auth4cData::ProductMap::const_iterator iter = in.begin();
			iter != in.end();
			++iter )
	{
		Product::toXML( pnodeMy, iter->second );
	}
}


//////////////////////////////////////////////////////////////////////////
//
// ProductInitMap
//

Auth4cData::ProductInitMap Factory::ProductInitMap::fromXML( const Rfc::XMLNode* pnode )
{
	Auth4cData::ProductInitMap rtn;

	if( pnode->name() != "productinitmap" )
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_MALFORMEDXML, "Not a productinitmap node." );
	}

	const Rfc::XMLNode* pnodeInitMap( NULL );
	pnodeInitMap = pnode->findChild( "initmap", pnodeInitMap );
	while( NULL != pnodeInitMap )
	{
		const Rfc::XMLNode* pnodeProduct = pnodeInitMap->findChild( "product" );
		const Rfc::XMLNode* pnodeInitMapI = pnodeInitMap->findChild( "initmap" );

		if( ( NULL != pnodeProduct ) &&
			( !pnodeProduct->value().isNull() ) &&
			( NULL != pnodeInitMapI ) )
		{
			Auth4cData::InitMap mapInit( InitMap::fromXML( pnodeInitMapI ) );
			rtn[atoi( (const char*)pnodeProduct->value() )] = mapInit;
		}

		pnodeInitMap = pnode->findChild( "initmap", pnodeInitMap );
	}

	return rtn;
}

void Factory::ProductInitMap::toXML( Rfc::XMLNode* pnode, const Auth4cData::ProductInitMap& in, bool bRecursive /*= false*/ )
{
	Rfc::XMLNode* pnodeMy = pnode->addNode( "productinitmap", Rfc::String() );
	for( Auth4cData::ProductInitMap::const_iterator iter = in.begin();
			iter != in.end();
			++iter )
	{
		Rfc::XMLNode* pnodeInitMap = pnodeMy->addNode( "initmap", Rfc::String() );
		Rfc::String strProduct;
		strProduct.format( "%i", iter->first );
		pnodeInitMap->addNode( "product", strProduct );
		InitMap::toXML( pnodeInitMap, iter->second );
	}
}

//////////////////////////////////////////////////////////////////////////
//
// Role
//

Auth4cData::Role Factory::Role::fromXML( const Rfc::XMLNode* pnode )
{
	if( pnode->name() != "role" )
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_MALFORMEDXML, "Not a role node." );
	}

	const Rfc::XMLNode* pnodeRole = pnode->findChild( "role" );
	if( ( NULL == pnodeRole ) || 
		( pnodeRole->value().isNull() ) )
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_MALFORMEDXML, "No role node in Role." );
	}

	const Rfc::XMLNode* pnodeDescription = pnode->findChild( "description" );
	if( NULL == pnodeDescription )
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_MALFORMEDXML, "No description node in Role." );
	}

	const Rfc::XMLNode* pnodeStatus = pnode->findChild( "status" );
	if( ( NULL == pnodeStatus ) ||
		( pnodeStatus->value().isNull() ) )
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_MALFORMEDXML, "No status node in Role." );
	}

	const Rfc::XMLNode* pnodePermissionS = pnode->findChild( "permissions" );


	Auth4cData::Product* pproduct( NULL );
	Auth4cData::Role rtn( *pproduct, atoi( pnodeRole->value() ) );

	if( !pnodeDescription->value().isNull() )
	{
		rtn.setDescription( (const char*)pnodeDescription->value() );
	}

	rtn.setStatus( Auth4cData::PRStatus::fromInt( atoi( pnodeStatus->value() ) ) );

	if( NULL != pnodePermissionS )
	{
		Auth4cData::PermissionMap permissionS( PermissionMap::fromXML( pnodePermissionS ) );
//		Auth4cData::PermissionPtrMap permissionPtrS;

//		for( Auth4cData::PermissionMap::const_iterator iter = permissionS.begin();
//				iter != permissionS.end();
//				++iter )
//		{
//			*( permissionPtrS[iter->first] ) = iter->second;
//		}
		rtn.permissions() = permissionS;
	}

	return rtn;
}

void Factory::Role::toXML( Rfc::XMLNode* pnode, const Auth4cData::Role& in, bool bRecursive /*= false*/ )
{
	Rfc::XMLNode* pnodeMy = pnode->addNode( "role", Rfc::String() );

	Rfc::String str;
	str.format( "%i", in.role() );
	pnodeMy->addNode( "role", str );
	pnodeMy->addNode( "description", in.description().c_str() );
	str.format( "%i", (Auth4cData::PRStatus::PRStatusRaw)in.status() );
	pnodeMy->addNode( "status", str );

	PermissionMap::toXML( pnodeMy, in.permissions() );
}


//////////////////////////////////////////////////////////////////////////
//
// RoleMap
//

Auth4cData::RoleMap Factory::RoleMap::fromXML( const Rfc::XMLNode* pnode )
{
	Auth4cData::RoleMap rtn;

	if( pnode->name() != "roles" )
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_MALFORMEDXML, "Not a roles node." );
	}

	const Rfc::XMLNode* pnodeRole( NULL );
	pnodeRole = pnode->findChild( "role", pnodeRole );
	while( NULL != pnodeRole )
	{
		Auth4cData::Role role( Role::fromXML( pnodeRole ) );
		rtn[role.role()] = role;

		pnodeRole = pnode->findChild( "role", pnodeRole );
	}

	return rtn;
}

void Factory::RoleMap::toXML( Rfc::XMLNode* pnode, const Auth4cData::RoleMap& in, bool bRecursive /*= false*/ )
{
	Rfc::XMLNode* pnodeMy = pnode->addNode( "roles", Rfc::String() );
	for( Auth4cData::RoleMap::const_iterator iter = in.begin();
			iter != in.end();
			++iter )
	{
		Role::toXML( pnodeMy, iter->second );
	}
}


Auth4cData::InitMap Factory::InitMap::fromXML( const Rfc::XMLNode* pnode )
{
	Auth4cData::InitMap rtn;

	if( pnode->name() != "initmap" )
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_MALFORMEDXML, "Not a initmap node." );
	}

	const Rfc::XMLNode* pnodeInit( NULL );
	pnodeInit = pnode->findChild( "init", pnodeInit );
	while( NULL != pnodeInit )
	{
		const Rfc::XMLNode* pnodeParam = pnodeInit->findChild( "param" );
		const Rfc::XMLNode* pnodeValue = pnodeInit->findChild( "value" );

		if( ( NULL != pnodeParam ) &&
			( !pnodeParam->value().isNull() ) &&
			( NULL != pnodeValue ) &&
			( !pnodeValue->value().isNull() ) )
		{
			rtn[(const char*)pnodeParam->value()] = (const char*)pnodeValue->value();
		}

		pnodeInit = pnode->findChild( "init", pnodeInit );
	}

	return rtn;
}

void Factory::InitMap::toXML( Rfc::XMLNode* pnode, const Auth4cData::InitMap& in, bool bRecursive /*= false*/ )
{
	Rfc::XMLNode* pnodeMy = pnode->addNode( "initmap", Rfc::String() );
	for( Auth4cData::InitMap::const_iterator iter = in.begin();
			iter != in.end();
			++iter )
	{
		Rfc::XMLNode* pnodeInit = pnodeMy->addNode( "init", Rfc::String() );
		pnodeInit->addNode( "param", iter->first.c_str() );
		pnodeInit->addNode( "value", iter->second.c_str() );
	}
}

Auth4cData::InitMapMap Factory::InitMapMap::fromXML( const Rfc::XMLNode* pnode )
{
	Auth4cData::InitMapMap rtn;

	if( pnode->name() != "initmapmap" )
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_MALFORMEDXML, "Not a initmapmap node." );
	}

	const Rfc::XMLNode* pnodeInitMap( NULL );
	pnodeInitMap = pnode->findChild( "initmap", pnodeInitMap );
	while( NULL != pnodeInitMap )
	{
		const Rfc::XMLNode* pnodeProfile = pnodeInitMap->findChild( "profile" );
		const Rfc::XMLNode* pnodeInitMapI = pnodeInitMap->findChild( "initmap" );

		if( ( NULL != pnodeProfile ) &&
			( !pnodeProfile->value().isNull() ) &&
			( NULL != pnodeInitMapI ) )
		{
			Auth4cData::InitMap mapInit( InitMap::fromXML( pnodeInitMapI ) );
			rtn[(const char*)pnodeProfile->value()] = mapInit;
		}

		pnodeInitMap = pnode->findChild( "initmap", pnodeInitMap );
	}

	return rtn;
}

void Factory::InitMapMap::toXML( Rfc::XMLNode* pnode, const Auth4cData::InitMapMap& in, bool bRecursive /*= false*/ )
{
	Rfc::XMLNode* pnodeMy = pnode->addNode( "initmapmap", Rfc::String() );
	for( Auth4cData::InitMapMap::const_iterator iter = in.begin();
			iter != in.end();
			++iter )
	{
		Rfc::XMLNode* pnodeInitMap = pnodeMy->addNode( "initmap", Rfc::String() );
		pnodeInitMap->addNode( "profile", iter->first.c_str() );
		InitMap::toXML( pnodeInitMap, iter->second );
	}
}
