/*

  $Id: Auth4cService.cpp 737 2020-09-23 18:25:13Z john $

  Copyright (c) 2008, The 4c Company. All rights reserved.
*/

#include <Auth4cDataXML/Auth4cService.h>

#include <librfc/XMLDoc.hpp>

#include <sstream>

using namespace Auth4cDataXML;

bool Auth4cService::_bIsConnected( false );
void(*Auth4cService::_usmCallback)( Base4c::DaemonComm::Status, const std::string& )( NULL );
Base4c::DaemonComm* Auth4cService::_pcomm( NULL );

//Base4c::DaemonComm g_commParse( NULL );

Base4c::Semaphore Auth4cService::_semaphore;
Rfc::XMLDoc Auth4cService::_docResponse;
bool Auth4cService::_bInTransaction( false );
std::string Auth4cService::_strHost;

bool Auth4cService::connect( const std::string& strService )
{
	if( _bIsConnected )
    {
		disconnect();
    }

	std::string strProtocol;
	std::string strUser;
	std::string strPassword;
	int nPort;

	if( _pcomm == NULL )
	{
		_pcomm = new Base4c::DaemonComm( usmCallback );
	}

	_pcomm->parse( strUser, strPassword, _strHost, nPort, strService, "auth4cd" );
	_pcomm->connect();
	_bIsConnected = true;

	return _bIsConnected;
}


void Auth4cService::disconnect()
{
	if( _bIsConnected )
	{
		if( _pcomm != NULL )
		{
			_pcomm->disconnect();
			delete _pcomm;
		}

		_pcomm = NULL;
		_bIsConnected = false;
	}
}

const std::string& Auth4cService::host()
{
	return _strHost;
}

bool Auth4cService::isConnected()
{
	return _bIsConnected;
}

Base4c::DaemonComm& Auth4cService::comm()
{
	if( _pcomm == NULL )
    {
		throw Base4c::Exception( __FILE__, __LINE__, Base4c::B4CX_SEELEGEND, false, "Daemon Comm pointer is null" );
    }

	return *_pcomm;
}

void Auth4cService::transact( Rfc::XMLDoc& docResponse, const Rfc::XMLDoc& docRequest )
{
	if( !Auth4cService::isConnected() )
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_NOSERVERCONNECT, "Not connected to any Auth4c service provider." );
	}

	try
	{
		_bInTransaction = true;
		comm().send( (const char*)docRequest.stream() );

		int nWaits = 0;

		while( _bInTransaction && ( nWaits < 10 ) )
		{
			_semaphore.wait( 1000 );
			++nWaits;
		}
	}
	catch( const Base4c::Exception& e )
	{
		std::ostringstream os;
		os << e;
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_ALIENEXCEPTION, os.str() );
	}

	if( _bInTransaction )
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_RESPONSETIMEOUT, "Request sent with no response (timeout)." );
	}

	docResponse = _docResponse;
}

void Auth4cService::setCallback( void(*usmCallback)( Base4c::DaemonComm::Status, const std::string& ) )
{
	_usmCallback = usmCallback;
}

void Auth4cService::usmCallback( Base4c::DaemonComm::Status status, const std::string& strMessage )
{
	try
	{
		Rfc::XMLDoc doc( strMessage.c_str() );

		Rfc::XMLNode* pnodeR = doc.rootNode();
		if( nullptr == pnodeR )
		{
			throw "Oops";
		}

		Rfc::String strType = pnodeR->attribute( "type" ); //should be "response" or "unsolicited"

		if( ( strType != "usm" ) &&
			( _bInTransaction ) )
		{
			_docResponse = doc;
			_bInTransaction = false;
			_semaphore.signal();
		}
		else if( _usmCallback != NULL )
		{
			_usmCallback( status, strMessage );
		}
	}
	catch( ... )
	{
		if( _usmCallback != NULL )
        {
			_usmCallback( status, strMessage );
        }
	}
}
