/*

  $Id: Auth4cMySQL.cpp 689 2018-11-28 17:54:35Z john $

  Copyright (c) 2008, The 4c Company. All rights reserved.
*/


#include <Auth4cMySQL/Auth4cMySQL.h>

#include <Auth4cMySQL/userQuery.h>

#include <Auth4cData/Exception.h>
#include <Auth4cData/AuthenticationToken.h>
#include <Auth4cData/InitMap.h>

#include <Auth4cData/SessionString.h>

#include <librfcdal/SqlFunction.h>

#include <sstream>

RfcDal::SqlSchema g_schema;

using namespace Auth4cMySQL;

//////////////////////////////////////////////////////////////////////////
//
// Authenticator
//

Authenticator::Authenticator( const Auth4cData::InitMap& initAuthenticator,
								const Auth4cData::Domain& domain,
								const std::string& strDescription,
								int nAuthenticationLibrary )
	: Auth4cAuth4c::Authenticator( initAuthenticator,
									domain,
									strDescription,
									nAuthenticationLibrary )
{
}

Authenticator::Authenticator( const Authenticator& authenticator )
	: Auth4cAuth4c::Authenticator( authenticator )
{
}

Authenticator::~Authenticator()
{
}

const Authenticator& Authenticator::operator=( const Authenticator& authenticator )
{
	if( this != &authenticator )
	{
		Auth4cAuth4c::Authenticator::operator=( authenticator );
	}

	return *this;
}

Auth4cData::AuthenticatedUser Authenticator::authenticate( const Auth4cData::User& user, const Auth4cData::AuthenticationToken& authToken, const Auth4cData::Profile& profile )
{
	RfcDal::DalMySQLDatabase db;
	init( db );
	return authenticate( user, authToken, profile, db );
}

Auth4cData::AuthenticatedUser Authenticator::authenticate( const Auth4cData::User& userA, const Auth4cData::AuthenticationToken& authToken, const Auth4cData::Profile& profile, RfcDal::DalDatabase& db )
{
	Auth4cMySQL::userQuery qry( g_schema );
	qry.constrainUser( RfcDal::SqlText( userA.userId() ) );

	RfcDal::SqlText txtAuthenticationToken( authToken.str() );
	RfcDal::SqlFunction fnPassword( "PASSWORD" );
	fnPassword.addColumn( txtAuthenticationToken );

	qry.constrainAuthenticationToken( fnPassword );

	RfcDal::DalDatabase::Result result;
	RfcDal::DalDatabase::Row row;
	if( 1 != qry.select( result, db ) )
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_AUTHENTICATIONFAILED, "Query did not return any results." );
	}
	else
	{
		Auth4cData::User userLocal( userA );
		user( userLocal );

		std::ostringstream os;
		os << &userA;
		Auth4cData::SessionString sesStr( os.str() );
		Auth4cData::AuthenticatedUser userAuth( userLocal, sesStr, profile );
		return userAuth;
	}

	throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_AUTHENTICATIONFAILED, "Query did not return any results." );
}

void Authenticator::init( RfcDal::DalDatabase& db )
{
	try
	{
		std::string strXConnect;

		Auth4cData::InitMap::const_iterator iterXConnect = authenticatorInit().find( "XConnect" );
		if( authenticatorInit().end() != iterXConnect )
		{
			strXConnect = iterXConnect->second;
		}

		RfcDal::DalDatabase::ConnectionParameterMap map;
		db.connectStringToMap( map, strXConnect );
		
		RfcDal::DalDatabase::ConnectionParameterMap::const_iterator iter = map.find( "database" );
		if( iter != map.end() )
		{
			try
			{
				db.connectX( map );
			}
			catch( const RfcDal::DalDatabaseException& e )
			{
				throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_AUTHENTICATORINITFAILED, e.nativeExplanation() );
			}
			
			RfcDal::SqlSchema schema( iter->second );
			g_schema = schema;
		}
		else
		{
			throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_AUTHENTICATORINITFAILED, "No schema/database." );
		}
	}
	catch( const Auth4cData::Exception& e )
	{
		throw;
	}
	catch( const RfcDal::DalDatabaseException& e )
	{
		throw Auth4cData::Exception( __FILE__, __LINE__, Auth4cData::nNAX_AUTHENTICATORINITFAILED, e.nativeExplanation() );
	}
}


//////////////////////////////////////////////////////////////////////////
//
// AuthenticationLibrary
//

AuthenticationLibrary::AuthenticationLibrary()
{
}

AuthenticationLibrary::AuthenticationLibrary( const Auth4cData::AuthenticationLibrary* ptemplate )
	: Auth4cData::AuthenticationLibrary( ptemplate )
{
}

AuthenticationLibrary::AuthenticationLibrary( const AuthenticationLibrary& library )
	: Auth4cData::AuthenticationLibrary( library )
{
}

AuthenticationLibrary::~AuthenticationLibrary()
{
}

const AuthenticationLibrary& AuthenticationLibrary::operator=( const AuthenticationLibrary& library )
{
	if( this != &library )
	{
		Auth4cData::AuthenticationLibrary::operator=( library );
	}

	return *this;
}

Auth4cData::Authenticator* AuthenticationLibrary::authenticator( const Auth4cData::Domain& domain,
																	const std::string& strDescription,
																	const Auth4cData::InitMap& initMap )
{
	Authenticator* pauthenticator = new Authenticator( initMap,
														domain,
														strDescription,
														authenticationLibrary() );

	return pauthenticator;
}

extern "C" Auth4cMySQL_Export Auth4cData::AuthenticationLibrary* library( const Auth4cData::AuthenticationLibrary* ptemplate )
{
	return new AuthenticationLibrary( ptemplate );
}
