/*

  $Id: AboutUserWidget.cpp 688 2018-11-28 17:10:21Z john $

  Copyright (c) 2008, The 4c Company. All rights reserved.
*/

#include <Auth4cUIComponents/AboutUserWidget.h>

#include <Auth4cDataXML/Auth4cFactory.h>
#include <Auth4cUIComponents/TreeWidgetItem.h>

#include <QDateTime>
#include <QListWidgetItem>
#include <QClipboard>

#include <librfc/XMLDoc.hpp>

#include <sstream>

using namespace Auth4cUIComponents;

AboutUserWidget::AboutUserWidget( QWidget* pparent )
	: QWidget( pparent )

{
    setupUi(this); //Setup the Qt Designer generated widgets

	_pteamS->header()->hide();
	_pproductS->setColumnCount( 2 );
	_pproductS->header()->show();
	QStringList strLabelS;
	strLabelS << tr( "Role/Permission" ) << tr( "Granted" );
	_pproductS->setHeaderLabels( strLabelS );
}

void AboutUserWidget::setUser( const Auth4cData::User& user )
{
	_pbtnCopy->setEnabled( false );

	std::ostringstream os;
	os << user.domain().domain() << "/" << user.userId();

	_pedtUser->setText( QString::fromStdString( os.str() ) );

	_pedtName->setText( QString::fromStdString( user.name() ) );

	_pedtContact->setText( QString::fromStdString( user.contact() ) );

	for( Auth4cData::TeamMap::const_iterator iterTeam = user.teams().begin();
			iterTeam != user.teams().end();
			++iterTeam )
	{
		QTreeWidgetItem* pitemTeam = new QTreeWidgetItem( _pteamS );
		pitemTeam->setText( 0, QString::fromStdString( iterTeam->second.name() ) );
	}

	for( Auth4cData::ProductMap::const_iterator iterProduct = user.products().begin();
			iterProduct != user.products().end();
			++iterProduct )
	{
		QTreeWidgetItem* pitemProduct = new QTreeWidgetItem( _pproductS );
		pitemProduct->setText( 0, QString::fromStdString( iterProduct->second.description() ) );

		QTreeWidgetItem* pitemRoleS = new QTreeWidgetItem( pitemProduct );
		pitemRoleS->setText( 0, tr( "Roles" ) );

		for( Auth4cData::RoleMap::const_iterator iterRole = iterProduct->second.roles().begin();
				iterRole != iterProduct->second.roles().end();
				++iterRole )
		{
			QTreeWidgetItem* pitemRole = new QTreeWidgetItem( pitemRoleS );
			pitemRole->setText( 0, QString::fromStdString( iterRole->second.description() ) );
			pitemRole->setText( 1, tr( iterRole->second.status() ? "Y" : "N" ) );
		}

		QTreeWidgetItem* pitemPermissionS = new QTreeWidgetItem( pitemProduct );
		pitemPermissionS->setText( 0, tr( "Permissions" ) );

		for( Auth4cData::PermissionMap::const_iterator iterPermission = iterProduct->second.permissions().begin();
				iterPermission != iterProduct->second.permissions().end();
				++iterPermission )
		{
			QTreeWidgetItem* pitemPermission = new QTreeWidgetItem( pitemPermissionS );
			pitemPermission->setText( 0, QString::fromStdString( iterPermission->second.description() ) );
			pitemPermission->setText( 1, tr( iterPermission->second.status() ? "Y" : "N" ) );
		}
	}
}

void AboutUserWidget::setUser( const Auth4cData::AuthenticatedUser& user )
{
	_user = user;

	setUser( (Auth4cData::User&)user );

	_pbtnCopy->setEnabled( true );

	_pedtProfile->setText( QString::fromStdString( user.profile().profile() ) );
}

AboutUserWidget::~AboutUserWidget()
{
}

void AboutUserWidget::onCopy()
{
	Rfc::XMLDoc docR( "auth4cservice", "1.0" );
	Rfc::XMLNode* pnodeR = docR.rootNode();

	Auth4cDataXML::Factory::AuthenticatedUser::toXML( pnodeR, _user, true );

	QApplication::clipboard()->setText( (const char*)docR.stream() );
}
