/*

  $Id: LoginDialog.cpp 404 2015-12-15 13:47:27Z john $

  Copyright (c) 2008, The 4c Company. All rights reserved.
*/

#include <Auth4cUIComponents/LoginDialog.h>

#include <Auth4cUIComponents/ExceptionDialog.h>
#include <Auth4cUIComponents/AboutAuth4cDialog.h>
#include <Auth4cUIComponents/Auth4cUIComponents.h>

#include <Auth4cData/PasswordAuthenticationToken.h>
#include <Auth4cData/Factory.h>
#include <Auth4cDataXML/Auth4cFactory.h>


#include <QMessageBox>

using namespace Auth4cUIComponents;

LoginDialog::LoginDialog( QWidget* pparent, Auth4cClientConfig::Settings& settings )
	: QDialog( pparent ),
	_settings( settings )
{
    setupUi(this); //Setup the Qt Designer generated widgets

	setWindowIcon( Auth4cUIComponents::icon4c() );

	#if defined (_DEBUG) || defined(DEBUG)
		setStyleSheet( "QDialog { Background-color: red; }" );
	#endif

	for( Auth4cClientConfig::Settings::ProfileMap::const_iterator iter = _settings.profiles().begin();
			iter != _settings.profiles().end();
			++iter )
	{
		_pcbxProfile->addItem( QString::fromStdString( iter->second.profile() ), QString::fromStdString( iter->second.profile() ) );
	}

	int nIdxProfile = _pcbxProfile->findData( QString::fromStdString( _settings.defaultProfile() ) );
	if( -1 != nIdxProfile )
	{
		_pcbxProfile->setCurrentIndex( nIdxProfile );
	}

	for( Auth4cClientConfig::Settings::DomainMap::const_iterator iter = _settings.domains().begin();
			iter != _settings.domains().end();
			++iter )
	{
		_pcbxDomain->addItem( QString::fromStdString( iter->second.domain() ), QString::fromStdString( iter->second.domain() ) );
	}

	int nIdxDomain = _pcbxDomain->findData( QString::fromStdString( _settings.defaultDomain() ) );
	if( -1 != nIdxDomain )
	{
		_pcbxDomain->setCurrentIndex( nIdxDomain );
	}

	if( _settings.userId().length() )
	{
		_pedtUserId->setText( QString::fromStdString( _settings.userId() ) );
		_pedtPassword->setFocus();
	}
	else
	{
		_pedtUserId->setFocus();
	}
}

LoginDialog::~LoginDialog()
{
}

const Auth4cData::AuthenticatedUser& LoginDialog::user() const
{
	return _user;
}

void LoginDialog::accept()
{
	bool bSucceeded( false );

	QString strProfile;
	QString strDomain;
	QString strUserId;
	QString strPassword;

	strProfile = _pcbxProfile->currentText();
	strDomain = _pcbxDomain->currentText();
	strUserId = _pedtUserId->text();
	strPassword = _pedtPassword->text();

	try
	{
		setCursor( Qt::WaitCursor );

		Auth4cData::Domain myDomain( Auth4cData::Customer::allCustomers, strDomain.toStdString() );
		Auth4cData::User myUser( myDomain, strUserId.toStdString() );
		Auth4cData::Profile profile( strProfile.toStdString() );

		_user = Auth4cDataXML::Factory::authenticate( myUser, Auth4cData::PasswordAuthenticationToken( strPassword.toStdString() ), profile );

		_settings.setDefaultProfile( strProfile.toStdString() );
		_settings.setDefaultDomain( strDomain.toStdString() );
		_settings.setUserId( strUserId.toStdString() );

		bSucceeded = true;

		unsetCursor();
	}
	catch( const Auth4cData::Exception& e )
	{
		unsetCursor();

		ExceptionDialog dlgE( this, e );
		dlgE.exec();
		_pedtPassword->setText( tr( "" ) );
	}
	catch( ... )
	{
		unsetCursor();

		// We may want to do something with this later
	}

	if( bSucceeded )
	{
		QDialog::accept();
	}
}

void LoginDialog::reject()
{
	QDialog::reject();
}

void LoginDialog::onHelp()
{
	AboutAuth4cDialog dlg( this );
	dlg.exec();
}
