/*

  $Id: ClientSocket.h 688 2018-11-28 17:10:21Z john $

  Copyright (c) 2008, The 4c Company. All rights reserved.
*/

#ifndef __auth4cd_ClientSocket_h__
#define __auth4cd_ClientSocket_h__

#include <Base4c/Mutex.h>
#include <string>

#include <Auth4cData/AuthenticatedUser.h>

namespace Rfc
{
	class XMLNode;
	class String;
}

namespace Auth4cD
{
	class ClientSocket
	{
		public:
			typedef enum { clientListen, clientConnect } Type;

		private:
			ClientSocket* _pnext;
			ClientSocket* _pprev;
			Type _type;
			int _s;
			std::string _strHost;
			bool _bContinue;
			std::string _strMessages;
			Auth4cData::AuthenticatedUser _user;

		protected:
			ClientSocket( Type type, int s, const std::string& strHost );
			virtual ~ClientSocket();

		public:
			const Auth4cData::AuthenticatedUser& user() const;
			Type type() const;
			int s() const;
			void postMessage( const std::string& strMessage );

		protected:
#if defined (LINUX)
			static void* clientProc( void* pvContext );
#elif defined (_WINDOWS)
			static void clientProc( void* pvContext );
#endif
			int readClient();
			void processClientRequests();
			void processClientRequest( const Rfc::String& strMessage );

			void about( Rfc::XMLNode* pnodeResponse, const Rfc::XMLNode* pnodeRequest );
			void reloadFactory( Rfc::XMLNode* pnodeResponse, const Rfc::XMLNode* pnodeRequest );
			void initialize( Rfc::XMLNode* pnodeResponse, const Rfc::XMLNode* pnodeRequest );
			void authenticate( Rfc::XMLNode* pnodeResponse, const Rfc::XMLNode* pnodeRequest );
			void validate( Rfc::XMLNode* pnodeResponse, const Rfc::XMLNode* pnodeRequest );
			void authenticatedUser( Rfc::XMLNode* pnodeResponse, const Rfc::XMLNode* pnodeRequest );
			void passwd( Rfc::XMLNode* pnodeResponse, const Rfc::XMLNode* pnodeRequest );
			void persistSession( Rfc::XMLNode* pnodeResponse, const Rfc::XMLNode* pnodeRequest );
			void retrieveSession( Rfc::XMLNode* pnodeResponse, const Rfc::XMLNode* pnodeRequest );
			void killSession( Rfc::XMLNode* pnodeResponse, const Rfc::XMLNode* pnodeRequest );
			void user( Rfc::XMLNode* pnodeResponse, const Rfc::XMLNode* pnodeRequest );
			void team( Rfc::XMLNode* pnodeResponse, const Rfc::XMLNode* pnodeRequest );
			void teams( Rfc::XMLNode* pnodeResponse, const Rfc::XMLNode* pnodeRequest );
			void domains( Rfc::XMLNode* pnodeResponse, const Rfc::XMLNode* pnodeRequest );
			void profiles( Rfc::XMLNode* pnodeResponse, const Rfc::XMLNode* pnodeRequest );
			void product( Rfc::XMLNode* pnodeResponse, const Rfc::XMLNode* pnodeRequest );
			void products( Rfc::XMLNode* pnodeResponse, const Rfc::XMLNode* pnodeRequest );
			void customers( Rfc::XMLNode* pnodeResponse, const Rfc::XMLNode* pnodeRequest );
			void who( Rfc::XMLNode* pnodeResponse, const Rfc::XMLNode* pnodeRequest );

			void unknownRequest( Rfc::XMLNode* pnodeResponse, const Rfc::XMLNode* pnodeRequest, const Rfc::String& strRequest );

			friend class ClientSocketList;
	};

	class ClientSocketList
	{
		private:
				ClientSocket* _pfirst;
				ClientSocket* _plast;
				Base4c::Mutex _mx;
				int _nCount;

		public:
				ClientSocketList();
				virtual ~ClientSocketList();

				void add( ClientSocket::Type type, int s, const std::string& strHost );
				void remove( int s );
				ClientSocket* find( int s );
				ClientSocket* next( ClientSocket* pcurrent );
				int count() const { return _nCount; }
				void broadcastMessage( const std::string& strMessage );
				void lock();
				void unlock();
	};
};

#endif /* !__auth4cd_ClientSocket_h__ */
