/*

  $Id: main.cpp 699 2018-12-11 17:51:05Z john $

  Copyright (c) 2008, The 4c Company. All rights reserved.
*/

#include <auth4cd/Auth4cDApp.h>

#if defined (LINUX)
#	include <signal.h>
#endif /* LINUX */

//#include <iostream>

#include <librfcdal/SqlLoggerSyslog.h>

#include <Base4c/Exception.h>
#include <Base4c/EventLogger.h>

#include <librfcdal/SqlException.h>
#include <librfcdal/DalDatabaseException.h>

#include <Auth4cDataDB/Auth4cFactory.h>
#include <Auth4cDataXML/Util.h>

#include <crypt.h>

int main( int argc, char* argv[] )
{
	int nRC = 0;

	std::string strAppName( "auth4cd" );
	Base4c::EventLogger theLogger( strAppName );
	RfcDal::SqlLoggerSyslog sqlLogger( strAppName );

#if defined (LINUX)
	signal( SIGHUP, shutdown );
	signal( SIGINT, shutdown );
	signal( SIGTERM, shutdown );
	signal( SIGPIPE, ignore );
#endif /* LINUX */

	try
	{
		nRC = theApp.initInstance( argc, argv );
		if( nRC >= 0 )
			nRC = theApp.run( nRC );
		nRC = theApp.exitInstance( nRC );
	}
	catch( const Base4c::Exception& e )
	{
		Base4c::EventLogger::theLogger()->log( e, Base4c::EventLogger::daemon );
	}
	catch( const RfcDal::SqlException& e )
	{
		nRC = -1;

//		Base4c::EventLogger::theLogger()->log( e, Base4c::EventLogger::daemon );
//		Already logged when exception instantiated
	}
	catch( const RfcDal::DalDatabaseException& e )
	{
		nRC = -1;

//		Base4c::EventLogger::theLogger()->log( e, Base4c::EventLogger::daemon );
//		Already logged when exception instantiated
	}
	catch( const char* e )
	{
		nRC = -1;

		Base4c::Exception e2( __FILE__, __LINE__, Base4c::B4CX_SEELEGEND, false, e );
		Base4c::EventLogger::theLogger()->log( e2, Base4c::EventLogger::daemon );
	}
	catch( ... )
	{
		nRC = -1;

		Base4c::EventLogger::theLogger()->logUnknown( Base4c::EventLogger::daemon );
	}

	return nRC;
}

